/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2010 All rights reserved.                  */
/*  File Name    : sample1.c                                                 */
/*  Abstract     : I/O toolkit sample program                                */
/*  Function List                                                            */
/*      main()                       : Sample program main                   */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2011/01/26                                                    */
/*  Version    1.08                                                          */
/*  Notice                                                                   */
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>

/** Include AMTK.h */
#ifdef __cplusplus
extern "C" {
#endif
#include "AMTK.h"
#ifdef __cplusplus
}
#endif

#include "sample_common.h"

/*---------------------------------------------------------------------------*
 * Common definition                                                         *
 *---------------------------------------------------------------------------*/
#define FN_SAMPLE_PRODUCT_L1B   ("../data/sample1_IOToolKit.h5")

#define META_PRODUCTNAME_NAME   ("ProductName")
#define META_PRODUCTNAME_VAL    ("AMSR2-L1B")
#define META_OVERLAP_NAME       ("OverlapScans")
#define META_OVERLAP_VALUE      ("20")

#define SCENE_SCAN_NUM          (2000)
#define OVERLAP_SCAN_NUM        (20)


/** Structure of dataset model. */
typedef struct
{
	/* Navigation Data[IN]. */
	float *p_navi_in;
	
	/* Navigation Data[OUT]. */
	float *p_navi_out;

} DATASET_MODEL;

/** File R/W flag. */
enum FLAG_FILERW
{
	FILE_READ,
	FILE_WRITE
};


/** Termination.
 * @param file_id   [IN]  Product file id
 * @param is_write  [IN]  Write mode flag
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, int is_write, DATASET_MODEL *p_dataset);


/** AMSR2 I/O toolkit sample program.
 * sample1: proudct I/O.
 * 
 * @param  argc [IN] Argument count
 * @param  argv [IN] Argument value
 * @return EXIT_SUCCESS
 * @return EXIT_FAILURE
 */
int main(int argc, char *argv[])
{
	hid_t file_id = 0;
	int dimsize[3] = {0};
	int i = 0;
	int j = 0;
	int ret = 0;
	char productname[12 + 1] = "";
	char *p_value = NULL;
	
	/* Number of Scans */
	const int scan_size = SCENE_SCAN_NUM + OVERLAP_SCAN_NUM * 2;
	const int scan_start = 1 - OVERLAP_SCAN_NUM;
	const int scan_end = SCENE_SCAN_NUM + OVERLAP_SCAN_NUM;
	const int navi_num = 6;
	
	/* Dataset */
	DATASET_MODEL dataset = {0};
	
	/*------------------------------------------------------------------------*
	 * Output product                                                         *
	 *------------------------------------------------------------------------*/
	dataset.p_navi_out = (float *) malloc(sizeof(float) * scan_size * navi_num);
	if (NULL == dataset.p_navi_out)
	{
		E_MSG("malloc() error.\n");
		exit(EXIT_FAILURE);
	}

	/* file open */
	file_id = AMTK_openH5_Write(FN_SAMPLE_PRODUCT_L1B, AM2_CREATE_MODE);
	if (0 > file_id)
	{
		E_MSG("AMTK_openH5_Write() error.[%d]\n", file_id);
		terminate(file_id, FILE_WRITE, &dataset);
		exit(EXIT_FAILURE);
	}

	/* set metadata */
	ret = AMTK_setMetaDataName(file_id, META_PRODUCTNAME_NAME,
		META_PRODUCTNAME_VAL);
	if (0 > ret)
	{
		E_MSG("AMTK_setMetaDataName() error.[%d]\n", ret);
		terminate(file_id, FILE_WRITE, &dataset);
		exit(EXIT_FAILURE);
	}
	
	ret = AMTK_setMetaDataName(file_id, META_OVERLAP_NAME, META_OVERLAP_VALUE);
	if (0 > ret)
	{
		E_MSG("AMTK_setMetaDataName() error.[%d]\n", ret);
		terminate(file_id, FILE_WRITE, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* set dataset */
	dimsize[0] = scan_size;
	dimsize[1] = navi_num;
	dimsize[2] = 0;
	ret = AMTK_setDimSize(file_id, AM2_NAVI, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_setMetaDataName() error.[%d]\n", ret);
		terminate(file_id, FILE_WRITE, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* set Navigation Data */
	for (i = 0; i < scan_size; i++)
	{
		for (j = 0; j < navi_num; j++)
		{
			
			*(dataset.p_navi_out + (i * navi_num) + j)
				= (i - OVERLAP_SCAN_NUM) * 0.1;
		}
	}
	
	ret = AMTK_set_SwathFloat(file_id, dataset.p_navi_out, scan_start, 
		scan_end, AM2_NAVI);
	if (0 > ret)
	{
		E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
		terminate(file_id, FILE_WRITE, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* file close */
	ret = AMTK_closeH5_Write(file_id);
	if (0 > ret)
	{
		E_MSG("AMTK_closeH5_Write() error.[%d]\n", ret);
		terminate(file_id, FILE_WRITE, &dataset);
		exit(EXIT_FAILURE);
	}
	
	file_id = -1;
	
	/*------------------------------------------------------------------------*
	 * Input product                                                          *
	 *------------------------------------------------------------------------*/
	/* file open */
	file_id = AMTK_openH5(FN_SAMPLE_PRODUCT_L1B);
	if (0 > file_id)
	{
		E_MSG("AMTK_openH5() error.[%d]\n", file_id);
		terminate(file_id, FILE_READ, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* get dimension size */
	ret = AMTK_getDimSize(file_id, AM2_NAVI, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		terminate(file_id, FILE_READ, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* memory allocation */
	dataset.p_navi_in = (float *) malloc(sizeof(float) *
		get_memory_size(dimsize));
	if (NULL == dataset.p_navi_in)
	{
		E_MSG("malloc() error.\n");
		terminate(file_id, FILE_READ, &dataset);
		exit(EXIT_FAILURE);
	}
	
	
	/* get metadata */
	p_value = productname;
	ret = AMTK_getMetaDataName(file_id, META_PRODUCTNAME_NAME, &p_value);
	if (0 > ret)
	{
		E_MSG("AMTK_getMetaDataName() error.[%d]\n", ret);
		terminate(file_id, FILE_READ, &dataset);
		exit(EXIT_FAILURE);
	}

	/* get dataset */
	ret = AMTK_get_SwathFloat(file_id, &dataset.p_navi_in, scan_start, scan_end,
		AM2_NAVI);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
		terminate(file_id, FILE_READ, &dataset);
		exit(EXIT_FAILURE);
	}
	
	printf("ProductName = %s Navigation Data[0] = %f \n", p_value,
		*(dataset.p_navi_in));

	/* termination */
	terminate(file_id, FILE_READ, &dataset);
	
	exit(EXIT_SUCCESS);
}



/** Termination.
 * @param file_id   [IN]  Product file id
 * @param is_write  [IN]  Write mode flag
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, int is_write, DATASET_MODEL *p_dataset)
{
	int ret = 0;
	
	if (0 <= file_id)
	{
		/* file close */
		if (is_write)
		{
			/* Read/Write mode */
			ret = AMTK_closeH5_Write(file_id);
			if (0 > ret)
			{
				E_MSG("AMTK_closeH5_Write() error.[%d]\n", ret);
			}
		}
		else
		{
			/* Read mode */
			ret = AMTK_closeH5(file_id);
			if (0 > ret)
			{
				E_MSG("AMTK_closeH5() error.[%d]\n", ret);
			}
		}
	}
	
	free(p_dataset->p_navi_in);
	p_dataset->p_navi_in = NULL;
	
	free(p_dataset->p_navi_out);
	p_dataset->p_navi_out = NULL;
	
	return;
}


