/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2010 All rights reserved.                  */
/*  File Name    : sample10_read_L3Dproduct.c                                */
/*  Abstract     : I/O toolkit sample program                                */
/*  Function List                                                            */
/*      main()                       : Sample program main                   */
/*      initialize()                 : Initialization                        */
/*      terminate()                  : Termination                           */
/*      get_value()                  : Get dataset value                     */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2011/02/01                                                    */
/*  Version    1.08                                                          */
/*  Notice                                                                   */
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/** Include AMTK.h */
#ifdef __cplusplus
extern "C" {
#endif
#include "AMTK.h"
#ifdef __cplusplus
}
#endif

#include "sample_common.h"

/*---------------------------------------------------------------------------*
 * Common definition                                                         *
 *---------------------------------------------------------------------------*/
 
/* PRINT_SCAN_NUM defines print data number */
#define PRINT_SCAN_NUM        (2)
/** Number of Geophysical Data. */
#define MAX_GEO_NUM           (3)

/** Sample L1R product file name. */
#define FN_SAMPLE_PRODUCT_L3D ("../data/sample5_IOToolKit_l3d.h5")
/** Metadata definition. */
#define FN_SAMPLE_META_L3D    ("../data/metadata_sample_L3_day.txt")


/** Structure of dataset model. */
typedef struct
{
	/* Geophysical Data (Layer: 1...3) */
	float *p_geo_data[MAX_GEO_NUM];
	/* Number of Geophysical Data. */
	int geo_data_num;
	
	/* Time Infomation */
	int *p_time_info;
	
	/* Latitude of Observation Point
	 * Longitude of Observation Point */
	AM2_COMMON_LATLON *p_latlon;
	
	/* Number of pixels in X direction(longitude) */
	int x_pixsize;
	/* Number of pixels in Y direction(latitude) */
	int y_pixsize;
	
} DATASET_MODEL;


/** Initialization.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(hid_t file_id, DATASET_MODEL *p_dataset);

/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset);

/** Get dataset value.
 * @param p_dataset [IN] Dataset
 */
static void get_value(const DATASET_MODEL *p_dataset);


/*---------------------------------------------------------------------------*
 * Main program                                                              *
 *---------------------------------------------------------------------------*/
 
/** AMSR2 I/O toolkit sample program.
 * sample10: intput L3(Day) proudct.
 * 
 * GeophysicalName: Cloud Liquid Water
 * Projection     : EQR
 * Resolution     : 0.25deg
 * 
 * @param  argc [IN] Argument count
 * @param  argv [IN] Argument value
 * @return EXIT_SUCCESS
 * @return EXIT_FAILURE
 */
int main(int argc, char *argv[])
{
	hid_t file_id = 0;
	
	int i = 0;
	int meta_num = 0;
	int ret = 0;
	
	/* sample meta data read */
	char meta_name[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
	char meta_value[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
    char *p_value = NULL; 

	/* Access label: Geophysical Data */
	const int geo_label[] = {AM2_GRID_GEO1, AM2_GRID_GEO2, AM2_GRID_GEO3};
	
	/* Dataset. */
	DATASET_MODEL dataset = {{0}};
	
	
	/* HDF product file open */
	file_id = AMTK_openH5(FN_SAMPLE_PRODUCT_L3D);
	if (0 > file_id)
	{
		if (-501 == file_id)
		{
			printf("Product file not found.[%s]\n", FN_SAMPLE_PRODUCT_L3D);
		}
		else
		{
			E_MSG("AMTK_openH5() error.[%d]\n", ret);
		}
		exit(EXIT_FAILURE);
	}
	
	/*------------------------------------------------------------------------*
	 * memory allocation                                                      *
	 *------------------------------------------------------------------------*/
	ret = initialize(file_id, &dataset);
	if (ret != RET_NORMAL)
	{
		E_MSG("initialize() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/*------------------------------------------------------------------------*
	 * input data                                                             *
	 *------------------------------------------------------------------------*/
	/* Read Metadata from Metadata file */
	if (RET_NORMAL != get_metadata_sample(FN_SAMPLE_META_L3D, meta_name,
		meta_value, &meta_num))
	{
		E_MSG("get_metadata_sample() error.\n");
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* meta data read */
	for (i = 0; i < meta_num; i++)
	{
		p_value = &meta_value[i][0];
		ret = AMTK_getMetaDataName(file_id, &meta_name[i][0], &p_value);
		if (0 > ret)
		{
			E_MSG("AMTK_getMetaDataName() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		printf("No=%d\tName=%s\tValue=%s\n", i, &meta_name[i][0], p_value);
	}
	
	/* Data set read */
	
	/* Geophysical Data (Layer: 1...3) */
	for (i = 0; i < dataset.geo_data_num; i++)
	{
		ret = AMTK_get_GridFloat(file_id, &(dataset.p_geo_data[i]),
			geo_label[i]);
		if (0 > ret)
		{
			E_MSG("AMTK_get_GridFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Time Infomation */
	ret = AMTK_get_GridInt(file_id, &dataset.p_time_info, AM2_GRID_TIME);
	if (0 > ret)
	{
		E_MSG("AMTK_get_GridInt() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Latitude of Observation Point
	 * Longitude of Observation Point
	 * # scan_start, scan_end ... unused(AM2_SCANNUM_NOCHK)
	 */
	ret = AMTK_getLatLon(file_id, &dataset.p_latlon, AM2_SCANNUM_NOCHK,
		AM2_SCANNUM_NOCHK, AM2_GRID_LATLON);
	if (0 > ret)
	{
		E_MSG("AMTK_getLatLon() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* get dataset value */
	get_value(&dataset);
	
	/* termination */
	terminate(file_id, &dataset);
	
	printf("L3(Day) product read end\n");
	
	exit(EXIT_SUCCESS);
}


/** Initialization.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(hid_t file_id, DATASET_MODEL *p_dataset)
{
	int ret = 0;
	int dimsize[3] = {0};
	int i = 0;
	
	/* Access label: Geophysical Data */
	const int geo_label[] = {AM2_GRID_GEO1, AM2_GRID_GEO2, AM2_GRID_GEO3};
	
	/* memory allocation */
	
	/* Geophysical Data (1...3) */
	
	/* Layer: ALL -> AM2_GRID_GEOA
	 * (type size * X pixel * Y pixel ) */
	ret = AMTK_getDimSize(file_id, AM2_GRID_GEOA, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	/*
	 * get size
	 * # dimsize[0] : Number of pixels in X direction(longitude)
	 * # dimsize[1] : Number of pixels in Y direction(latitude)
	 * # dimsize[2] : Number of Geophysical Data
	 */
	p_dataset->x_pixsize = dimsize[0];
	p_dataset->y_pixsize = dimsize[1];
	p_dataset->geo_data_num = dimsize[2];
	if ((0 > p_dataset->geo_data_num) || (MAX_GEO_NUM < p_dataset->geo_data_num))
	{
		/* out of bounds(MAX:3) */
		E_MSG("Illegal error: Number of Geophysical Data:%d\n",
			p_dataset->geo_data_num);
		return RET_ERROR;
	}
	
	for (i = 0; i < p_dataset->geo_data_num; i++)
	{
		/*  Layer: 1...3 -> AM2_GRID_GEO1 ... AM2_GRID_GEO3
		 * (type size * X pixel * Y pixel ) */
		ret = AMTK_getDimSize(file_id, geo_label[i], dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
			return RET_ERROR;
		}
		p_dataset->p_geo_data[i] = (float *) malloc(sizeof(float)
			* get_memory_size(dimsize));
		if (NULL == p_dataset->p_geo_data[i])
		{
			E_MSG("malloc() error.\n");
			return RET_ERROR;
		}
	}
	
	/* Time Infomation
	 * (type size * X pixel * Y pixel ) */
	ret = AMTK_getDimSize(file_id, AM2_GRID_TIME, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_time_info = (int *) malloc(sizeof(int)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_time_info)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Latitude of Observation Point
	 * Longitude of Observation Point
	 * (type size * filesize) */
	ret = AMTK_getDimSize(file_id, AM2_GRID_LATLON, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	
	printf("AM2_GRID_LATLON dimsize = (%d, %d, %d)\n", dimsize[0], dimsize[1],
		dimsize[2]);
	
	p_dataset->p_latlon = (AM2_COMMON_LATLON *) malloc(
		sizeof(AM2_COMMON_LATLON) * dimsize[0]);
	if (NULL == p_dataset->p_latlon)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	return RET_NORMAL;
}


/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset)
{
	int i = 0;
	int ret = 0;
	
	if (0 <= file_id)
	{
		/* HDF product file close */
		ret = AMTK_closeH5(file_id);
		if (0 > ret)
		{
			E_MSG("AMTK_closeH5() error.[%d]\n", ret);
		}
	}
	
	/* free memory */
	for (i = 0; i < p_dataset->geo_data_num; i++)
	{
		free(p_dataset->p_geo_data[i]);
		p_dataset->p_geo_data[i] = NULL;
	}
	
	free(p_dataset->p_time_info);
	p_dataset->p_time_info = NULL;
	
	free(p_dataset->p_latlon);
	p_dataset->p_latlon = NULL;
	
	return;
}


/** Get dataset value.
 * @param p_dataset [IN] Dataset
 */
static void get_value(const DATASET_MODEL *p_dataset)
{
	int i = 0;
	int x = 0;
	int y = 0;
	
	for (x = 0; x < PRINT_SCAN_NUM; x++)
	{
		/* Geophysical Data */
		for (i = 0; i < p_dataset->geo_data_num; i++)
		{
			/* first pixel */
			y = 0;
			printf("X = %d Y = %d\n", x, y);
			printf("Geophysical Data (Layer:%d) = %8.4f\n", i + 1,
				*(p_dataset->p_geo_data[i] + (x * p_dataset->y_pixsize) + y));
			
			/* last pixel */
			y = p_dataset->y_pixsize - 1;
			printf("Scan = %d Pixel = %d\n", x, y);
			printf("Geophysical Data (Layer:%d) = %8.4f\n", i + 1,
				*(p_dataset->p_geo_data[i] + (x * p_dataset->y_pixsize) + y));
		}
		
		/* Time Infomation */
		printf("Time Infomation\n" );
		/* first pixel */
		y = 0;
		printf("X  = %d Y = %d\n", x, y);
		printf("Time Infomation = %d\n",
			*(p_dataset->p_time_info + (x * p_dataset->y_pixsize) + y));
		
		/* last pixel */
		y = p_dataset->y_pixsize - 1;
		printf("X  = %d Y = %d\n", x, y);
		printf("Time Infomation = %d\n",
			*(p_dataset->p_time_info + (x * p_dataset->y_pixsize) + y));

		/* Latitude/Longitude of Observation Point */
		printf("Latitude/Longitude of Observation Point\n");
		/*
		 * # L3LATLONFILE: data[Y pixel][X pixel]
		 * #  Example) data[720][1440]
		 * #  -> X pixel and Y pixel is reversed. 
		 */
		/* first pixel (x -> y, y -> x) */
		y = 0;
		printf("X  = %d Y = %d\n", x, y);
		printf("Latitude  = %8.2f\n",
			(p_dataset->p_latlon + (y * p_dataset->x_pixsize) + x)->lat);
		printf("Longitude = %8.2f\n",
			(p_dataset->p_latlon + (y * p_dataset->x_pixsize) + x)->lon);
		
		/* last pixel (x -> y, y -> x) */
		y = p_dataset->y_pixsize - 1;
		printf("X  = %d Y = %d\n", x, y);
		printf("Latitude  = %8.2f\n",
			(p_dataset->p_latlon + (y * p_dataset->x_pixsize) + x)->lat);
		printf("Longitude = %8.2f\n",
			(p_dataset->p_latlon + (y * p_dataset->x_pixsize) + x)->lon);
	}
	
	return;
}

