/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2010 All rights reserved.                  */
/*  File Name    : sample11_read_L3Mproduct.c                                */
/*  Abstract     : I/O toolkit sample program                                */
/*  Function List                                                            */
/*      main()                       : Sample program main                   */
/*      initialize()                 : Initialization                        */
/*      terminate()                  : Termination                           */
/*      get_value()                  : Get dataset value                     */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2011/02/01                                                    */
/*  Version    1.08                                                          */
/*  Notice                                                                   */
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/** Include AMTK.h */
#ifdef __cplusplus
extern "C" {
#endif
#include "AMTK.h"
#ifdef __cplusplus
}
#endif

#include "sample_common.h"

/*---------------------------------------------------------------------------*
 * Common definition                                                         *
 *---------------------------------------------------------------------------*/
 
/* PRINT_SCAN_NUM defines print data number */
#define PRINT_SCAN_NUM        (2)
/** Number of Geophysical Data. */
#define MAX_GEO_NUM           (3)

/** Sample L1R product file name. */
#define FN_SAMPLE_PRODUCT_L3M ("../data/sample6_IOToolKit_l3m.h5")
/** Metadata definition. */
#define FN_SAMPLE_META_L3M    ("../data/metadata_sample_L3_month.txt")

/** Structure of dataset model. */
typedef struct
{
	/* Brightness Temperature (V) */
	float *p_tb_v;
	/* Brightness Temperature (H) */
	float *p_tb_h;
	/* Standard Deviation (V) */
	float *p_std_dev_v;
	/* Standard Deviation (H) */
	float *p_std_dev_h;
	/* Average Number (V) */
	int *p_avg_v;
	/* Average Number (H) */
	int *p_avg_h;
	/* Total Number (V) */
	int *p_total_v;
	/* Total Number (H) */
	int *p_total_h;
	
	/* Latitude of Observation Point
	 * Longitude of Observation Point */
	AM2_COMMON_LATLON *p_latlon;
	
	/* Number of pixels in X direction(longitude) */
	int x_pixsize;
	/* Number of pixels in Y direction(latitude) */
	int y_pixsize;
	
} DATASET_MODEL;


/** Initialization.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(hid_t file_id, DATASET_MODEL *p_dataset);

/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset);

/** Get dataset value.
 * @param p_dataset [IN] Dataset
 */
static void get_value(const DATASET_MODEL *p_dataset);


/*---------------------------------------------------------------------------*
 * Main program                                                              *
 *---------------------------------------------------------------------------*/
 
/** AMSR2 I/O toolkit sample program.
 * sample11: intput L3(Month) proudct.
 * 
 * GeophysicalName: Brightness Temperature (89GHz)
 * Projection     : PS-N
 * Resolution     : 10km
 * 
 * @param  argc [IN] Argument count
 * @param  argv [IN] Argument value
 * @return EXIT_SUCCESS
 * @return EXIT_FAILURE
 */
int main(int argc, char *argv[])
{
	hid_t file_id = 0;
	
	int i = 0;
	int meta_num = 0;
	int ret = 0;
	
	/* sample meta data read */
	char meta_name[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
	char meta_value[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
    char *p_value = NULL; 
	
	/* Dataset. */
	DATASET_MODEL dataset = {0};
	
	
	/* HDF product file open */
	file_id = AMTK_openH5(FN_SAMPLE_PRODUCT_L3M);
	if (0 > file_id)
	{
		if (-501 == file_id)
		{
			printf("Product file not found.[%s]\n", FN_SAMPLE_PRODUCT_L3M);
		}
		else
		{
			E_MSG("AMTK_openH5() error.[%d]\n", ret);
		}
		exit(EXIT_FAILURE);
	}
	
	/*------------------------------------------------------------------------*
	 * memory allocation                                                      *
	 *------------------------------------------------------------------------*/
	ret = initialize(file_id, &dataset);
	if (ret != RET_NORMAL)
	{
		E_MSG("initialize() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/*------------------------------------------------------------------------*
	 * input data                                                             *
	 *------------------------------------------------------------------------*/
	/* Read Metadata from Metadata file */
	if (RET_NORMAL != get_metadata_sample(FN_SAMPLE_META_L3M, meta_name,
		meta_value, &meta_num))
	{
		E_MSG("get_metadata_sample() error.\n");
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* meta data read */
	for (i = 0; i < meta_num; i++)
	{
		p_value = &meta_value[i][0];
		ret = AMTK_getMetaDataName(file_id, &meta_name[i][0], &p_value);
		if (0 > ret)
		{
			E_MSG("AMTK_getMetaDataName() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		printf("No=%d\tName=%s\tValue=%s\n", i, &meta_name[i][0], p_value);
	}
	
	/* Data set read */
	
	/* Brightness Temperature (V) */
	ret = AMTK_get_GridFloat(file_id, &dataset.p_tb_v, AM2_GRID_TBV);
	if (0 > ret)
	{
		E_MSG("AMTK_get_GridFloat() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Brightness Temperature (H) */
	ret = AMTK_get_GridFloat(file_id, &dataset.p_tb_h, AM2_GRID_TBH);
	if (0 > ret)
	{
		E_MSG("AMTK_get_GridFloat() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Standard Deviation (V) */
	ret = AMTK_get_GridFloat(file_id, &dataset.p_std_dev_v, AM2_GRID_TBV_STD);
	if (0 > ret)
	{
		E_MSG("AMTK_get_GridFloat() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Standard Deviation (H) */
	ret = AMTK_get_GridFloat(file_id, &dataset.p_std_dev_h, AM2_GRID_TBH_STD);
	if (0 > ret)
	{
		E_MSG("AMTK_get_GridFloat() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Average Number (V) */
	ret = AMTK_get_GridInt(file_id, &dataset.p_avg_v, AM2_GRID_TBV_ANUM);
	if (0 > ret)
	{
		E_MSG("AMTK_get_GridInt() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Average Number (H) */
	ret = AMTK_get_GridInt(file_id, &dataset.p_avg_h, AM2_GRID_TBH_ANUM);
	if (0 > ret)
	{
		E_MSG("AMTK_get_GridInt() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Total Number (V) */
	ret = AMTK_get_GridInt(file_id, &dataset.p_total_v, AM2_GRID_TBV_TNUM);
	if (0 > ret)
	{
		E_MSG("AMTK_get_GridInt() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Total Number (H) */
	ret = AMTK_get_GridInt(file_id, &dataset.p_total_h, AM2_GRID_TBH_TNUM);
	if (0 > ret)
	{
		E_MSG("AMTK_get_GridInt() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Latitude of Observation Point
	 * Longitude of Observation Point
	 * # scan_start, scan_end ... unused(AM2_SCANNUM_NOCHK)
	 */
	ret = AMTK_getLatLon(file_id, &dataset.p_latlon, AM2_SCANNUM_NOCHK,
		AM2_SCANNUM_NOCHK, AM2_GRID_LATLON);
	if (0 > ret)
	{
		E_MSG("AMTK_getLatLon() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* get dataset value */
	get_value(&dataset);
	
	/* termination */
	terminate(file_id, &dataset);
	
	printf("L3(Month) product read end\n");
	
	exit(EXIT_SUCCESS);
}


/** Initialization.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(hid_t file_id, DATASET_MODEL *p_dataset)
{
	int ret = 0;
	int dimsize[3] = {0};
	
	/* memory allocation */
	
	/* Brightness Temperature (V)
	 * (type size * X pixel * Y pixel ) */
	ret = AMTK_getDimSize(file_id, AM2_GRID_TBV, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	/*
	 * get size
	 * # dimsize[0] : Number of pixels in X direction(longitude)
	 * # dimsize[1] : Number of pixels in Y direction(latitude)
	 */
	p_dataset->x_pixsize = dimsize[0];
	p_dataset->y_pixsize = dimsize[1];
	
	p_dataset->p_tb_v = (float *) malloc(sizeof(float)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_tb_v)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Brightness Temperature (H)
	 * (type size * X pixel * Y pixel ) */
	ret = AMTK_getDimSize(file_id, AM2_GRID_TBH, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_tb_h = (float *) malloc(sizeof(float)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_tb_h)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	
	/* Standard Deviation (V)
	 * (type size * X pixel * Y pixel ) */
	ret = AMTK_getDimSize(file_id, AM2_GRID_TBV_STD, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_std_dev_v = (float *) malloc(sizeof(float)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_std_dev_v)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Standard Deviation (H)
	 * (type size * X pixel * Y pixel ) */
	ret = AMTK_getDimSize(file_id, AM2_GRID_TBH_STD, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_std_dev_h = (float *) malloc(sizeof(float)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_std_dev_h)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Average Number (V)
	 * (type size * X pixel * Y pixel ) */
	ret = AMTK_getDimSize(file_id, AM2_GRID_TBV_ANUM, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_avg_v = (int *) malloc(sizeof(int)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_avg_v)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Average Number (H)
	 * (type size * X pixel * Y pixel ) */
	ret = AMTK_getDimSize(file_id, AM2_GRID_TBH_ANUM, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_avg_h = (int *) malloc(sizeof(int)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_avg_h)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Total Number (V)
	 * (type size * X pixel * Y pixel ) */
	ret = AMTK_getDimSize(file_id, AM2_GRID_TBV_TNUM, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_total_v = (int *) malloc(sizeof(int)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_total_v)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Total Number (H)
	 * (type size * X pixel * Y pixel ) */
	ret = AMTK_getDimSize(file_id, AM2_GRID_TBH_TNUM, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_total_h = (int *) malloc(sizeof(int)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_total_h)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Latitude of Observation Point
	 * Longitude of Observation Point
	 * (type size * dimsize[0](filesize)) */
	ret = AMTK_getDimSize(file_id, AM2_GRID_LATLON, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	
	printf("AM2_GRID_LATLON dimsize = (%d, %d, %d)\n", dimsize[0], dimsize[1],
		dimsize[2]);
	
	p_dataset->p_latlon = (AM2_COMMON_LATLON *) malloc(
		sizeof(AM2_COMMON_LATLON) * dimsize[0]);
	if (NULL == p_dataset->p_latlon)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	return RET_NORMAL;
}


/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset)
{
	int ret = 0;
	
	if (0 <= file_id)
	{
		/* HDF product file close */
		ret = AMTK_closeH5(file_id);
		if (0 > ret)
		{
			E_MSG("AMTK_closeH5() error.[%d]\n", ret);
		}
	}
	
	/* free memory */
	free(p_dataset->p_tb_v);
	p_dataset->p_tb_v = NULL;
	
	free(p_dataset->p_tb_h);
	p_dataset->p_tb_h = NULL;
	
	free(p_dataset->p_std_dev_v);
	p_dataset->p_std_dev_v = NULL;
	
	free(p_dataset->p_std_dev_h);
	p_dataset->p_std_dev_h = NULL;
	
	free(p_dataset->p_avg_v);
	p_dataset->p_avg_v = NULL;
	
	free(p_dataset->p_avg_h);
	p_dataset->p_avg_h = NULL;
	
	free(p_dataset->p_total_v);
	p_dataset->p_total_v = NULL;
	
	free(p_dataset->p_total_h);
	p_dataset->p_total_h = NULL;
	
	free(p_dataset->p_latlon);
	p_dataset->p_latlon = NULL;
	
	return;
}


/** Get dataset value.
 * @param p_dataset [IN] Dataset
 */
static void get_value(const DATASET_MODEL *p_dataset)
{
	int x = 0;
	int y = 0;
	
	for (x = 0; x < PRINT_SCAN_NUM; x++)
	{
		/* Brightness Temperature (V) */
		/* first pixel */
		y = 0;
		printf("X  = %d Y = %d\n", x, y);
		printf("Brightness Temperature (V) = %8.2f\n",
			*(p_dataset->p_tb_v + (x * p_dataset->y_pixsize) + y));
		/* last pixel */
		y = p_dataset->y_pixsize - 1;
		printf("X  = %d Y = %d\n", x, y);
		printf("Brightness Temperature (V) = %8.2f\n",
			*(p_dataset->p_tb_v + (x * p_dataset->y_pixsize) + y));
		
		/* Brightness Temperature (H) */
		/* first pixel */
		y = 0;
		printf("X  = %d Y = %d\n", x, y);
		printf("Brightness Temperature (H) = %8.2f\n",
			*(p_dataset->p_tb_h + (x * p_dataset->y_pixsize) + y));
		/* last pixel */
		y = p_dataset->y_pixsize - 1;
		printf("X  = %d Y = %d\n", x, y);
		printf("Brightness Temperature (H) = %8.2f\n",
			*(p_dataset->p_tb_h + (x * p_dataset->y_pixsize) + y));
		
		/* Standard Deviation (V) */
		/* first pixel */
		y = 0;
		printf("X  = %d Y = %d\n", x, y);
		printf("Standard Deviation (V) = %8.2f\n",
			*(p_dataset->p_std_dev_v + (x * p_dataset->y_pixsize) + y));
		/* last pixel */
		y = p_dataset->y_pixsize - 1;
		printf("X  = %d Y = %d\n", x, y);
		printf("Standard Deviation (V) = %8.2f\n",
			*(p_dataset->p_std_dev_v + (x * p_dataset->y_pixsize) + y));
		
		/* Standard Deviation (H) */
		/* first pixel */
		y = 0;
		printf("X  = %d Y = %d\n", x, y);
		printf("Standard Deviation (H) = %8.2f\n",
			*(p_dataset->p_std_dev_h + (x * p_dataset->y_pixsize) + y));
		/* last pixel */
		y = p_dataset->y_pixsize - 1;
		printf("X  = %d Y = %d\n", x, y);
		printf("Standard Deviation (H) = %8.2f\n",
			*(p_dataset->p_std_dev_h + (x * p_dataset->y_pixsize) + y));
		
		/* Average Number (V) */
		/* first pixel */
		y = 0;
		printf("X  = %d Y = %d\n", x, y);
		printf("Average Number (V) = %d\n",
			*(p_dataset->p_avg_v + (x * p_dataset->y_pixsize) + y));
		/* last pixel */
		y = p_dataset->y_pixsize - 1;
		printf("X  = %d Y = %d\n", x, y);
		printf("Average Number (V) = %d\n",
			*(p_dataset->p_avg_v + (x * p_dataset->y_pixsize) + y));
		
		/* Average Number (H) */
		/* first pixel */
		y = 0;
		printf("X  = %d Y = %d\n", x, y);
		printf("Average Number (H) = %d\n",
			*(p_dataset->p_avg_h + (x * p_dataset->y_pixsize) + y));
		/* last pixel */
		y = p_dataset->y_pixsize - 1;
		printf("X  = %d Y = %d\n", x, y);
		printf("Average Number (H) = %d\n",
			*(p_dataset->p_avg_h + (x * p_dataset->y_pixsize) + y));
		
		/* Total Number (V) */
		/* first pixel */
		y = 0;
		printf("X  = %d Y = %d\n", x, y);
		printf("Total Number (V) = %d\n",
			*(p_dataset->p_total_v + (x * p_dataset->y_pixsize) + y));
		/* last pixel */
		y = p_dataset->y_pixsize - 1;
		printf("X  = %d Y = %d\n", x, y);
		printf("Total Number (V) = %d\n",
			*(p_dataset->p_total_v + (x * p_dataset->y_pixsize) + y));
		
		/* Total Number (H) */
		/* first pixel */
		y = 0;
		printf("X  = %d Y = %d\n", x, y);
		printf("Total Number (H) = %d\n",
			*(p_dataset->p_total_h + (x * p_dataset->y_pixsize) + y));
		/* last pixel */
		y = p_dataset->y_pixsize - 1;
		printf("X  = %d Y = %d\n", x, y);
		printf("Total Number (H) = %d\n",
			*(p_dataset->p_total_h + (x * p_dataset->y_pixsize) + y));
		
		
		/* Latitude/Longitude of Observation Point */
		printf("Latitude/Longitude of Observation Point\n");
        /*
		 * # L3LATLONFILE: data[Y pixel][X pixel]
		 * #  Example) data[760][1120]
		 * #  -> X pixel and Y pixel is reversed. 
		 */
		/* first pixel (x -> y, y -> x) */
		y = 0;
		printf("X  = %d Y = %d\n", x, y);
		printf("Latitude  = %8.2f\n",
			(p_dataset->p_latlon + (y * p_dataset->x_pixsize) + x)->lat);
		printf("Longitude = %8.2f\n",
			(p_dataset->p_latlon + (y * p_dataset->x_pixsize) + x)->lon);
		
		/* last pixel (x -> y, y -> x) */
		y = p_dataset->y_pixsize - 1;
		printf("X  = %d Y = %d\n", x, y);
		printf("Latitude  = %8.2f\n",
			(p_dataset->p_latlon + (y * p_dataset->x_pixsize) + x)->lat);
		printf("Longitude = %8.2f\n",
			(p_dataset->p_latlon + (y * p_dataset->x_pixsize) + x)->lon);
	}
	
	return;
}

