/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2010 All rights reserved.                  */
/*  File Name    : sample12_make_L1Rproduct.c                                */
/*  Abstract     : I/O toolkit sample program                                */
/*  Function List                                                            */
/*      main()                       : Sample program main                   */
/*      initialize()                 : Initialization                        */
/*      terminate()                  : Termination                           */
/*      set_value()                  : Set dataset value                     */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2011/01/26                                                    */
/*  Version    1.08                                                          */
/*  Notice                                                                   */
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/** Include AMTK.h */
#ifdef __cplusplus
extern "C" {
#endif
#include "AMTK.h"
#ifdef __cplusplus
}
#endif

#include "sample_common.h"

/*---------------------------------------------------------------------------*
 * Common definition                                                         *
 *---------------------------------------------------------------------------*/

/** Sample L1R product file name. */
#define FN_SAMPLE_PRODUCT_L1R ("../data/sample12_IOToolKit_l1r.h5")
/** Metadata definition. */
#define FN_SAMPLE_META_L1R    ("../data/metadata_sample_L1R.txt")

/** Number of scans. */
#define SCENE_SCAN_NUM        (2000)
/** Overlap scans. */
#define OVERLAP_SCAN_NUM      (0)


/** Brightness Temperature data model. */
typedef struct
{
	/* Access Label. */
	int access_label;
	/* Brightness Tempertature Data. */
	float *p_data;
	
} TB_DATA;

/** Structure of dataset model. */
typedef struct
{
	/* Scan Time */
	AM2_COMMON_SCANTIME *p_scan_time;
	
	/* Position in Orbit */
	double *p_pos_orbit;
	/* Navigation Data */
	float *p_navi;
	/* Attitude Data */
	float *p_att;
	
	/* Brightness Temperature <06GHz resolution> */
	TB_DATA *p_res06_tb_table;
	int res06_tb_num;
	/* Brightness Temperature <10GHz resolution> */
	TB_DATA *p_res10_tb_table;
	int res10_tb_num;
	/* Brightness Temperature <23GHz resolution> */
	TB_DATA *p_res23_tb_table;
	int res23_tb_num;
	/* Brightness Temperature <36GHz resolution> */
	TB_DATA *p_res36_tb_table;
	int res36_tb_num;
	/* Brightness Temperature <89GHz resolution> */
	TB_DATA *p_res89_tb_table;
	int res89_tb_num;
	
	/* Sun Azimuth */
	float *p_sun_azimuth;
	/* Sun Elevation */
	float *p_sun_elevation;
	/* Earth Incidencet */
	float *p_earth_incidence;
	/* Earth Azimuth */
	float *p_earth_azimuth;
	/* Area Mean Height */
	float *p_height;
	/* Land_Ocean Flag 6 to 36 */
	int *p_lof_low;
	/* Land_Ocean Flag 896 */
	int *p_lof_high;
	
	/* Latitude of Observation Point for 89A
	 * Longitude of Observation Point for 89A */
	AM2_COMMON_LATLON *p_latlon89A;
	/* Latitude of Observation Point for 89B
	 * Longitude of Observation Point for 89B */
	AM2_COMMON_LATLON *p_latlon89B;
	/* Scan Data Quality */
	AMTK_SCAN_DATA_QUALITY *p_scan_quality;
	
	/* Pixel Data Quality 6 to 36 */
	unsigned char *p_pixel_quality_low;
	/* Pixel Data Quality 89 */
	unsigned char *p_pixel_quality_high;

	
} DATASET_MODEL;


/** Initialization.
 * @param scan_size [IN]  Number of scans
 * @param p_dataset [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(int scan_size, DATASET_MODEL *p_dataset);

/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset);

/** Set dataset value.
 * @param scan_size [IN]  Number of scans
 * @param p_dataset [OUT] Dataset
 */
static void set_value(int scan_size, DATASET_MODEL *p_dataset);


/*---------------------------------------------------------------------------*
 * Main program                                                              *
 *---------------------------------------------------------------------------*/
 
/** AMSR2 I/O toolkit sample program.
 * sample12: output L1R proudct.
 * 
 * @param  argc [IN] Argument count
 * @param  argv [IN] Argument value
 * @return EXIT_SUCCESS
 * @return EXIT_FAILURE
 */
int main(int argc, char *argv[])
{
	hid_t file_id = 0;

	int i = 0;
	int meta_num = 0;
	int dimsize[3] = {0};
	int ret = 0;

	/* sample meta data read */
	char meta_name[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
	char meta_value[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};

	const int scan_size = SCENE_SCAN_NUM + OVERLAP_SCAN_NUM * 2;
	const int scan_start = 1 - OVERLAP_SCAN_NUM;
	const int scan_end = SCENE_SCAN_NUM + OVERLAP_SCAN_NUM;

	/* Dataset. */
	DATASET_MODEL dataset = {0};

	/* Brightness Temperature <6GHz resolution> */
	TB_DATA res06_tb_table[] =
	{
		{AM2_RES06_TB06V, NULL},
		{AM2_RES06_TB06H, NULL},
		{AM2_RES06_TB07V, NULL},
		{AM2_RES06_TB07H, NULL},
		{AM2_RES06_TB10V, NULL},
		{AM2_RES06_TB10H, NULL},
		{AM2_RES06_TB18V, NULL},
		{AM2_RES06_TB18H, NULL},
		{AM2_RES06_TB23V, NULL},
		{AM2_RES06_TB23H, NULL},
		{AM2_RES06_TB36V, NULL},
		{AM2_RES06_TB36H, NULL},
		{AM2_RES06_TB89V, NULL},
		{AM2_RES06_TB89H, NULL}
	};
	int res06_tb_num = sizeof(res06_tb_table) / sizeof(res06_tb_table[0]);

	/* Brightness Temperature <10GHz resolution> */
	TB_DATA res10_tb_table[] =
	{
		{AM2_RES10_TB10V, NULL},
		{AM2_RES10_TB10H, NULL},
		{AM2_RES10_TB18V, NULL},
		{AM2_RES10_TB18H, NULL},
		{AM2_RES10_TB23V, NULL},
		{AM2_RES10_TB23H, NULL},
		{AM2_RES10_TB36V, NULL},
		{AM2_RES10_TB36H, NULL},
		{AM2_RES10_TB89V, NULL},
		{AM2_RES10_TB89H, NULL}
	};
	int res10_tb_num = sizeof(res10_tb_table) / sizeof(res10_tb_table[0]);

	/* Brightness Temperature <23GHz resolution> */
	TB_DATA res23_tb_table[] =
	{
		{AM2_RES23_TB18V, NULL},
		{AM2_RES23_TB18H, NULL},
		{AM2_RES23_TB23V, NULL},
		{AM2_RES23_TB23H, NULL},
		{AM2_RES23_TB36V, NULL},
		{AM2_RES23_TB36H, NULL},
		{AM2_RES23_TB89V, NULL},
		{AM2_RES23_TB89H, NULL}
	};
	int res23_tb_num = sizeof(res23_tb_table) / sizeof(res23_tb_table[0]);

	/* Brightness Temperature <36GHz resolution> */
	TB_DATA res36_tb_table[] =
	{
		{AM2_RES36_TB36V, NULL},
		{AM2_RES36_TB36H, NULL},
		{AM2_RES36_TB89V, NULL},
		{AM2_RES36_TB89H, NULL}
	};
	int res36_tb_num = sizeof(res36_tb_table) / sizeof(res36_tb_table[0]);

	/* Brightness Temperature <89GHz resolution> */
	TB_DATA res89_tb_table[] =
	{
		{AM2_TB89AV, NULL},
		{AM2_TB89AH, NULL},
		{AM2_TB89BV, NULL},
		{AM2_TB89BH, NULL}
	};
	int res89_tb_num = sizeof(res89_tb_table) / sizeof(res89_tb_table[0]);

	/* Set Brightness Temperature data */
	dataset.p_res06_tb_table = res06_tb_table;
	dataset.res06_tb_num = res06_tb_num;
	
	dataset.p_res10_tb_table = res10_tb_table;
	dataset.res10_tb_num = res10_tb_num;
	
	dataset.p_res23_tb_table = res23_tb_table;
	dataset.res23_tb_num = res23_tb_num;
	
	dataset.p_res36_tb_table = res36_tb_table;
	dataset.res36_tb_num = res36_tb_num;
	
	dataset.p_res89_tb_table = res89_tb_table;
	dataset.res89_tb_num = res89_tb_num;
	
	/* memory allocation */
	ret = initialize(scan_size, &dataset);
	if (ret != RET_NORMAL)
	{
		E_MSG("initialize() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* set dataset value */
	set_value(scan_size, &dataset);
	
	/*------------------------------------------------------------------------*
	 * output data                                                            *
	 *------------------------------------------------------------------------*/
	/* HDF product file open (write mode) */
	file_id = AMTK_openH5_Write(FN_SAMPLE_PRODUCT_L1R, AM2_CREATE_MODE);
	if (0 > file_id)
	{
		E_MSG("AMTK_openH5_Write() error.[%d]\n", file_id);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}

	/* Read Metadata from Metadata file */
	if (RET_NORMAL != get_metadata_sample(FN_SAMPLE_META_L1R, meta_name,
		meta_value, &meta_num))
	{
		E_MSG("get_metadata_sample() error.\n");
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Metadata write */
	for (i = 0; i < meta_num; i++) 
	{
		ret = AMTK_setMetaDataName(file_id, &meta_name[i][0],
			&meta_value[i][0]);
		if (0 > ret)
		{
			E_MSG("AMTK_setMetaDataName() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}

	/* Data set write */
	/* Scan Time */
	{
		dimsize[0] = scan_size;
		dimsize[1] = 0;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_SCAN_TIME_DEF, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_setScanTime(file_id, dataset.p_scan_time, scan_start,
			scan_end);
		if (0 > ret)
		{
			E_MSG("AMTK_setScanTime() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Position in Orbit */
	{
		dimsize[0] = scan_size;
		dimsize[1] = 0;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_POS_ORBIT, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathDouble(file_id, dataset.p_pos_orbit, scan_start,
			scan_end, AM2_POS_ORBIT);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathDouble() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Navigation Data */
	{
		dimsize[0] = scan_size;
		dimsize[1] = 6;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_NAVI, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathFloat(file_id, dataset.p_navi, scan_start, scan_end,
			AM2_NAVI);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Attitude Data */
	{
		dimsize[0] = scan_size;
		dimsize[1] = 3;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_ATT, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathFloat(file_id, dataset.p_att, scan_start, scan_end,
			AM2_ATT);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Brightness Temperature (res06,6.9GHz,V) ... (res06,89.0GHz,H) */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_LO;
		dimsize[2] = 0;
		for (i = 0; i < res06_tb_num; i++)
		{
			ret = AMTK_setDimSize(file_id, res06_tb_table[i].access_label,
				dimsize);
			if (0 > ret)
			{
				E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
				terminate(file_id, &dataset);
				exit(EXIT_FAILURE);
			}
			
			ret = AMTK_set_SwathFloat(file_id, res06_tb_table[i].p_data,
				scan_start, scan_end, res06_tb_table[i].access_label);
			if (0 > ret)
			{
				E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
				terminate(file_id, &dataset);
				exit(EXIT_FAILURE);
			}
		}
	}
	
	/* Brightness Temperature (res10,10.7GHz,V) ... (res10,89.0GHz,H) */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_LO;
		dimsize[2] = 0;
		for (i = 0; i < res10_tb_num; i++)
		{
			ret = AMTK_setDimSize(file_id, res10_tb_table[i].access_label,
				dimsize);
			if (0 > ret)
			{
				E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
				terminate(file_id, &dataset);
				exit(EXIT_FAILURE);
			}
			
			ret = AMTK_set_SwathFloat(file_id, res10_tb_table[i].p_data,
				scan_start, scan_end, res10_tb_table[i].access_label);
			if (0 > ret)
			{
				E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
				terminate(file_id, &dataset);
				exit(EXIT_FAILURE);
			}
		}
	}
	
	/* Brightness Temperature (res23,18.7GHz,V) ... (res23,89.0GHz,H) */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_LO;
		dimsize[2] = 0;
		for (i = 0; i < res23_tb_num; i++)
		{
			ret = AMTK_setDimSize(file_id, res23_tb_table[i].access_label,
				dimsize);
			if (0 > ret)
			{
				E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
				terminate(file_id, &dataset);
				exit(EXIT_FAILURE);
			}
			
			ret = AMTK_set_SwathFloat(file_id, res23_tb_table[i].p_data,
				scan_start, scan_end, res23_tb_table[i].access_label);
			if (0 > ret)
			{
				E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
				terminate(file_id, &dataset);
				exit(EXIT_FAILURE);
			}
		}
	}
	
	/* Brightness Temperature (res36,36.5GHz,V) ... (res36,89.0GHz,H) */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_LO;
		dimsize[2] = 0;
		for (i = 0; i < res36_tb_num; i++)
		{
			ret = AMTK_setDimSize(file_id, res36_tb_table[i].access_label,
				dimsize);
			if (0 > ret)
			{
				E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
				terminate(file_id, &dataset);
				exit(EXIT_FAILURE);
			}
			
			ret = AMTK_set_SwathFloat(file_id, res36_tb_table[i].p_data,
				scan_start, scan_end, res36_tb_table[i].access_label);
			if (0 > ret)
			{
				E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
				terminate(file_id, &dataset);
				exit(EXIT_FAILURE);
			}
		}
	}
	
	/* Brightness Temperature (original,89GHz-A,V) ... (original,89GHz-B,H) */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_HI;
		dimsize[2] = 0;
		for (i = 0; i < res89_tb_num; i++)
		{
			ret = AMTK_setDimSize(file_id, res89_tb_table[i].access_label,
				dimsize);
			if (0 > ret)
			{
				E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
				terminate(file_id, &dataset);
				exit(EXIT_FAILURE);
			}
			
			ret = AMTK_set_SwathFloat(file_id, res89_tb_table[i].p_data,
				scan_start, scan_end, res89_tb_table[i].access_label);
			if (0 > ret)
			{
				E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
				terminate(file_id, &dataset);
				exit(EXIT_FAILURE);
			}
		}
	}
	
	/* Latitude of Observation Point for 89A
	 * Longitude of Observation Point for 89A */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_HI;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_LATLON_RS_89A, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_setLatLon(file_id, dataset.p_latlon89A, scan_start, scan_end,
			AM2_LATLON_RS_89A);
		if (0 > ret)
		{
			E_MSG("AMTK_setLatLon() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Latitude of Observation Point for 89B
	 * Longitude of Observation Point for 89B */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_HI;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_LATLON_RS_89B, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_setLatLon(file_id, dataset.p_latlon89B, scan_start, scan_end,
			AM2_LATLON_RS_89B);
		if (0 > ret)
		{
			E_MSG("AMTK_setLatLon() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}

	/* Area Mean Height */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_LO;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_MEAN_HEIGHT, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathFloat(file_id, dataset.p_height, scan_start,
			scan_end, AM2_MEAN_HEIGHT);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}

	/* Sun Azimuth */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_LO;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_SUN_AZ, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathFloat(file_id, dataset.p_sun_azimuth, scan_start,
			scan_end, AM2_SUN_AZ);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}

	/* Sun Elevation */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_LO;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_SUN_EL, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathFloat(file_id, dataset.p_sun_elevation, scan_start,
			scan_end, AM2_SUN_EL);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}

	/* Earth Incidence */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_LO;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_EARTH_INC, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathFloat(file_id, dataset.p_earth_incidence,
			scan_start, scan_end, AM2_EARTH_INC);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}

	/* Earth Azimuth */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_LO;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_EARTH_AZ, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathFloat(file_id, dataset.p_earth_azimuth, scan_start,
			scan_end, AM2_EARTH_AZ);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}

	/* Land_Ocean Flag 6 to 36 */
	{
		dimsize[0] = 6;
		dimsize[1] = scan_size;
		dimsize[2] = AM2_DEF_SNUM_LO;
		ret = AMTK_setDimSize(file_id, AM2_LOF_RES_LO, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathInt(file_id, dataset.p_lof_low, scan_start,
			scan_end, AM2_LOF_RES_LO);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathInt() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Land_Ocean Flag 89 */
	{
		dimsize[0] = 2;
		dimsize[1] = scan_size;
		dimsize[2] = AM2_DEF_SNUM_HI;
		ret = AMTK_setDimSize(file_id, AM2_LOF_RES_HI, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathInt(file_id, dataset.p_lof_high, scan_start,
			scan_end, AM2_LOF_RES_HI);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathInt() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Scan Data Quality */
	{
		dimsize[0] = scan_size;
		dimsize[1] = sizeof(AMTK_SCAN_DATA_QUALITY);
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_SCAN_QUAL, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_ScanDataQuality(file_id, dataset.p_scan_quality,
			scan_start, scan_end, AM2_SCAN_QUAL);
		if (0 > ret)
		{
			E_MSG("AMTK_set_ScanDataQuality() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Pixel Data Quality 6 to 36 */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_LO * 2; /* binary (2byte) */
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_PIX_QUAL_LO, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathUChar(file_id, dataset.p_pixel_quality_low,
			scan_start, scan_end, AM2_PIX_QUAL_LO);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathUChar() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Pixel Data Quality 89 */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_HI;	 /* binary (1byte) */
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_PIX_QUAL_HI, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathUChar(file_id, dataset.p_pixel_quality_high,
			scan_start, scan_end, AM2_PIX_QUAL_HI);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathUChar() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* termination */
	terminate(file_id, &dataset);
	
	printf("L1R product created\n");
	
	exit(EXIT_SUCCESS);
}


/** Initialization.
 * @param scan_size [IN]  Number of scans
 * @param p_dataset [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(int scan_size, DATASET_MODEL *p_dataset)
{
	int i = 0;

	/* memory allocation */
	
	/* Scan Time
	 * (type size * Number of scans) */
	p_dataset->p_scan_time = (AM2_COMMON_SCANTIME *) malloc(
		sizeof(AM2_COMMON_SCANTIME) * scan_size);
	if (NULL == p_dataset->p_scan_time)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Position in Orbit
	 * (type size * Number of scans) */
	p_dataset->p_pos_orbit = (double *) malloc(sizeof(double) * scan_size);
	if (NULL == p_dataset->p_pos_orbit)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	
	/* Navigation Data
	 * (type size * Number of scans * 6) */
	p_dataset->p_navi = (float *) malloc(sizeof(float) * scan_size * 6);
	if (NULL == p_dataset->p_navi)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	
	/* Attitude Data
	 * (type size * Number of scans * 3) */
	p_dataset->p_att = (float *) malloc(sizeof(float) * scan_size * 3);
	if (NULL == p_dataset->p_att)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Brightness Temperature (res06,6.9GHz,V) ... (res06,89.0GHz,H)
	 * (type size * Number of scans * 243) */
	for (i = 0; i < p_dataset->res06_tb_num; i++)
	{
		p_dataset->p_res06_tb_table[i].p_data =
			(float *) malloc(sizeof(float) * scan_size * AM2_DEF_SNUM_LO);
		if (NULL == p_dataset->p_res06_tb_table[i].p_data)
		{
			E_MSG("malloc() error.\n");
			return RET_ERROR;
		}
	}
	/* Brightness Temperature (res10,10.7GHz,V) ... (res10,89.0GHz,H)
	 * (type size * Number of scans * 243) */
	for (i = 0; i < p_dataset->res10_tb_num; i++)
	{
		p_dataset->p_res10_tb_table[i].p_data =
			(float *) malloc(sizeof(float) * scan_size * AM2_DEF_SNUM_LO);
		if (NULL == p_dataset->p_res10_tb_table[i].p_data)
		{
			E_MSG("malloc() error.\n");
			return RET_ERROR;
		}
	}
	/* Brightness Temperature (res23,18.7GHz,V) ... (res23,89.0GHz,H)
	 * (type size * Number of scans * 243) */
	for (i = 0; i < p_dataset->res23_tb_num; i++)
	{
		p_dataset->p_res23_tb_table[i].p_data = 
			(float *) malloc(sizeof(float) * scan_size * AM2_DEF_SNUM_LO);
		if (NULL == p_dataset->p_res23_tb_table[i].p_data)
		{
			E_MSG("malloc() error.\n");
			return RET_ERROR;
		}
	}
	/* Brightness Temperature (res36,36.5GHz,V) ... (res36,89.0GHz,H)
	 * (type size * Number of scans * 243) */
	for (i = 0; i < p_dataset->res36_tb_num; i++)
	{
		p_dataset->p_res36_tb_table[i].p_data =
			(float *) malloc(sizeof(float) * scan_size * AM2_DEF_SNUM_LO);
		if (NULL == p_dataset->p_res36_tb_table[i].p_data)
		{
			E_MSG("malloc() error.\n");
			return RET_ERROR;
		}
	}
	/* Brightness Temperature (original,89GHz-A,V) ... (original,89GHz-B,H)
	 * (type size * Number of scans * 486) */
	for (i = 0; i < p_dataset->res89_tb_num; i++)
	{
		p_dataset->p_res89_tb_table[i].p_data =
			(float *) malloc(sizeof(float) * scan_size * AM2_DEF_SNUM_HI);
		if (NULL == p_dataset->p_res89_tb_table[i].p_data)
		{
			E_MSG("malloc() error.\n");
			return RET_ERROR;
		}
	}
	
	/* Latitude of Observation Point for 89A
	 * Longitude of Observation Point for 89A
	 * (type size * Number of scans * 486) */
	p_dataset->p_latlon89A = (AM2_COMMON_LATLON *) malloc(
		sizeof(AM2_COMMON_LATLON) * scan_size * AM2_DEF_SNUM_HI);
	if (NULL == p_dataset->p_latlon89A)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Latitude of Observation Point for 89B
	 * Longitude of Observation Point for 89B
	 * (type size * Number of scans * 486) */
	p_dataset->p_latlon89B = (AM2_COMMON_LATLON *) malloc(
		sizeof(AM2_COMMON_LATLON) * scan_size * AM2_DEF_SNUM_HI);
	if (NULL == p_dataset->p_latlon89B)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Area Mean Height
	 * (type size * Number of scans * 243) */
	p_dataset->p_height = (float *) malloc(
		sizeof(float) * scan_size * AM2_DEF_SNUM_LO);
	if (NULL == p_dataset->p_height)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Sun Azimuth
	 * (type size * Number of scans * 243) */
	p_dataset->p_sun_azimuth = (float *) malloc(sizeof(float) * scan_size *
		AM2_DEF_SNUM_LO);
	if (NULL == p_dataset->p_sun_azimuth)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Sun Elevation
	 * (type size * Number of scans * 243) */
	p_dataset->p_sun_elevation = (float *) malloc(sizeof(float) * scan_size *
		AM2_DEF_SNUM_LO);
	if (NULL == p_dataset->p_sun_elevation)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Earth Incidence
	 * (type size * Number of scans * 243) */
	p_dataset->p_earth_incidence = (float *) malloc(sizeof(float) * scan_size *
		AM2_DEF_SNUM_LO);
	if (NULL == p_dataset->p_earth_incidence)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Earth Azimuth
	 * (type size * Number of scans * 243) */
	p_dataset->p_earth_azimuth = (float *) malloc(sizeof(float) * scan_size *
		AM2_DEF_SNUM_LO);
	if (NULL == p_dataset->p_earth_azimuth)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Land_Ocean Flag 6 to 36
	 * (type size * 6 * Number of scans * 243) */
	p_dataset->p_lof_low = (int *) malloc(sizeof(int) * 6 * scan_size *
		AM2_DEF_SNUM_LO);
	if (NULL == p_dataset->p_lof_low)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Land_Ocean Flag 89
	 * (type size * 2 * Number of scans * 486) */
	p_dataset->p_lof_high = (int *) malloc(sizeof(int) * 2 * scan_size *
		AM2_DEF_SNUM_HI);
	if (NULL == p_dataset->p_lof_high)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Scan Data Quality
	 * (type size * Number of scans) */
	p_dataset->p_scan_quality = (AMTK_SCAN_DATA_QUALITY *) malloc(
		sizeof(AMTK_SCAN_DATA_QUALITY) * scan_size);
	if (NULL == p_dataset->p_scan_quality)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Pixel Data Quality 6 to 36
	 * (type size * Numbef of scans * 486[243 * binary 2byte]) */
	p_dataset->p_pixel_quality_low = (unsigned char *) malloc(scan_size
		* (AM2_DEF_SNUM_LO * 2));
	if (NULL == p_dataset->p_pixel_quality_low)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Pixel Data Quality 89
	 * (type size * Numbef of scans * 486[486 * binary 1byte]) */
	p_dataset->p_pixel_quality_high = (unsigned char *) malloc(scan_size
		* (AM2_DEF_SNUM_HI * 1));
	if (NULL == p_dataset->p_pixel_quality_high)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	return RET_NORMAL;
}


/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset)
{
	int i = 0;
	int ret = 0;
	
	if (0 <= file_id)
	{
		/* HDF product file close (write mode) */
		ret = AMTK_closeH5_Write(file_id);
		if (0 > ret)
		{
			E_MSG("AMTK_closeH5_Write() error.[%d]\n", ret);
		}
	}
	
	/* free memory */
	free(p_dataset->p_scan_time);
	p_dataset->p_scan_time = NULL;
	
	free(p_dataset->p_pos_orbit);
	p_dataset->p_pos_orbit = NULL;
	
	free(p_dataset->p_navi);
	p_dataset->p_navi = NULL;
	
	free(p_dataset->p_att);
	p_dataset->p_att = NULL;
	
	for (i = 0; i < p_dataset->res06_tb_num; i++)
	{
		free(p_dataset->p_res06_tb_table[i].p_data);
		p_dataset->p_res06_tb_table[i].p_data = NULL;
	}
	for (i = 0; i < p_dataset->res10_tb_num; i++)
	{
		free(p_dataset->p_res10_tb_table[i].p_data);
		p_dataset->p_res10_tb_table[i].p_data = NULL;
	}
	for (i = 0; i < p_dataset->res23_tb_num; i++)
	{
		free(p_dataset->p_res23_tb_table[i].p_data);
		p_dataset->p_res23_tb_table[i].p_data = NULL;
	}
	for (i = 0; i < p_dataset->res36_tb_num; i++)
	{
		free(p_dataset->p_res36_tb_table[i].p_data);
		p_dataset->p_res36_tb_table[i].p_data = NULL;
	}
	for (i = 0; i < p_dataset->res89_tb_num; i++)
	{
		free(p_dataset->p_res89_tb_table[i].p_data);
		p_dataset->p_res89_tb_table[i].p_data = NULL;
	}
	
	free(p_dataset->p_latlon89A);
	p_dataset->p_latlon89A = NULL;
	
	free(p_dataset->p_latlon89B);
	p_dataset->p_latlon89B = NULL;
	
	free(p_dataset->p_height);
	p_dataset->p_height = NULL;
	
	free(p_dataset->p_sun_azimuth);
	p_dataset->p_sun_azimuth = NULL;
	
	free(p_dataset->p_sun_elevation);
	p_dataset->p_sun_elevation = NULL;
	
	free(p_dataset->p_earth_incidence);
	p_dataset->p_earth_incidence = NULL;
	
	free(p_dataset->p_earth_azimuth);
	p_dataset->p_earth_azimuth = NULL;
	
	free(p_dataset->p_scan_quality);
	p_dataset->p_scan_quality = NULL;
	
	free(p_dataset->p_lof_low);
	p_dataset->p_lof_low = NULL;
	
	free(p_dataset->p_lof_high);
	p_dataset->p_lof_high = NULL;
	
	free(p_dataset->p_pixel_quality_low);
	p_dataset->p_pixel_quality_low = NULL;
	
	free(p_dataset->p_pixel_quality_high);
	p_dataset->p_pixel_quality_high = NULL;
	
	return;
}


/** Set dataset value.
 * @param scan_size [IN]  Number of scans
 * @param p_dataset [OUT] Dataset
 */
static void set_value(int scan_size, DATASET_MODEL *p_dataset)
{
	int i = 0;
	int scan_num = 0;
	int pixel_num = 0;
	
	/* Scan Data Quality(sample) */
	AMTK_SCAN_DATA_QUALITY sample_data = {{0}};
	
	/* Scan loop */
	for (scan_num = 0; scan_num < scan_size; scan_num++)
	{
		/* Scan Time */
		(p_dataset->p_scan_time + scan_num)->year = 2010;
		(p_dataset->p_scan_time + scan_num)->month = scan_num % 12;
		if ((p_dataset->p_scan_time + scan_num)->month == 0)
			(p_dataset->p_scan_time + scan_num)->month = 1;
		(p_dataset->p_scan_time + scan_num)->day = scan_num % 28;
		if ((p_dataset->p_scan_time + scan_num)->day == 0)
			(p_dataset->p_scan_time + scan_num)->day = 1;
		(p_dataset->p_scan_time + scan_num)->hour= scan_num % 24;
		(p_dataset->p_scan_time + scan_num)->minute = scan_num % 60;
		(p_dataset->p_scan_time + scan_num)->second = scan_num % 60;
		(p_dataset->p_scan_time + scan_num)->ms = scan_num % 100;
		
		/* Position in Orbit */
		*(p_dataset->p_pos_orbit + scan_num) = scan_num * 0.001;
		
		/* Navigation Data */
		for (i = 0; i < 6; i++)
			*(p_dataset->p_navi + (6 * scan_num) + i) = scan_num * 0.01 + i + 1;
		
		/* Attitude Data */
		for (i = 0; i < 3; i++)
			*(p_dataset->p_att + (3 * scan_num) + i) = scan_num * 0.01 + i + 1;
		
		/* Pixel loop */
		for (pixel_num = 0; pixel_num < AM2_DEF_SNUM_LO; pixel_num++)
		{
			/* Brightness Temperature (res Low) */
			for (i = 0; i < p_dataset->res06_tb_num; i++)
			{
				*(p_dataset->p_res06_tb_table[i].p_data +
					(AM2_DEF_SNUM_LO * scan_num) + pixel_num) =
						0.01 * pixel_num + i + 1;
			}
			for (i = 0; i < p_dataset->res10_tb_num; i++)
			{
				*(p_dataset->p_res10_tb_table[i].p_data +
					(AM2_DEF_SNUM_LO * scan_num) + pixel_num) =
						0.01 * pixel_num + i + 1;
			}
			for (i = 0; i < p_dataset->res23_tb_num; i++)
			{
				*(p_dataset->p_res23_tb_table[i].p_data +
					(AM2_DEF_SNUM_LO * scan_num) + pixel_num) =
						0.01 * pixel_num + i + 1;
			}
			for (i = 0; i < p_dataset->res36_tb_num; i++)
			{
				*(p_dataset->p_res36_tb_table[i].p_data +
					(AM2_DEF_SNUM_LO * scan_num) + pixel_num) =
						0.01 * pixel_num + i + 1;
			}
			/* Area Mean Height */
			*(p_dataset->p_height + (AM2_DEF_SNUM_LO * scan_num) + pixel_num) =
				pixel_num;
			
			/* Sun Azimuth */
			*(p_dataset->p_sun_azimuth + (AM2_DEF_SNUM_LO * scan_num) +
				pixel_num) = 0.01 * pixel_num;
			
			/* Sun Elevation */
			*(p_dataset->p_sun_elevation + (AM2_DEF_SNUM_LO * scan_num) +
				pixel_num) = 0.01 * pixel_num;
			
			/* Earth Incidence */
			*(p_dataset->p_earth_incidence + (AM2_DEF_SNUM_LO * scan_num) +
				pixel_num) = 0.01 * pixel_num;
			
			/* Earth Azimuth */
			*(p_dataset->p_earth_azimuth + (AM2_DEF_SNUM_LO * scan_num) +
				pixel_num) = 0.01 * pixel_num;
			
			/* Land_Ocean Flag 6 to 36 */
			for (i = 0; i < 6; i++)
			{
				*(p_dataset->p_lof_low
					+ (i * scan_size * AM2_DEF_SNUM_LO)
					+ (scan_num * AM2_DEF_SNUM_LO)
					+ (pixel_num)) = (i + scan_num + pixel_num) % 0xFF;
			}
		}
		
		for (pixel_num = 0; pixel_num < AM2_DEF_SNUM_HI; pixel_num++)
		{
			/* Brightness Temperature (res High) */
			for (i = 0; i < p_dataset->res89_tb_num; i++)
			{
				*(p_dataset->p_res89_tb_table[i].p_data +
					(AM2_DEF_SNUM_HI * scan_num) + pixel_num) =
						0.01 * pixel_num + i + 1;
			}
			
			/* Latitude of Observation Point
			 * Longitude of Observation Point */
			(p_dataset->p_latlon89A + (AM2_DEF_SNUM_HI * scan_num)
				+ pixel_num)->lat = (pixel_num - 90) % 91;
			(p_dataset->p_latlon89A + (AM2_DEF_SNUM_HI * scan_num)
				+ pixel_num)->lon = (pixel_num - 180) % 181;
			(p_dataset->p_latlon89B + (AM2_DEF_SNUM_HI * scan_num)
				+ pixel_num)->lat = (pixel_num - 89) % 91;
			(p_dataset->p_latlon89B + (AM2_DEF_SNUM_HI * scan_num)
				+ pixel_num)->lon = (pixel_num - 179) % 181;
			
			/* Land_Ocean Flag 89 */
			for (i = 0; i < 2; i++)
			{
				*(p_dataset->p_lof_high
					+ (i * scan_size * AM2_DEF_SNUM_HI)
					+ (scan_num * AM2_DEF_SNUM_HI)
					+ (pixel_num)) = (i + scan_num + pixel_num) % 0xFF;
			}
		}
		
		/* Scan Data Quality */
		sample_data.quality_of_packet = scan_num;
		*(p_dataset->p_scan_quality + scan_num) = sample_data;
		
		/* Pixel Data Quality 6 to 36 */
		memset(p_dataset->p_pixel_quality_low + scan_num * (AM2_DEF_SNUM_LO * 2),
			scan_num % 0xFF, (AM2_DEF_SNUM_LO * 2));
		
		/* Pixel Data Quality 89 */
		memset(p_dataset->p_pixel_quality_high + scan_num * (AM2_DEF_SNUM_HI * 1),
			scan_num % 0xFF, (AM2_DEF_SNUM_HI * 1));
		
	}
	
	return;
}

