/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2010 All rights reserved.                  */
/*  File Name    : sample13_read_L1Rproduct.c                                */
/*  Abstract     : I/O toolkit sample program                                */
/*  Function List                                                            */
/*      main()                       : Sample program main                   */
/*      initialize()                 : Initialization                        */
/*      terminate()                  : Termination                           */
/*      get_value()                  : Get dataset value                     */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2011/01/26                                                    */
/*  Version    1.08                                                          */
/*  Notice                                                                   */
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/** Include AMTK.h */
#ifdef __cplusplus
extern "C" {
#endif
#include "AMTK.h"
#ifdef __cplusplus
}
#endif

#include "sample_common.h"

/*---------------------------------------------------------------------------*
 * Common definition                                                         *
 *---------------------------------------------------------------------------*/
 
/* PRINT_SCAN_NUM defines print data number */
#define PRINT_SCAN_NUM        (2)

/** Sample L1R product file name. */
#define FN_SAMPLE_PRODUCT_L1R ("../data/sample12_IOToolKit_l1r.h5")
/** Metadata definition. */
#define FN_SAMPLE_META_L1R    ("../data/metadata_sample_L1R.txt")

/** Number of scans. */
#define META_NUMSCAN_NAME     ("NumberOfScans")
/** Overlap scans. */
#define META_OVERLAP_NAME     ("OverlapScans")


/** Brightness Temperature data model. */
typedef struct
{
	/* Access Label. */
	const int access_label;
	/* Brightness Tempertature Data. */
	float *p_data;
	/* Dataset Name. */
	const char *p_dataset_name;
	
} TB_DATA;

/** Structure of dataset model. */
typedef struct
{
	/* Scan Time */
	AM2_COMMON_SCANTIME *p_scan_time;
	
	/* Position in Orbit */
	double *p_pos_orbit;
	/* Navigation Data */
	float *p_navi;
	/* Attitude Data */
	float *p_att;
	
	/* Brightness Temperature <06GHz resolution> */
	TB_DATA *p_res06_tb_table;
	int res06_tb_num;
	/* Brightness Temperature <10GHz resolution> */
	TB_DATA *p_res10_tb_table;
	int res10_tb_num;
	/* Brightness Temperature <23GHz resolution> */
	TB_DATA *p_res23_tb_table;
	int res23_tb_num;
	/* Brightness Temperature <36GHz resolution> */
	TB_DATA *p_res36_tb_table;
	int res36_tb_num;
	/* Brightness Temperature <89GHz resolution> */
	TB_DATA *p_res89_tb_table;
	int res89_tb_num;
	
	/* Sun Azimuth */
	float *p_sun_azimuth;
	/* Sun Elevation */
	float *p_sun_elevation;
	/* Earth Incidencet */
	float *p_earth_incidence;
	/* Earth Azimuth */
	float *p_earth_azimuth;
	/* Area Mean Height */
	float *p_height;
	/* Land_Ocean Flag 6 to 36 */
	int *p_lof_low;
	/* Land_Ocean Flag 896 */
	int *p_lof_high;
	
	/* Latitude of Observation Point for 89A
	 * Longitude of Observation Point for 89A */
	AM2_COMMON_LATLON *p_latlon89A;
	/* Latitude of Observation Point for 89B
	 * Longitude of Observation Point for 89B */
	AM2_COMMON_LATLON *p_latlon89B;
	/* Scan Data Quality */
	AMTK_SCAN_DATA_QUALITY *p_scan_quality;
	
	/* Pixel Data Quality 6 to 36 */
	unsigned char *p_pixel_quality_low;
	/* Pixel Data Quality 89 */
	unsigned char *p_pixel_quality_high;

	
} DATASET_MODEL;


/** Initialization.
 * @param file_id   [IN]  Product file id
 * @param scan_size [IN]  Number of scans
 * @param p_dataset [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(hid_t file_id, int scan_size, DATASET_MODEL *p_dataset);

/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset);

/** Get dataset value.
 * @param scan_size [IN] Number of scans
 * @param p_dataset [IN] Dataset
 */
static void get_value(int scan_size, const DATASET_MODEL *p_dataset);


/*---------------------------------------------------------------------------*
 * Main program                                                              *
 *---------------------------------------------------------------------------*/
 
/** AMSR2 I/O toolkit sample program.
 * sample13: input L1R proudct.
 * 
 * @param  argc [IN] Argument count
 * @param  argv [IN] Argument value
 * @return EXIT_SUCCESS
 * @return EXIT_FAILURE
 */
int main(int argc, char *argv[])
{
	hid_t file_id = 0;
	
	int i = 0;
	int meta_num = 0;
	int ret = 0;
	
	/* sample meta data read */
	char meta_name[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
	char meta_value[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
	
	char buffer[META_DATA_LEN_MAX] = "";
	char *p_value = NULL;
	
	int num_scans = 0;
	int num_overlap_scans = 0;
	int scan_size = 0;
	int scan_start = 0;
	int scan_end = 0;
	
	/* Dataset. */
	DATASET_MODEL dataset = {0};
	
	
	/* Brightness Temperature <6GHz resolution> */
	TB_DATA res06_tb_table[] =
	{
		{AM2_RES06_TB06V, NULL, "(res06,6.9GHz,V)" },
		{AM2_RES06_TB06H, NULL, "(res06,6.9GHz,H)" },
		{AM2_RES06_TB07V, NULL, "(res06,7.3GHz,V)" },
		{AM2_RES06_TB07H, NULL, "(res06,7.3GHz,H)" },
		{AM2_RES06_TB10V, NULL, "(res06,10.7GHz,V)"},
		{AM2_RES06_TB10H, NULL, "(res06,10.7GHz,H)"},
		{AM2_RES06_TB18V, NULL, "(res06,18.7GHz,V)"},
		{AM2_RES06_TB18H, NULL, "(res06,18.7GHz,H)"},
		{AM2_RES06_TB23V, NULL, "(res06,23.8GHz,V)"},
		{AM2_RES06_TB23H, NULL, "(res06,23.8GHz,H)"},
		{AM2_RES06_TB36V, NULL, "(res06,36.5GHz,V)"},
		{AM2_RES06_TB36H, NULL, "(res06,36.5GHz,H)"},
		{AM2_RES06_TB89V, NULL, "(res06,89.0GHz,V)"},
		{AM2_RES06_TB89H, NULL, "(res06,89.0GHz,H)"}
	};
	int res06_tb_num = sizeof(res06_tb_table) / sizeof(res06_tb_table[0]);
	
	/* Brightness Temperature <10GHz resolution> */
	TB_DATA res10_tb_table[] =
	{
		{AM2_RES10_TB10V, NULL, "(res10,10.7GHz,V)"},
		{AM2_RES10_TB10H, NULL, "(res10,10.7GHz,H)"},
		{AM2_RES10_TB18V, NULL, "(res10,18.7GHz,V)"},
		{AM2_RES10_TB18H, NULL, "(res10,18.7GHz,H)"},
		{AM2_RES10_TB23V, NULL, "(res10,23.8GHz,V)"},
		{AM2_RES10_TB23H, NULL, "(res10,23.8GHz,H)"},
		{AM2_RES10_TB36V, NULL, "(res10,36.5GHz,V)"},
		{AM2_RES10_TB36H, NULL, "(res10,36.5GHz,H)"},
		{AM2_RES10_TB89V, NULL, "(res10,89.0GHz,V)"},
		{AM2_RES10_TB89H, NULL, "(res10,89.0GHz,H)"}
	};
	
	int res10_tb_num = sizeof(res10_tb_table) / sizeof(res10_tb_table[0]);
	
	/* Brightness Temperature <23GHz resolution> */
	TB_DATA res23_tb_table[] =
	{
		{AM2_RES23_TB18V, NULL, "(res23,18.7GHz,V)"},
		{AM2_RES23_TB18H, NULL, "(res23,18.7GHz,H)"},
		{AM2_RES23_TB23V, NULL, "(res23,23.8GHz,V)"},
		{AM2_RES23_TB23H, NULL, "(res23,23.8GHz,H)"},
		{AM2_RES23_TB36V, NULL, "(res23,36.5GHz,V)"},
		{AM2_RES23_TB36H, NULL, "(res23,36.5GHz,H)"},
		{AM2_RES23_TB89V, NULL, "(res23,89.0GHz,V)"},
		{AM2_RES23_TB89H, NULL, "(res23,89.0GHz,H)"}
	};
	int res23_tb_num = sizeof(res23_tb_table) / sizeof(res23_tb_table[0]);
	
	/* Brightness Temperature <36GHz resolution> */
	TB_DATA res36_tb_table[] =
	{
		{AM2_RES36_TB36V, NULL, "(res36,36.5GHz,V)"},
		{AM2_RES36_TB36H, NULL, "(res36,36.5GHz,H)"},
		{AM2_RES36_TB89V, NULL, "(res36,89.0GHz,V)"},
		{AM2_RES36_TB89H, NULL, "(res36,89.0GHz,H)"}
	};
	int res36_tb_num = sizeof(res36_tb_table) / sizeof(res36_tb_table[0]);
	
	/* Brightness Temperature <89GHz resolution> */
	TB_DATA res89_tb_table[] =
	{
		{AM2_TB89AV, NULL, "(original,89GHz-A,V)"},
		{AM2_TB89AH, NULL, "(original,89GHz-A,H)"},
		{AM2_TB89BV, NULL, "(original,89GHz-B,V)"},
		{AM2_TB89BH, NULL, "(original,89GHz-B,H)"}
	};
	int res89_tb_num = sizeof(res89_tb_table) / sizeof(res89_tb_table[0]);
	
	/* Set Brightness Temperature data */
	dataset.p_res06_tb_table = res06_tb_table;
	dataset.res06_tb_num = res06_tb_num;
	
	dataset.p_res10_tb_table = res10_tb_table;
	dataset.res10_tb_num = res10_tb_num;
	
	dataset.p_res23_tb_table = res23_tb_table;
	dataset.res23_tb_num = res23_tb_num;
	
	dataset.p_res36_tb_table = res36_tb_table;
	dataset.res36_tb_num = res36_tb_num;
	
	dataset.p_res89_tb_table = res89_tb_table;
	dataset.res89_tb_num = res89_tb_num;
	
	
	/* HDF product file open */
	file_id = AMTK_openH5(FN_SAMPLE_PRODUCT_L1R);
	if (0 > file_id)
	{
		if (-501 == file_id)
		{
			printf("Product file not found.[%s]\n", FN_SAMPLE_PRODUCT_L1R);
		}
		else
		{
			E_MSG("AMTK_openH5() error.[%d]\n", ret);
		}
		exit(EXIT_FAILURE);
	}
	
	/*------------------------------------------------------------------------*
	 * memory allocation                                                      *
	 *------------------------------------------------------------------------*/
	p_value = buffer;
	
	/* Get "NumberOfScans" */
	ret = AMTK_getMetaDataName(file_id, META_NUMSCAN_NAME, &p_value);
	if (0 > ret)
	{
		E_MSG("AMTK_getMetaDataName() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	num_scans = atoi(buffer);
	
	/* Get "OverlapScans" */
	ret = AMTK_getMetaDataName(file_id, META_OVERLAP_NAME, &p_value);
	if (0 > ret)
	{
		E_MSG("AMTK_getMetaDataName() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	num_overlap_scans = atoi(buffer);
	
	/* Get scan size. */
	scan_size = num_scans + num_overlap_scans * 2;
	scan_start = 1 - num_overlap_scans;
	scan_end =  num_scans + num_overlap_scans;
	
	printf("Scan size = %d, Scan Start = %d, Scan End = %d\n", scan_size,
		scan_start, scan_end);
	
	/* memory allocation */
	ret = initialize(file_id, scan_size, &dataset);
	if (ret != RET_NORMAL)
	{
		E_MSG("initialize() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/*------------------------------------------------------------------------*
	 * input data                                                             *
	 *------------------------------------------------------------------------*/
	/* Read Metadata from Metadata file */
	if (RET_NORMAL != get_metadata_sample(FN_SAMPLE_META_L1R, meta_name,
		meta_value, &meta_num))
	{
		E_MSG("get_metadata_sample() error.\n");
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* meta data read */
	for (i = 0; i < meta_num; i++)
	{
		p_value = &meta_value[i][0];
		ret = AMTK_getMetaDataName(file_id, &meta_name[i][0], &p_value);
		if (0 > ret)
		{
			E_MSG("AMTK_getMetaDataName() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		printf("No=%d\tName=%s\tValue=%s\n", i, &meta_name[i][0], p_value);
	}
	
	/* Data set read */
	/* Scan Time */
	ret = AMTK_getScanTime(file_id, scan_start, scan_end, &dataset.p_scan_time);
	if (0 > ret)
	{
		E_MSG("AMTK_getScanTime() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Position in Orbit */
	ret = AMTK_get_SwathDouble(file_id, &dataset.p_pos_orbit, scan_start,
		scan_end, AM2_POS_ORBIT);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathDouble() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Navigation Data */
	ret = AMTK_get_SwathFloat(file_id, &dataset.p_navi, scan_start, scan_end,
		AM2_NAVI);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Attitude Data */
	ret = AMTK_get_SwathFloat(file_id, &dataset.p_att, scan_start, scan_end,
		AM2_ATT);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Brightness Temperature (res06,6.9GHz,V) ... (res06,89.0GHz,H) */
	for (i = 0; i < res06_tb_num; i++)
	{
		ret = AMTK_get_SwathFloat(file_id, &(res06_tb_table[i].p_data),
			scan_start, scan_end, res06_tb_table[i].access_label);
		if (0 > ret)
		{
			E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Brightness Temperature (res10,10.7GHz,V) ... (res10,89.0GHz,H) */
	for (i = 0; i < res10_tb_num; i++)
	{
		ret = AMTK_get_SwathFloat(file_id, &(res10_tb_table[i].p_data),
			scan_start, scan_end, res10_tb_table[i].access_label);
		if (0 > ret)
		{
			E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Brightness Temperature (res23,18.7GHz,V) ... (res23,89.0GHz,H) */
	for (i = 0; i < res23_tb_num; i++)
	{
		ret = AMTK_get_SwathFloat(file_id, &(res23_tb_table[i].p_data),
			scan_start, scan_end, res23_tb_table[i].access_label);
		if (0 > ret)
		{
			E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Brightness Temperature (res36,36.5GHz,V) ... (res36,89.0GHz,H) */
	for (i = 0; i < res36_tb_num; i++)
	{
		ret = AMTK_get_SwathFloat(file_id, &(res36_tb_table[i].p_data),
			scan_start, scan_end, res36_tb_table[i].access_label);
		if (0 > ret)
		{
			E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Brightness Temperature (original,89GHz-A,V) ... (original,89GHz-B,H) */
	for (i = 0; i < res89_tb_num; i++)
	{
		ret = AMTK_get_SwathFloat(file_id, &(res89_tb_table[i].p_data),
			scan_start, scan_end, res89_tb_table[i].access_label);
		if (0 > ret)
		{
			E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Latitude of Observation Point for 89A
	 * Longitude of Observation Point for 89A */
	ret = AMTK_getLatLon(file_id, &dataset.p_latlon89A, scan_start, scan_end,
		AM2_LATLON_RS_89A);
	if (0 > ret)
	{
		E_MSG("AMTK_getLatLon() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
		
	/* Latitude of Observation Point for 89B
	 * Longitude of Observation Point for 89B */
	ret = AMTK_getLatLon(file_id, &dataset.p_latlon89B, scan_start, scan_end,
		AM2_LATLON_RS_89B);
	if (0 > ret)
	{
		E_MSG("AMTK_getLatLon() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Sun Azimuth */
	ret = AMTK_get_SwathFloat(file_id, &dataset.p_sun_azimuth, scan_start,
		scan_end, AM2_SUN_AZ);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Sun Elevation */
	ret = AMTK_get_SwathFloat(file_id, &dataset.p_sun_elevation, scan_start,
		scan_end, AM2_SUN_EL);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Earth Incidence */
	ret = AMTK_get_SwathFloat(file_id, &dataset.p_earth_incidence, scan_start,
		scan_end, AM2_EARTH_INC);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Earth Azimuth */
	ret = AMTK_get_SwathFloat(file_id, &dataset.p_earth_azimuth, scan_start,
		scan_end, AM2_EARTH_AZ);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Area Mean Height */
	ret = AMTK_get_SwathFloat(file_id, &dataset.p_height, scan_start, scan_end,
		AM2_MEAN_HEIGHT);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Land_Ocean Flag 6 to 36 */
	ret = AMTK_get_SwathInt(file_id, &dataset.p_lof_low, scan_start, scan_end,
		AM2_LOF_RES_LO);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathInt() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Land_Ocean Flag 89 */
	ret = AMTK_get_SwathInt(file_id, &dataset.p_lof_high, scan_start, scan_end,
		AM2_LOF_RES_HI);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathInt() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Scan Data Quality */
	ret = AMTK_get_ScanDataQuality(file_id, &dataset.p_scan_quality, scan_start,
		scan_end, AM2_SCAN_QUAL);
	if (0 > ret)
	{
		E_MSG("AMTK_get_ScanDataQuality() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Pixel Data Quality 6 to 36 */
	ret = AMTK_get_SwathUChar(file_id, &dataset.p_pixel_quality_low, scan_start,
		scan_end, AM2_PIX_QUAL_LO);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathUChar() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Pixel Data Quality 89 */
	ret = AMTK_get_SwathUChar(file_id, &dataset.p_pixel_quality_high,
		scan_start, scan_end, AM2_PIX_QUAL_HI);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathUChar() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* get dataset value */
	get_value(scan_size, &dataset);
	
	/* termination */
	terminate(file_id, &dataset);
	
	printf("L1R product read end\n");
	
	exit(EXIT_SUCCESS);
}


/** Initialization.
 * @param file_id   [IN]  Product file id
 * @param scan_size [IN]  Number of scans
 * @param p_dataset [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(hid_t file_id, int scan_size, DATASET_MODEL *p_dataset)
{
	int ret = 0;
	int dimsize[3] = {0};
	int i = 0;

	/* memory allocation */
	
	/* Scan Time
	 * (type size * Number of scans) */
	/* get dimension size */
	ret = AMTK_getDimSize(file_id, AM2_SCAN_TIME_DEF, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_scan_time = (AM2_COMMON_SCANTIME *) malloc(
		sizeof(AM2_COMMON_SCANTIME) * get_memory_size(dimsize));
	if (NULL == p_dataset->p_scan_time)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Position in Orbit
	 * (type size * Number of scans) */
	ret = AMTK_getDimSize(file_id, AM2_POS_ORBIT, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_pos_orbit = (double *) malloc(sizeof(double) *
		get_memory_size(dimsize));
	if (NULL == p_dataset->p_pos_orbit)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Navigation Data
	 * (type size * Number of scans * 6) */
	ret = AMTK_getDimSize(file_id, AM2_NAVI, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_navi = (float *) malloc(sizeof(float) * get_memory_size(dimsize));
	if (NULL == p_dataset->p_navi)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Attitude Data
	 * (type size * Number of scans * 3) */
	ret = AMTK_getDimSize(file_id, AM2_ATT, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_att = (float *) malloc(sizeof(float) * get_memory_size(dimsize));
	if (NULL == p_dataset->p_att)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Brightness Temperature (res06,6.9GHz,V) ... (res06,89.0GHz,H)
	 * (type size * Number of scans * 243) */
	for (i = 0; i < p_dataset->res06_tb_num; i++)
	{
		ret = AMTK_getDimSize(file_id, p_dataset->p_res06_tb_table[i].access_label,
			dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
			return RET_ERROR;
		}
		p_dataset->p_res06_tb_table[i].p_data = (float *) malloc(sizeof(float) *
			get_memory_size(dimsize));
		if (NULL == p_dataset->p_res06_tb_table[i].p_data)
		{
			E_MSG("malloc() error.\n");
			return RET_ERROR;
		}
	}
	
	/* Brightness Temperature (res10,10.7GHz,V) ... (res10,89.0GHz,H)
	 * (type size * Number of scans * 243) */
	for (i = 0; i < p_dataset->res10_tb_num; i++)
	{
		ret = AMTK_getDimSize(file_id, p_dataset->p_res10_tb_table[i].access_label,
			dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
			return RET_ERROR;
		}
		p_dataset->p_res10_tb_table[i].p_data = (float *) malloc(sizeof(float) *
			get_memory_size(dimsize));
		if (NULL == p_dataset->p_res10_tb_table[i].p_data)
		{
			E_MSG("malloc() error.\n");
			return RET_ERROR;
		}
	}
	
	/* Brightness Temperature (res23,18.7GHz,V) ... (res23,89.0GHz,H)
	 * (type size * Number of scans * 243) */
	for (i = 0; i < p_dataset->res23_tb_num; i++)
	{
		ret = AMTK_getDimSize(file_id, p_dataset->p_res23_tb_table[i].access_label,
			dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
			return RET_ERROR;
		}
		p_dataset->p_res23_tb_table[i].p_data = (float *) malloc(sizeof(float) *
			get_memory_size(dimsize));
		if (NULL == p_dataset->p_res23_tb_table[i].p_data)
		{
			E_MSG("malloc() error.\n");
			return RET_ERROR;
		}
	}
	
	/* Brightness Temperature (res36,36.5GHz,V) ... (res36,89.0GHz,H)
	 * (type size * Number of scans * 243) */
	for (i = 0; i < p_dataset->res36_tb_num; i++)
	{
		ret = AMTK_getDimSize(file_id, p_dataset->p_res36_tb_table[i].access_label,
			dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
			return RET_ERROR;
		}
		p_dataset->p_res36_tb_table[i].p_data = (float *) malloc(sizeof(float) *
			get_memory_size(dimsize));
		if (NULL == p_dataset->p_res36_tb_table[i].p_data)
		{
			E_MSG("malloc() error.\n");
			return RET_ERROR;
		}
	}
	
	/* Brightness Temperature (original,89GHz-A,V) ... (original,89GHz-B,H)
	 * (type size * Number of scans * 486) */
	for (i = 0; i < p_dataset->res89_tb_num; i++)
	{
		ret = AMTK_getDimSize(file_id, p_dataset->p_res89_tb_table[i].access_label,
			dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
			return RET_ERROR;
		}
		p_dataset->p_res89_tb_table[i].p_data = (float *) malloc(sizeof(float) *
			get_memory_size(dimsize));
		if (NULL == p_dataset->p_res89_tb_table[i].p_data)
		{
			E_MSG("malloc() error.\n");
			return RET_ERROR;
		}
	}
	
	/* Latitude of Observation Point for 89A
	 * Longitude of Observation Point for 89A
	 * (type size * Number of scans * 486) */
	ret = AMTK_getDimSize(file_id, AM2_LATLON_RS_89A, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_latlon89A = (AM2_COMMON_LATLON *) malloc(
		sizeof(AM2_COMMON_LATLON) * get_memory_size(dimsize));
	if (NULL == p_dataset->p_latlon89A)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Latitude of Observation Point for 89B
	 * Longitude of Observation Point for 89B
	 * (type size * Number of scans * 486) */
	ret = AMTK_getDimSize(file_id, AM2_LATLON_RS_89B, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_latlon89B = (AM2_COMMON_LATLON *) malloc(
		sizeof(AM2_COMMON_LATLON) * get_memory_size(dimsize));
	if (NULL == p_dataset->p_latlon89B)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Area Mean Height
	 * (type size * Number of scans * 243) */
	ret = AMTK_getDimSize(file_id, AM2_MEAN_HEIGHT, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_height = (float *) malloc(sizeof(float) *
		get_memory_size(dimsize));
	if (NULL == p_dataset->p_height)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Sun Azimuth
	 * (type size * Number of scans * 243) */
	ret = AMTK_getDimSize(file_id, AM2_SUN_AZ, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_sun_azimuth = (float *) malloc(sizeof(float) *
		get_memory_size(dimsize));
	if (NULL == p_dataset->p_sun_azimuth)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Sun Elevation
	 * (type size * Number of scans * 243) */
	ret = AMTK_getDimSize(file_id, AM2_SUN_EL, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_sun_elevation = (float *) malloc(sizeof(float) *
		get_memory_size(dimsize));
	if (NULL == p_dataset->p_sun_elevation)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Earth Incidence
	 * (type size * Number of scans * 243) */
	ret = AMTK_getDimSize(file_id, AM2_EARTH_INC, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_earth_incidence = (float *) malloc(sizeof(float) *
		get_memory_size(dimsize));
	if (NULL == p_dataset->p_earth_incidence)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Earth Azimuth
	 * (type size * Number of scans * 243) */
	ret = AMTK_getDimSize(file_id, AM2_EARTH_AZ, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_earth_azimuth = (float *) malloc(sizeof(float) *
		get_memory_size(dimsize));
	if (NULL == p_dataset->p_earth_azimuth)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	
	/* Land_Ocean Flag 6 to 36
	 * (type size * 6 * Number of scans * 243) */
	ret = AMTK_getDimSize(file_id, AM2_LOF_RES_LO, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_lof_low = (int *) malloc(sizeof(int) * get_memory_size(dimsize));
	if (NULL == p_dataset->p_lof_low)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Land_Ocean Flag 89
	 * (type size * 2 * Number of scans * 486) */
	ret = AMTK_getDimSize(file_id, AM2_LOF_RES_HI, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_lof_high = (int *) malloc(sizeof(int) * get_memory_size(dimsize));
	if (NULL == p_dataset->p_lof_high)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Scan Data Quality
	 * (type size * Number of scans) */
	ret = AMTK_getDimSize(file_id, AM2_SCAN_QUAL, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_scan_quality = (AMTK_SCAN_DATA_QUALITY *) malloc(
		sizeof(AMTK_SCAN_DATA_QUALITY) * dimsize[0]);
	if (NULL == p_dataset->p_scan_quality)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Pixel Data Quality 6 to 36
	 * (type size * Numbef of scans * 486[243 * binary 2byte]) */
	ret = AMTK_getDimSize(file_id, AM2_PIX_QUAL_LO, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_pixel_quality_low = (unsigned char *) malloc(
		get_memory_size(dimsize));
	if (NULL == p_dataset->p_pixel_quality_low)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}

	/* Pixel Data Quality 89
	 * (type size * Numbef of scans * 486[486 * binary 1byte]) */
	ret = AMTK_getDimSize(file_id, AM2_PIX_QUAL_HI, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_pixel_quality_high = (unsigned char *) malloc(
		get_memory_size(dimsize));
	if (NULL == p_dataset->p_pixel_quality_high)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	return RET_NORMAL;
}


/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset)
{
	int i = 0;
	int ret = 0;
	
	if (0 <= file_id)
	{
		/* HDF product file close */
		ret = AMTK_closeH5(file_id);
		if (0 > ret)
		{
			E_MSG("AMTK_closeH5() error.[%d]\n", ret);
		}
	}
	
	/* free memory */
	free(p_dataset->p_scan_time);
	p_dataset->p_scan_time = NULL;
	
	free(p_dataset->p_pos_orbit);
	p_dataset->p_pos_orbit = NULL;
	
	free(p_dataset->p_navi);
	p_dataset->p_navi = NULL;
	
	free(p_dataset->p_att);
	p_dataset->p_att = NULL;
	
	for (i = 0; i < p_dataset->res06_tb_num; i++)
	{
		free(p_dataset->p_res06_tb_table[i].p_data);
		p_dataset->p_res06_tb_table[i].p_data = NULL;
	}
	for (i = 0; i < p_dataset->res10_tb_num; i++)
	{
		free(p_dataset->p_res10_tb_table[i].p_data);
		p_dataset->p_res10_tb_table[i].p_data = NULL;
	}
	for (i = 0; i < p_dataset->res23_tb_num; i++)
	{
		free(p_dataset->p_res23_tb_table[i].p_data);
		p_dataset->p_res23_tb_table[i].p_data = NULL;
	}
	for (i = 0; i < p_dataset->res36_tb_num; i++)
	{
		free(p_dataset->p_res36_tb_table[i].p_data);
		p_dataset->p_res36_tb_table[i].p_data = NULL;
	}
	for (i = 0; i < p_dataset->res89_tb_num; i++)
	{
		free(p_dataset->p_res89_tb_table[i].p_data);
		p_dataset->p_res89_tb_table[i].p_data = NULL;
	}
	
	free(p_dataset->p_latlon89A);
	p_dataset->p_latlon89A = NULL;
	
	free(p_dataset->p_latlon89B);
	p_dataset->p_latlon89B = NULL;
	
	free(p_dataset->p_height);
	p_dataset->p_height = NULL;
	
	free(p_dataset->p_sun_azimuth);
	p_dataset->p_sun_azimuth = NULL;
	
	free(p_dataset->p_sun_elevation);
	p_dataset->p_sun_elevation = NULL;
	
	free(p_dataset->p_earth_incidence);
	p_dataset->p_earth_incidence = NULL;
	
	free(p_dataset->p_earth_azimuth);
	p_dataset->p_earth_azimuth = NULL;
	
	free(p_dataset->p_scan_quality);
	p_dataset->p_scan_quality = NULL;
	
	free(p_dataset->p_lof_low);
	p_dataset->p_lof_low = NULL;
	
	free(p_dataset->p_lof_high);
	p_dataset->p_lof_high = NULL;
	
	free(p_dataset->p_pixel_quality_low);
	p_dataset->p_pixel_quality_low = NULL;
	
	free(p_dataset->p_pixel_quality_high);
	p_dataset->p_pixel_quality_high = NULL;
	
	return;
}


/** Get dataset value.
 * @param scan_size [IN] Number of scans
 * @param p_dataset [IN] Dataset
 */
static void get_value(int scan_size, const DATASET_MODEL *p_dataset)
{
	int i = 0;
	int scan_num = 0;
	int pixel_num = 0;
	
	for (scan_num = 0; scan_num < PRINT_SCAN_NUM; scan_num++)
	{
		/* Scan Time */
		printf("Scan Time No=%d [%d]   [%02d/%02d]  %02d:%02d:%02d.%03d\n",
			scan_num,
			(p_dataset->p_scan_time + scan_num)->year,
			(p_dataset->p_scan_time + scan_num)->month,
			(p_dataset->p_scan_time + scan_num)->day,
			(p_dataset->p_scan_time + scan_num)->hour,
			(p_dataset->p_scan_time + scan_num)->minute,
			(p_dataset->p_scan_time + scan_num)->second,
			(p_dataset->p_scan_time + scan_num)->ms);
		
		/* Brightness Temperature */
		for (i = 0; i < p_dataset->res06_tb_num; i++)
		{
			/* first pixel */
			pixel_num = 0;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Brightness Temperature %s = %8.2f  \n",
				p_dataset->p_res06_tb_table[i].p_dataset_name,
				*(p_dataset->p_res06_tb_table[i].p_data
					+ (AM2_DEF_SNUM_LO * scan_num)
					+ pixel_num));
			
			/* last pixel */
			pixel_num = AM2_DEF_SNUM_LO - 1;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Brightness Temperature %s = %8.2f  \n",
				p_dataset->p_res06_tb_table[i].p_dataset_name,
				*(p_dataset->p_res06_tb_table[i].p_data
					+ (AM2_DEF_SNUM_LO * scan_num)
					+ pixel_num));
		}
		
		for (i = 0; i < p_dataset->res10_tb_num; i++)
		{
			/* first pixel */
			pixel_num = 0;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Brightness Temperature %s = %8.2f  \n",
				p_dataset->p_res10_tb_table[i].p_dataset_name,
				*(p_dataset->p_res10_tb_table[i].p_data
					+ (AM2_DEF_SNUM_LO * scan_num)
					+ pixel_num));
			
			/* last pixel */
			pixel_num = AM2_DEF_SNUM_LO - 1;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Brightness Temperature %s = %8.2f  \n",
				p_dataset->p_res10_tb_table[i].p_dataset_name,
				*(p_dataset->p_res10_tb_table[i].p_data
					+ (AM2_DEF_SNUM_LO * scan_num)
					+ pixel_num));
		}
		
		for (i = 0; i < p_dataset->res23_tb_num; i++)
		{
			/* first pixel */
			pixel_num = 0;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Brightness Temperature %s = %8.2f  \n",
				p_dataset->p_res23_tb_table[i].p_dataset_name,
				*(p_dataset->p_res23_tb_table[i].p_data
					+ (AM2_DEF_SNUM_LO * scan_num)
					+ pixel_num));
			
			/* last pixel */
			pixel_num = AM2_DEF_SNUM_LO - 1;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Brightness Temperature %s = %8.2f  \n",
				p_dataset->p_res23_tb_table[i].p_dataset_name,
				*(p_dataset->p_res23_tb_table[i].p_data
					+ (AM2_DEF_SNUM_LO * scan_num)
					+ pixel_num));
		}
		
		for (i = 0; i < p_dataset->res36_tb_num; i++)
		{
			/* first pixel */
			pixel_num = 0;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Brightness Temperature %s = %8.2f  \n",
				p_dataset->p_res36_tb_table[i].p_dataset_name,
				*(p_dataset->p_res36_tb_table[i].p_data
					+ (AM2_DEF_SNUM_LO * scan_num)
					+ pixel_num));
			
			/* last pixel */
			pixel_num = AM2_DEF_SNUM_LO - 1;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Brightness Temperature %s = %8.2f  \n",
				p_dataset->p_res36_tb_table[i].p_dataset_name,
				*(p_dataset->p_res36_tb_table[i].p_data
					+ (AM2_DEF_SNUM_LO * scan_num)
					+ pixel_num));
		}
		
		for (i = 0; i < p_dataset->res89_tb_num; i++)
		{
			/* first pixel */
			pixel_num = 0;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Brightness Temperature %s = %8.2f  \n",
				p_dataset->p_res89_tb_table[i].p_dataset_name,
				*(p_dataset->p_res89_tb_table[i].p_data
					+ (AM2_DEF_SNUM_HI * scan_num)
					+ pixel_num));
			
			/* last pixel */
			pixel_num = AM2_DEF_SNUM_HI - 1;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Brightness Temperature %s = %8.2f  \n",
				p_dataset->p_res89_tb_table[i].p_dataset_name,
				*(p_dataset->p_res89_tb_table[i].p_data
					+ (AM2_DEF_SNUM_HI * scan_num)
					+ pixel_num));
		}
		
		/* Latitude/Longitude of Observation Point for 89A */
		printf("Latitude/Longitude of Observation Point for 89A\n" );
		/* first pixel */
		pixel_num = 0;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Latitude  = %8.2f  \n",
			(p_dataset->p_latlon89A + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lat);
		printf("Longitude = %8.2f  \n",
			(p_dataset->p_latlon89A + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lon);
		
		/* last pixel */
		pixel_num = AM2_DEF_SNUM_HI - 1;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Latitude  = %8.2f  \n",
			(p_dataset->p_latlon89A + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lat);
		printf("Longitude = %8.2f  \n",
			(p_dataset->p_latlon89A + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lon);
		
		/* Latitude/Longitude of Observation Point for 89B */
		printf("Latitude/Longitude of Observation Point for 89B\n" );
		/* first pixel */
		pixel_num = 0;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Latitude  = %8.2f  \n",
			(p_dataset->p_latlon89B + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lat);
		printf("Longitude = %8.2f  \n",
			(p_dataset->p_latlon89B + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lon);
		
		/* last pixel */
		pixel_num = AM2_DEF_SNUM_HI - 1;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Latitude  = %8.2f  \n",
			(p_dataset->p_latlon89B + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lat);
		printf("Longitude = %8.2f  \n",
			(p_dataset->p_latlon89B + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lon);
		
		/* Land_Ocean Flag 6 to 36 */
		printf("Land_Ocean Flag 6 to 36\n" );
		pixel_num = 0;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Land_Ocean Flag 06GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (0 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		printf("Land_Ocean Flag 07GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (1 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		printf("Land_Ocean Flag 10GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (2 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		printf("Land_Ocean Flag 18GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (3 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		printf("Land_Ocean Flag 23GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (4 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		printf("Land_Ocean Flag 36GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (5 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		
		/* last pixel */
		pixel_num = AM2_DEF_SNUM_LO - 1;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Land_Ocean Flag 06GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (0 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		printf("Land_Ocean Flag 07GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (1 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		printf("Land_Ocean Flag 10GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (2 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		printf("Land_Ocean Flag 18GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (3 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		printf("Land_Ocean Flag 23GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (4 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		printf("Land_Ocean Flag 36GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (5 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		
		/* Land_Ocean Flag 89 */
		/* first pixel */
		pixel_num = 0;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Land_Ocean Flag 89AGHz = 0x%02x\n",
			*(p_dataset->p_lof_high + (0 * scan_size * AM2_DEF_SNUM_HI) +
				(scan_num * AM2_DEF_SNUM_HI) + pixel_num));
		printf("Land_Ocean Flag 89BGHz = 0x%02x\n",
			*(p_dataset->p_lof_high + (1 * scan_size * AM2_DEF_SNUM_HI) +
				(scan_num * AM2_DEF_SNUM_HI) + pixel_num));
		
		/* last pixel */
		pixel_num = AM2_DEF_SNUM_HI - 1;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Land_Ocean Flag 89AGHz = 0x%02x\n",
			*(p_dataset->p_lof_high + (0 * scan_size * AM2_DEF_SNUM_HI) +
				(scan_num * AM2_DEF_SNUM_HI) + pixel_num));
		printf("Land_Ocean Flag 89BGHz = 0x%02x\n",
			*(p_dataset->p_lof_high + (1 * scan_size * AM2_DEF_SNUM_HI) +
				(scan_num * AM2_DEF_SNUM_HI) + pixel_num));
	}
	
	/* The following data will not print, so each function use same memory area.*/
	/* Just show as access example */
	
	return;
}

