/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2010 All rights reserved.                  */
/*  File Name    : sample2_make_L1Bproduct.c                                 */
/*  Abstract     : I/O toolkit sample program                                */
/*  Function List                                                            */
/*      main()                       : Sample program main                   */
/*      initialize()                 : Initialization                        */
/*      terminate()                  : Termination                           */
/*      set_value()                  : Set dataset value                     */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2012/12/03                                                    */
/*  Version    1.11                                                          */
/*  Notice                                                                   */
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/** Include AMTK.h */
#ifdef __cplusplus
extern "C" {
#endif
#include "AMTK.h"
#ifdef __cplusplus
}
#endif

#include "sample_common.h"

/*---------------------------------------------------------------------------*
 * Common definition                                                         *
 *---------------------------------------------------------------------------*/
 
/** Sample L1B product file name. */
#define FN_SAMPLE_PRODUCT_L1B ("../data/sample2_IOToolKit_l1b.h5")
/** Metadata definition. */
#define FN_SAMPLE_META_L1B    ("../data/metadata_sample_L1B.txt")

/** Number of scans. */
#define SCENE_SCAN_NUM        (2000)
/** Overlap scans. */
#define OVERLAP_SCAN_NUM      (20)


/** Brightness Temperature data model. */
typedef struct
{
	/* Access Label. */
	int access_label;
	/* Brightness Tempertature Data. */
	float *p_data;
	
} TB_DATA;

/** Structure of dataset model. */
typedef struct
{
	/* Scan Time */
	AM2_COMMON_SCANTIME *p_scan_time;
	
	/* Position in Orbit */
	double *p_pos_orbit;
	/* Navigation Data */
	float *p_navi;
	/* Attitude Data */
	float *p_att;
	
	/* BrightHotness Temperature <Low resolution> */
	TB_DATA *p_low_tb_table;
	int low_tb_num;
	/* Brightness Temperature <High resolution>  */
	TB_DATA *p_high_tb_table;
	int high_tb_num;
	
	/* Hot Load Count 6 to 36 */
	int *p_hts_low;
	/* Hot Load Count 89 */
	int *p_hts_high;
	/* Cold Sky Mirror Count 6 to 36 */
	int *p_csm_low;
	/* Cold Sky Mirror Count 89 */
	int *p_csm_high;
	/* Rx Offset_Gain Count */
	int *p_rx_offset_gain;
	
	/* Latitude of Observation Point for 89A
	 * Longitude of Observation Point for 89A */
	AM2_COMMON_LATLON *p_latlon89A;
	/* Latitude of Observation Point for 89B
	 * Longitude of Observation Point for 89B */
	AM2_COMMON_LATLON *p_latlon89B;
	
	/* Sun Azimuth */
	float *p_sun_azimuth;
	/* Sun Elevation */
	float *p_sun_elevation;
	/* Earth Incidencet */
	float *p_earth_incidence;
	/* Earth Azimuth */
	float *p_earth_azimuth;
	/* Land_Ocean Flag 6 to 36 */
	int *p_lof_low;
	/* Land_Ocean Flag 89 */
	int *p_lof_high;
	/* Observation Supplement */
	unsigned char *p_obs_spl;
	/* SPC Temperature Count */
	float *p_spc_temp;
	/* SPS Temperature Count */
	float *p_sps_temp;
	/* PCD Data */
	unsigned char *p_pcd_data;
	
	/* Scan Data Quality */
	AMTK_SCAN_DATA_QUALITY *p_scan_quality;
	
	/* Pixel Data Quality 6 to 36 */
	unsigned char *p_pixel_quality_low;
	/* Pixel Data Quality 89 */
	unsigned char *p_pixel_quality_high;
	/* Interpolation Flag 6 to 36 */
	unsigned char *p_intpol_low;
	/* Interpolation Flag 89 */
	unsigned char *p_intpol_high;
	
} DATASET_MODEL;


/** Initialization.
 * @param scan_size      [IN]  Number of scans
 * @param p_dataset      [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(int scan_size, DATASET_MODEL *p_dataset);

/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset);

/** Set dataset value.
 * @param scan_size      [IN]  Number of scans
 * @param p_dataset      [OUT] Dataset
 */
static void set_value(int scan_size, DATASET_MODEL *p_dataset);


/*---------------------------------------------------------------------------*
 * Main program                                                              *
 *---------------------------------------------------------------------------*/
 
/** AMSR2 I/O toolkit sample program.
 * sample2: output L1B proudct.
 * 
 * @param  argc [IN] Argument count
 * @param  argv [IN] Argument value
 * @return EXIT_SUCCESS
 * @return EXIT_FAILURE
 */
int main(int argc, char *argv[])
{
	hid_t file_id = 0;
	
	int i = 0;
	int meta_num = 0;
	int dimsize[3] = {0};
	int ret = 0;
	
	/* sample meta data read */
	char meta_name[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
	char meta_value[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
	
	/* Number of Scans */
	const int scan_size = SCENE_SCAN_NUM + OVERLAP_SCAN_NUM * 2;
	const int scan_start = 1 - OVERLAP_SCAN_NUM;
	const int scan_end = SCENE_SCAN_NUM + OVERLAP_SCAN_NUM;
	
	/* Dataset. */
	DATASET_MODEL dataset = {0};
	
	/* Brightness Temperature <Low resolution> */
	TB_DATA low_tb_table[] =
	{
		{AM2_TB06V, NULL},
		{AM2_TB06H, NULL},
		{AM2_TB07V, NULL},
		{AM2_TB07H, NULL},
		{AM2_TB10V, NULL},
		{AM2_TB10H, NULL},
		{AM2_TB18V, NULL},
		{AM2_TB18H, NULL},
		{AM2_TB23V, NULL},
		{AM2_TB23H, NULL},
		{AM2_TB36V, NULL},
		{AM2_TB36H, NULL},
	};
	int low_tb_num = sizeof(low_tb_table) / sizeof(low_tb_table[0]);

	/* Brightness Temperature <High resolution> */
	TB_DATA high_tb_table[] =
	{
		{AM2_TB89AV, NULL},
		{AM2_TB89AH, NULL},
		{AM2_TB89BV, NULL},
		{AM2_TB89BH, NULL},
	};
	int high_tb_num = sizeof(high_tb_table) / sizeof(high_tb_table[0]);
	
	/* Set Brightness Temperature data */
	dataset.p_low_tb_table = low_tb_table;
	dataset.low_tb_num = low_tb_num;
	
	dataset.p_high_tb_table = high_tb_table;
	dataset.high_tb_num = high_tb_num;
	
	/* memory allocation */
	ret = initialize(scan_size, &dataset);
	if (ret != RET_NORMAL)
	{
		E_MSG("initialize() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* set dataset value */
	set_value(scan_size, &dataset);
	
	/*------------------------------------------------------------------------*
	 * output data                                                            *
	 *------------------------------------------------------------------------*/
	/* HDF product file open (write mode) */
	file_id = AMTK_openH5_Write(FN_SAMPLE_PRODUCT_L1B, AM2_CREATE_MODE);
	if (0 > file_id)
	{
		E_MSG("AMTK_openH5_Write() error.[%d]\n", file_id);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Read Metadata from Metadata file */
	if (RET_NORMAL != get_metadata_sample(FN_SAMPLE_META_L1B, meta_name,
		meta_value, &meta_num))
	{
		E_MSG("get_metadata_sample() error.\n");
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Metadata write */
	for (i = 0; i < meta_num; i++) 
	{
		ret = AMTK_setMetaDataName(file_id, &meta_name[i][0],
			&meta_value[i][0]);
		if (0 > ret)
		{
			E_MSG("AMTK_setMetaDataName() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Data set write */
	/* Scan Time */
	{
		dimsize[0] = scan_size;
		dimsize[1] = 0;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_SCAN_TIME_DEF, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_setScanTime(file_id, dataset.p_scan_time, scan_start,
			scan_end);
		if (0 > ret)
		{
			E_MSG("AMTK_setScanTime() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Position in Orbit */
	{
		dimsize[0] = scan_size;
		dimsize[1] = 0;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_POS_ORBIT, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathDouble(file_id, dataset.p_pos_orbit, scan_start,
			scan_end, AM2_POS_ORBIT);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathDouble() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Navigation Data */
	{
		dimsize[0] = scan_size;
		dimsize[1] = 6;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_NAVI, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathFloat(file_id, dataset.p_navi, scan_start, scan_end,
			AM2_NAVI);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Attitude Data */
	{
		dimsize[0] = scan_size;
		dimsize[1] = 3;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_ATT, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathFloat(file_id, dataset.p_att, scan_start, scan_end,
			AM2_ATT);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Brightness Temperature (6.9GHz,V) ... (36.5GHz,H) */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_LO;
		dimsize[2] = 0;
		for (i = 0; i < low_tb_num; i++)
		{
			ret = AMTK_setDimSize(file_id, low_tb_table[i].access_label,
				dimsize);
			if (0 > ret)
			{
				E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
				terminate(file_id, &dataset);
				exit(EXIT_FAILURE);
			}
			
			ret = AMTK_set_SwathFloat(file_id, low_tb_table[i].p_data,
				scan_start, scan_end, low_tb_table[i].access_label);
			if (0 > ret)
			{
				E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
				terminate(file_id, &dataset);
				exit(EXIT_FAILURE);
			}
		}
	}
	
	/* Brightness Temperature (89.0GHz-A,V) ... (89.0GHz-B,H) */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_HI;
		dimsize[2] = 0;
		for (i = 0; i < high_tb_num; i++)
		{
			ret = AMTK_setDimSize(file_id, high_tb_table[i].access_label,
				dimsize);
			if (0 > ret)
			{
				E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
				terminate(file_id, &dataset);
				exit(EXIT_FAILURE);
			}
			
			ret = AMTK_set_SwathFloat(file_id, high_tb_table[i].p_data,
				scan_start, scan_end, high_tb_table[i].access_label);
			if (0 > ret)
			{
				E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
				terminate(file_id, &dataset);
				exit(EXIT_FAILURE);
			}
		}
	}
	
	/* Hot Load Count 6 to 36 */
	{
		dimsize[0] = 12;
		dimsize[1] = scan_size;
		dimsize[2] = 16;
		ret = AMTK_setDimSize(file_id, AM2_HTS_LO, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathInt(file_id, dataset.p_hts_low, scan_start, 
			scan_end, AM2_HTS_LO);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathInt() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Hot Load Count 89 */
	{
		dimsize[0] = 4;
		dimsize[1] = scan_size;
		dimsize[2] = 32;
		ret = AMTK_setDimSize(file_id, AM2_HTS_HI, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathInt(file_id, dataset.p_hts_high, scan_start, 
			scan_end, AM2_HTS_HI);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathInt() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Cold Sky Mirror Count 6 to 36 */
	{
		dimsize[0] = 12;
		dimsize[1] = scan_size;
		dimsize[2] = 16;
		ret = AMTK_setDimSize(file_id, AM2_CSM_LO, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathInt(file_id, dataset.p_csm_low, scan_start, 
			scan_end, AM2_CSM_LO);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathInt() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Cold Sky Mirror Count 89 */
	{
		dimsize[0] = 4;
		dimsize[1] = scan_size;
		dimsize[2] = 32;
		ret = AMTK_setDimSize(file_id, AM2_CSM_HI, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathInt(file_id, dataset.p_csm_high, scan_start, 
			scan_end, AM2_CSM_HI);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathInt() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Rx Offset_Gain Count */
	{
		dimsize[0] = scan_size;
		dimsize[1] = 32;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_OFF_GAIN, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathInt(file_id, dataset.p_rx_offset_gain, scan_start, 
			scan_end, AM2_OFF_GAIN);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathInt() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Latitude of Observation Point for 89A
	 * Longitude of Observation Point for 89A */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_HI;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_LATLON_89A, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_setLatLon(file_id, dataset.p_latlon89A, scan_start, scan_end,
			AM2_LATLON_89A);
		if (0 > ret)
		{
			E_MSG("AMTK_setLatLon() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Latitude of Observation Point for 89B
	 * Longitude of Observation Point for 89B */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_HI;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_LATLON_89B, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_setLatLon(file_id, dataset.p_latlon89B, scan_start, scan_end,
			AM2_LATLON_89B);
		if (0 > ret)
		{
			E_MSG("AMTK_setLatLon() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}

	/* Sun Azimuth */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_LO;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_SUN_AZ, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathFloat(file_id, dataset.p_sun_azimuth, scan_start,
			scan_end, AM2_SUN_AZ);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Sun Elevation */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_LO;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_SUN_EL, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathFloat(file_id, dataset.p_sun_elevation, scan_start,
			scan_end, AM2_SUN_EL);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Earth Incidence */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_LO;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_EARTH_INC, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathFloat(file_id, dataset.p_earth_incidence,
			scan_start, scan_end, AM2_EARTH_INC);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Earth Azimuth */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_LO;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_EARTH_AZ, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathFloat(file_id, dataset.p_earth_azimuth, scan_start,
			scan_end, AM2_EARTH_AZ);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Land_Ocean Flag 6 to 36 */
	{
		dimsize[0] = 6;
		dimsize[1] = scan_size;
		dimsize[2] = AM2_DEF_SNUM_LO;
		ret = AMTK_setDimSize(file_id, AM2_LOF_LO, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathInt(file_id, dataset.p_lof_low, scan_start,
			scan_end, AM2_LOF_LO);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathInt() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Land_Ocean Flag 89 */
	{
		dimsize[0] = 2;
		dimsize[1] = scan_size;
		dimsize[2] = AM2_DEF_SNUM_HI;
		ret = AMTK_setDimSize(file_id, AM2_LOF_HI, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathInt(file_id, dataset.p_lof_high, scan_start,
			scan_end, AM2_LOF_HI);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathInt() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Observation Supplement */
	{
		dimsize[0] = scan_size;
		dimsize[1] = 124 * 2; /* binary (2byte) */
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_OB_SPL, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathUChar(file_id, dataset.p_obs_spl, scan_start,
			scan_end, AM2_OB_SPL);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathUChar() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* SPC Temperature Count */
	{
		dimsize[0] = scan_size;
		dimsize[1] = 34;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_SPC_TEMP, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathFloat(file_id, dataset.p_spc_temp, scan_start,
			scan_end, AM2_SPC_TEMP);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* SPS Temperature Count */
	{
		dimsize[0] = scan_size;
		dimsize[1] = 46;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_SPS_TEMP, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathFloat(file_id, dataset.p_sps_temp, scan_start,
			scan_end, AM2_SPS_TEMP);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* PCD Data */
	{
		dimsize[0] = scan_size;
		dimsize[1] = 64;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_PCD, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathUChar(file_id, dataset.p_pcd_data, scan_start,
			scan_end, AM2_PCD);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathUChar() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Scan Data Quality */
	{
		dimsize[0] = scan_size;
		dimsize[1] = sizeof(AMTK_SCAN_DATA_QUALITY);
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_SCAN_QUAL, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_ScanDataQuality(file_id, dataset.p_scan_quality,
			scan_start, scan_end, AM2_SCAN_QUAL);
		if (0 > ret)
		{
			E_MSG("AMTK_set_ScanDataQuality() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Pixel Data Quality 6 to 36 */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_LO * 2; /* binary (2byte) */
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_PIX_QUAL_LO, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathUChar(file_id, dataset.p_pixel_quality_low,
			scan_start, scan_end, AM2_PIX_QUAL_LO);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathUChar() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Pixel Data Quality 89 */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_HI;	 /* binary (1byte) */
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_PIX_QUAL_HI, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathUChar(file_id, dataset.p_pixel_quality_high,
			scan_start, scan_end, AM2_PIX_QUAL_HI);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathUChar() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Interpolation Flag 6 to 36 */
	{
		dimsize[0] = 12;
		dimsize[1] = scan_size;
		dimsize[2] = 16;
		ret = AMTK_setDimSize(file_id, AM2_INTPL_LO, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathUChar(file_id, dataset.p_intpol_low, scan_start,
			scan_end, AM2_INTPL_LO);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathUChar() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Interpolation Flag 89 */
	{
		dimsize[0] = 4;
		dimsize[1] = scan_size;
		dimsize[2] = 32;
		ret = AMTK_setDimSize(file_id, AM2_INTPL_HI, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathUChar(file_id, dataset.p_intpol_high, scan_start,
			scan_end, AM2_INTPL_HI);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathUChar() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* termination */
	terminate(file_id, &dataset);
	
	printf("L1B product created\n");
	
	exit(EXIT_SUCCESS);
}


/** Initialization.
 * @param [IN]  scan_size Number of scans
 * @param [OUT] p_dataset Dataset pointers
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(int scan_size, DATASET_MODEL *p_dataset)
{
	int i = 0;
	
	/* memory allocation */
	
	/* Scan Time
	 * (type size * Number of scans) */
	p_dataset->p_scan_time = (AM2_COMMON_SCANTIME *) malloc(
		sizeof(AM2_COMMON_SCANTIME) * scan_size);
	if (NULL == p_dataset->p_scan_time)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Position in Orbit
	 * (type size * Number of scans) */
	p_dataset->p_pos_orbit = (double *) malloc(sizeof(double) * scan_size);
	if (NULL == p_dataset->p_pos_orbit)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Navigation Data
	 * (type size * Number of scans * 6) */
	p_dataset->p_navi = (float *) malloc(sizeof(float) * scan_size * 6);
	if (NULL == p_dataset->p_navi)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Attitude Data
	 * (type size * Number of scans * 3) */
	p_dataset->p_att = (float *) malloc(sizeof(float) * scan_size * 3);
	if (NULL == p_dataset->p_att)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Brightness Temperature (6.9GHz,V) ... (36.5GHz,H)
	 * (type size * Number of scans * 243) */
	for (i = 0; i < p_dataset->low_tb_num; i++)
	{
		p_dataset->p_low_tb_table[i].p_data =
			(float *) malloc(sizeof(float) * scan_size * AM2_DEF_SNUM_LO);
		if (NULL == p_dataset->p_low_tb_table[i].p_data)
		{
			E_MSG("malloc() error.\n");
			return RET_ERROR;
		}
	}
	
	/* Brightness Temperature (89.0GHz-A,V) ... (89.0GHz-B,H)
	 * (type size * Number of scans * 486) */
	for (i = 0; i < p_dataset->high_tb_num; i++)
	{
		p_dataset->p_high_tb_table[i].p_data =
			(float *) malloc(sizeof(float) * scan_size * AM2_DEF_SNUM_HI);
		if (NULL == p_dataset->p_high_tb_table[i].p_data)
		{
			E_MSG("malloc() error.\n");
			return RET_ERROR;
		}
	}
	
	/* Hot Load Count 6 to 36
	 * (type size * 12 * Number of scans * 16) */
	p_dataset->p_hts_low = (int *) malloc(sizeof(int) * 12 * scan_size * 16);
	if (NULL == p_dataset->p_hts_low)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Hot Load Count 89
	 * (type size * 4 * Number of scans * 32) */
	p_dataset->p_hts_high = (int *) malloc(sizeof(int) * 4 * scan_size * 32);
	if (NULL == p_dataset->p_hts_high)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Cold Sky Mirror Count 6 to 36
	 * (type size * 12 * Number of scans * 16) */
	p_dataset->p_csm_low = (int *) malloc(sizeof(int) * 12 * scan_size * 16);
	if (NULL == p_dataset->p_csm_low)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Cold Sky Mirror Count 89
	 * (type size * 4 * Number of scans * 32) */
	p_dataset->p_csm_high = (int *) malloc(sizeof(int) * 4 * scan_size * 32);
	if (NULL == p_dataset->p_csm_high)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Rx Offset_Gain Count
	 * (type size * Number of scans * 32) */
	p_dataset->p_rx_offset_gain = (int *) malloc(sizeof(int) * scan_size * 32);
	if (NULL == p_dataset->p_rx_offset_gain)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Latitude of Observation Point for 89A
	 * Longitude of Observation Point for 89A
	 * (type size * Number of scans * 486) */
	p_dataset->p_latlon89A = (AM2_COMMON_LATLON *) malloc(
		sizeof(AM2_COMMON_LATLON) * scan_size * AM2_DEF_SNUM_HI);
	if (NULL == p_dataset->p_latlon89A)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Latitude of Observation Point for 89B
	 * Longitude of Observation Point for 89B
	 * (type size * Number of scans * 486) */
	p_dataset->p_latlon89B = (AM2_COMMON_LATLON *) malloc(
		sizeof(AM2_COMMON_LATLON) * scan_size * AM2_DEF_SNUM_HI);
	if (NULL == p_dataset->p_latlon89B)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Sun Azimuth
	 * (type size * Number of scans * 243) */
	p_dataset->p_sun_azimuth = (float *) malloc(sizeof(float) * scan_size
		* AM2_DEF_SNUM_LO);
	if (NULL == p_dataset->p_sun_azimuth)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Sun Elevation
	 * (type size * Number of scans * 243) */
	p_dataset->p_sun_elevation = (float *) malloc(sizeof(float) * scan_size
		* AM2_DEF_SNUM_LO);
	if (NULL == p_dataset->p_sun_elevation)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Earth Incidence
	 * (type size * Number of scans * 243) */
	p_dataset->p_earth_incidence = (float *) malloc(sizeof(float) * scan_size
		* AM2_DEF_SNUM_LO);
	if (NULL == p_dataset->p_earth_incidence)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Earth Azimuth
	 * (type size * Number of scans * 243) */
	p_dataset->p_earth_azimuth = (float *) malloc(sizeof(float) * scan_size
		* AM2_DEF_SNUM_LO);
	if (NULL == p_dataset->p_earth_azimuth)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Land_Ocean Flag 6 to 36
	 * (type size * 6 * Number of scans * 243) */
	p_dataset->p_lof_low = (int *) malloc(sizeof(int) * 6 * scan_size
		* AM2_DEF_SNUM_LO);
	if (NULL == p_dataset->p_lof_low)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Land_Ocean Flag 89
	 * (type size * 2 * Number of scans * 486) */
	p_dataset->p_lof_high = (int *) malloc(sizeof(int) * 2 * scan_size
		* AM2_DEF_SNUM_HI);
	if (NULL == p_dataset->p_lof_high)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Observation Supplement
	 * (type size * Number of scans * 248[124 * binary 2byte]) */
	p_dataset->p_obs_spl = (unsigned char *) malloc(scan_size * (124 * 2));
	if (NULL == p_dataset->p_obs_spl)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* SPC Temperature Count
	 * (type size * Number of scans * 34) */
	p_dataset->p_spc_temp = (float *) malloc(sizeof(float) * scan_size * 34);
	if (NULL == p_dataset->p_spc_temp)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* SPS Temperature Count
	 * (type size * Number of scans * 46) */
	p_dataset->p_sps_temp = (float *) malloc(sizeof(float) * scan_size * 46);
	if (NULL == p_dataset->p_sps_temp)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* PCD Data
	 * (type size * Number of scans * 64) */
	p_dataset->p_pcd_data = (unsigned char *) malloc(scan_size * 64);
	if (NULL == p_dataset->p_pcd_data)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Scan Data Quality
	 * (type size * Number of scans) */
	p_dataset->p_scan_quality = (AMTK_SCAN_DATA_QUALITY *) malloc(
		sizeof(AMTK_SCAN_DATA_QUALITY) * scan_size);
	if (NULL == p_dataset->p_scan_quality)
	{
		E_MSG("malloc() error.\n");
		
		return RET_ERROR;
	}
	
	/* Pixel Data Quality 6 to 36
	 * (type size * Numbef of scans * 486[243 * binary 2byte]) */
	p_dataset->p_pixel_quality_low = (unsigned char *) malloc(
		scan_size * (AM2_DEF_SNUM_LO * 2));
	if (NULL == p_dataset->p_pixel_quality_low)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Pixel Data Quality 89
	 * (type size * Numbef of scans * 486[486 * binary 1byte]) */
	p_dataset->p_pixel_quality_high = (unsigned char *) malloc(
		scan_size * (AM2_DEF_SNUM_HI * 1));
	if (NULL == p_dataset->p_pixel_quality_high)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Interpolation Flag 6 to 36
	 * (type size * 12 * Numbef of scans * 16) */
	p_dataset->p_intpol_low = (unsigned char *) malloc(12 * scan_size * 16);
	if (NULL == p_dataset->p_intpol_low)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Interpolation Flag 89
	 * (type size * 4 * Numbef of scans * 32) */
	p_dataset->p_intpol_high = (unsigned char *) malloc(4 * scan_size * 32);
	if (NULL == p_dataset->p_intpol_high)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}

	return RET_NORMAL;
}


/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset)
{
	int i = 0;
	int ret = 0;
	
	if (0 <= file_id)
	{
		/* HDF product file close (write mode) */
		ret = AMTK_closeH5_Write(file_id);
		if (0 > ret)
		{
			E_MSG("AMTK_closeH5_Write() error.[%d]\n", ret);
		}
	}
	
	/* free memory */
	free(p_dataset->p_scan_time);
	p_dataset->p_scan_time = NULL;
	
	free(p_dataset->p_pos_orbit);
	p_dataset->p_pos_orbit = NULL;
	
	free(p_dataset->p_navi);
	p_dataset->p_navi = NULL;
	
	free(p_dataset->p_att);
	p_dataset->p_att = NULL;
	
	for (i = 0; i < p_dataset->low_tb_num; i++)
	{
		free(p_dataset->p_low_tb_table[i].p_data);
		p_dataset->p_low_tb_table[i].p_data = NULL;
	}
	for (i = 0; i < p_dataset->high_tb_num; i++)
	{
		free(p_dataset->p_high_tb_table[i].p_data);
		p_dataset->p_high_tb_table[i].p_data = NULL;
	}
	
	free(p_dataset->p_hts_low);
	p_dataset->p_hts_low = NULL;
	
	free(p_dataset->p_hts_high);
	p_dataset->p_hts_high = NULL;
	
	free(p_dataset->p_csm_low);
	p_dataset->p_csm_low = NULL;
	
	free(p_dataset->p_csm_high);
	p_dataset->p_csm_high = NULL;
	
	free(p_dataset->p_rx_offset_gain);
	p_dataset->p_rx_offset_gain = NULL;
	
	free(p_dataset->p_latlon89A);
	p_dataset->p_latlon89A = NULL;
	
	free(p_dataset->p_latlon89B);
	p_dataset->p_latlon89B = NULL;
	
	free(p_dataset->p_sun_azimuth);
	p_dataset->p_sun_azimuth = NULL;
	
	free(p_dataset->p_sun_elevation);
	p_dataset->p_sun_elevation = NULL;
	
	free(p_dataset->p_earth_incidence);
	p_dataset->p_earth_incidence = NULL;
	
	free(p_dataset->p_earth_azimuth);
	p_dataset->p_earth_azimuth = NULL;
	
	free(p_dataset->p_lof_low);
	p_dataset->p_lof_low = NULL;
	
	free(p_dataset->p_lof_high);
	p_dataset->p_lof_high = NULL;
	
	free(p_dataset->p_obs_spl);
	p_dataset->p_obs_spl = NULL;
	
	free(p_dataset->p_spc_temp);
	p_dataset->p_spc_temp = NULL;
	
	free(p_dataset->p_sps_temp);
	p_dataset->p_sps_temp = NULL;
	
	free(p_dataset->p_pcd_data);
	p_dataset->p_pcd_data = NULL;
	
	free(p_dataset->p_scan_quality);
	p_dataset->p_scan_quality = NULL;
	
	free(p_dataset->p_pixel_quality_low);
	p_dataset->p_pixel_quality_low = NULL;
	
	free(p_dataset->p_pixel_quality_high);
	p_dataset->p_pixel_quality_high = NULL;
	
	free(p_dataset->p_intpol_low);
	p_dataset->p_intpol_low = NULL;
	
	free(p_dataset->p_intpol_high);
	p_dataset->p_intpol_high = NULL;
	
	return;
}


/** Set dataset value.
 * @param scan_size      [IN]  Number of scans
 * @param p_dataset      [OUT] Dataset
 */
static void set_value(int scan_size, DATASET_MODEL *p_dataset)
{
	int i = 0, j = 0;
	int scan_num = 0;
	int pixel_num = 0;
	
	/* Scan Data Quality(sample) */
	AMTK_SCAN_DATA_QUALITY sample_data = {{0}};
	AM2_COMMON_SCANTIME *p_scan_time = NULL;
	
	/* Scan loop */
	for (scan_num = 0; scan_num < scan_size; scan_num++)
	{
		/* Scan Time */
		p_scan_time = p_dataset->p_scan_time + scan_num;
		p_scan_time->year = 2010;
		p_scan_time->month = scan_num % 12;
		if (p_scan_time->month == 0) p_scan_time->month = 1;
		p_scan_time->day = scan_num % 28;
		if (p_scan_time->day == 0) p_scan_time->day = 1;
		p_scan_time->hour= scan_num % 24;
		p_scan_time->minute = scan_num % 60;
		p_scan_time->second = scan_num % 60;
		p_scan_time->ms = scan_num % 100;
		
		/* Position in Orbit */
		*(p_dataset->p_pos_orbit + scan_num) = scan_num * 0.001;
		
		/* Navigation Data */
		for (i = 0; i < 6; i++)
			*(p_dataset->p_navi + (6 * scan_num) + i) = scan_num * 0.01 + i + 1;
		
		/* Attitude Data */
		for (i = 0; i < 3; i++)
			*(p_dataset->p_att + (3 * scan_num) + i) = scan_num * 0.01 + i + 1;
		
		/* Low pixel loop */
		for (pixel_num = 0; pixel_num < AM2_DEF_SNUM_LO; pixel_num++)
		{
			/* Brightness Temperature <Low resolution> */
			for (i = 0; i < p_dataset->low_tb_num; i++)
			{
				*(p_dataset->p_low_tb_table[i].p_data +
					(AM2_DEF_SNUM_LO * scan_num) + pixel_num) =
						0.01 * pixel_num + i + 1;
			}
			
			/* Sun Azimuth */
			*(p_dataset->p_sun_azimuth + (AM2_DEF_SNUM_LO * scan_num) +
				pixel_num) = 0.01 * pixel_num;
			
			/* Sun Elevation */
			*(p_dataset->p_sun_elevation + (AM2_DEF_SNUM_LO * scan_num) +
				pixel_num) = 0.01 * pixel_num;
			
			/* Earth Incidence */
			*(p_dataset->p_earth_incidence + (AM2_DEF_SNUM_LO * scan_num) +
				pixel_num) = 0.01 * pixel_num;
			
			/* Earth Azimuth */
			*(p_dataset->p_earth_azimuth + (AM2_DEF_SNUM_LO * scan_num) +
				pixel_num) = 0.01 * pixel_num;
			
			/* Land_Ocean Flag 6 to 36 */
			for (i = 0; i < 6; i++)
			{
				*(p_dataset->p_lof_low
					+ (i * scan_size * AM2_DEF_SNUM_LO)
					+ (scan_num * AM2_DEF_SNUM_LO)
					+ (pixel_num)) = (i + scan_num + pixel_num) % 0xFF;
			}
		}
		
		/* High pixel loop */
		for (pixel_num = 0; pixel_num < AM2_DEF_SNUM_HI; pixel_num++)
		{
			/* Brightness Temperature <High resolution> */
			for (i = 0; i < p_dataset->high_tb_num; i++)
			{
				*(p_dataset->p_high_tb_table[i].p_data +
					(AM2_DEF_SNUM_HI * scan_num) + pixel_num) =
						0.01 * pixel_num + i + 1;
			}
			
			/* Latitude of Observation Point
			 * Longitude of Observation Point */
			(p_dataset->p_latlon89A + (AM2_DEF_SNUM_HI * scan_num)
				+ pixel_num)->lat = (pixel_num - 90) % 91;
			(p_dataset->p_latlon89A + (AM2_DEF_SNUM_HI * scan_num)
				+ pixel_num)->lon = (pixel_num - 180) % 181;
			(p_dataset->p_latlon89B + (AM2_DEF_SNUM_HI * scan_num)
				+ pixel_num)->lat = (pixel_num - 89) % 91;
			(p_dataset->p_latlon89B + (AM2_DEF_SNUM_HI * scan_num)
				+ pixel_num)->lon = (pixel_num - 179) % 181;
			
			/* Land_Ocean Flag 89 */
			for (i = 0; i < 2; i++)
			{
				*(p_dataset->p_lof_high
					+ (i * scan_size * AM2_DEF_SNUM_HI)
					+ (scan_num * AM2_DEF_SNUM_HI)
					+ (pixel_num)) = (i + scan_num + pixel_num) % 0xFF;
			}
		}
		
		
		for (i = 0; i < 12; i++)
		{
			for (j = 0; j < 16; j++)
			{
				/* Hot Load Count 6 to 36 */
				*(p_dataset->p_hts_low + (i * scan_size * 16) + (scan_num * 16) + j)
					= (i + scan_num + j) % 0x7FFF;
				
				/* Cold Sky Mirror Count 6 to 36 */
				*(p_dataset->p_csm_low + (i * scan_size * 16) + (scan_num * 16) + j)
					= (i + scan_num + j) % 0x7FFF;
			}
		}
		
		for (i = 0; i < 4; i++)
		{
			for (j = 0; j < 32; j++)
			{
				/* Hot Load Count 89 */
				*(p_dataset->p_hts_high + (i * scan_size * 32) + (scan_num * 32)
					+ j) = (i + scan_num + j) % 0x7FFF;
				
				/* Cold Sky Mirror Count 89 */
				*(p_dataset->p_csm_high + (i * scan_size * 32) + (scan_num * 32)
					+ j) = (i + scan_num + j) % 0x7FFF;
			}
		}
		
		for (i = 0; i < 32; i++)
		{
			/* Rx Offset_Gain Count */
			*(p_dataset->p_rx_offset_gain + (scan_num * 32) + i)
				= (scan_num + i) % 0xFFFF;
		}
		
		/* Observation Supplement */
		memset(p_dataset->p_obs_spl + (scan_num * (124 * 2)), scan_num % 0xFF,
			(124 * 2));
		
		for (i = 0; i < 34; i++)
		{
			/* SPC Temperature Count */
			*(p_dataset->p_spc_temp + (scan_num * 34) + i)
				= (scan_num + i) % 0xFFFF;
		}
		
		for (i = 0; i < 46; i++)
		{
			/* SPS Temperature Count */
			*(p_dataset->p_sps_temp + (scan_num * 46) + i)
				= (scan_num + i) % 0xFFFF;
		}
		
		/* PCD Data */
		memset(p_dataset->p_pcd_data + (scan_num * 64), scan_num % 0xFF, 64);
		
		/* Scan Data Quality */
		sample_data.quality_of_packet = scan_num;
		*(p_dataset->p_scan_quality + scan_num) = sample_data;
		
		/* Pixel Data Quality 6 to 36 */
		memset(p_dataset->p_pixel_quality_low
			+ (scan_num * (AM2_DEF_SNUM_LO * 2)),
			scan_num % 0xFF, (AM2_DEF_SNUM_LO * 2));
		
		/* Pixel Data Quality 89 */
		memset(p_dataset->p_pixel_quality_high
			+ (scan_num * (AM2_DEF_SNUM_HI * 1)),
			scan_num % 0xFF, (AM2_DEF_SNUM_HI * 1));
		
		/* Interpolation Flag 6 to 36 */
		for (i = 0; i < 12; i++)
		{
			for (j = 0; j < 16; j++)
			{
				*(p_dataset->p_intpol_low + (i * scan_size * 16)
					+ (scan_num * 16) + j)
					= (i + scan_num + j) % 0xFF;
			}
		}
		
		/* Interpolation Flag 89 */
		for (i = 0; i < 4; i++)
		{
			for (j = 0; j < 32; j++)
			{
				*(p_dataset->p_intpol_high + (i * scan_size * 32)
					+ (scan_num * 32) + j)
					= (i + scan_num + j) % 0xFF;
			}
		}
	}
	return;
}

