/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2010 All rights reserved.                  */
/*  File Name    : sample3_make_L2Lproduct.c                                 */
/*  Abstract     : I/O toolkit sample program                                */
/*  Function List                                                            */
/*      main()                       : Sample program main                   */
/*      initialize()                 : Initialization                        */
/*      terminate()                  : Termination                           */
/*      set_value()                  : Set dataset value                     */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2011/01/26                                                    */
/*  Version    1.08                                                          */
/*  Notice                                                                   */
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/** Include AMTK.h */
#ifdef __cplusplus
extern "C" {
#endif
#include "AMTK.h"
#ifdef __cplusplus
}
#endif

#include "sample_common.h"

/*---------------------------------------------------------------------------*
 * Common definition                                                         *
 *---------------------------------------------------------------------------*/
 
/** Sample L2(Low) product file name. */
#define FN_SAMPLE_PRODUCT_L2L ("../data/sample3_IOToolKit_l2l.h5")
/** Metadata definition. */
#define FN_SAMPLE_META_L2L    ("../data/metadata_sample_L2L.txt")

/** Number of scans. */
#define SCENE_SCAN_NUM        (2000)
/** Overlap scans. */
#define OVERLAP_SCAN_NUM      (0)
/** Number of Geophysical Data. */
#define GEO_DATA_LAYER_NUM    (3)

/** Structure of dataset model. */
typedef struct
{
	/* Scan Time */
	AM2_COMMON_SCANTIME *p_scan_time;
	
	/* Position in Orbit */
	double *p_pos_orbit;
	
	/* Geophysical Data (Layer: 1...3) */
	float *p_geo_data[GEO_DATA_LAYER_NUM];
	
	/* Latitude of Observation Point
	 * Longitude of Observation Point */
	AM2_COMMON_LATLON *p_latlon;
	
	/* Pixel Data Quality */
	unsigned char *p_pixel_quality;
	
} DATASET_MODEL;


/** Initialization.
 * @param scan_size [IN]  Number of scans
 * @param p_dataset [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(int scan_size, DATASET_MODEL *p_dataset);

/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset);

/** Set dataset value.
 * @param scan_size [IN]  Number of scans
 * @param p_dataset [OUT] Dataset
 */
static void set_value(int scan_size, DATASET_MODEL *p_dataset);


/*---------------------------------------------------------------------------*
 * Main program                                                              *
 *---------------------------------------------------------------------------*/
 
/** AMSR2 I/O toolkit sample program.
 * sample3: output L2(Low) proudct.
 * 
 * @param  argc [IN] Argument count
 * @param  argv [IN] Argument value
 * @return EXIT_SUCCESS
 * @return EXIT_FAILURE
 */
int main(int argc, char *argv[])
{
	hid_t file_id = 0;
	
	int i = 0;
	int meta_num = 0;
	int dimsize[3] = {0};
	int ret = 0;
	
	/* sample meta data read */
	char meta_name[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
	char meta_value[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
	
	/* Number of Scans */
	const int scan_size = SCENE_SCAN_NUM + OVERLAP_SCAN_NUM * 2;
	const int scan_start = 1 - OVERLAP_SCAN_NUM;
	const int scan_end = SCENE_SCAN_NUM + OVERLAP_SCAN_NUM;
	
	/* Access label: Geophysical Data */
	const int geo_data_label[] = {AM2_SWATH_GEO1, AM2_SWATH_GEO2, AM2_SWATH_GEO3};
	
	/* Dataset. */
	DATASET_MODEL dataset = {0};
	
	/* memory allocation */
	ret = initialize(scan_size, &dataset);
	if (ret != RET_NORMAL)
	{
		E_MSG("initialize() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* set dataset value */
	set_value(scan_size, &dataset);
	
	/*------------------------------------------------------------------------*
	 * output data                                                            *
	 *------------------------------------------------------------------------*/
	/* HDF product file open (write mode) */
	file_id = AMTK_openH5_Write(FN_SAMPLE_PRODUCT_L2L, AM2_CREATE_MODE);
	if (0 > file_id)
	{
		E_MSG("AMTK_openH5_Write() error.[%d]\n", file_id);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Read Metadata from Metadata file */
	if (RET_NORMAL != get_metadata_sample(FN_SAMPLE_META_L2L, meta_name,
		meta_value, &meta_num))
	{
		E_MSG("get_metadata_sample() error.\n");
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Metadata write */
	for (i = 0; i < meta_num; i++) 
	{
		ret = AMTK_setMetaDataName(file_id, &meta_name[i][0],
			&meta_value[i][0]);
		if (0 > ret)
		{
			E_MSG("AMTK_setMetaDataName() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Data set write */
	/* Scan Time */
	{
		dimsize[0] = scan_size;
		dimsize[1] = 0;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_SCAN_TIME_DEF, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_setScanTime(file_id, dataset.p_scan_time, scan_start,
			scan_end);
		if (0 > ret)
		{
			E_MSG("AMTK_setScanTime() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Position in Orbit */
	{
		dimsize[0] = scan_size;
		dimsize[1] = 0;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_POS_ORBIT, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathDouble(file_id, dataset.p_pos_orbit, scan_start,
			scan_end, AM2_POS_ORBIT);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathDouble() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Geophysical Data (Layer: 1...3) */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_LO;
		dimsize[2] = GEO_DATA_LAYER_NUM;  /* 1 ... 3 */
		
		/* Layer: ALL -> AM2_SWATH_GEOA */
		ret = AMTK_setDimSize(file_id, AM2_SWATH_GEOA, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		/* Layer: 1...3 -> AM2_SWATH_GEO1 ... AM2_SWATH_GEO3 */
		for (i = 0; i < GEO_DATA_LAYER_NUM; i++)
		{
			ret = AMTK_set_SwathFloat(file_id, dataset.p_geo_data[i], scan_start, 
				scan_end, geo_data_label[i]);
			if (0 > ret)
			{
				E_MSG("AMTK_set_SwathFloat() error.[%d]\n", ret);
				terminate(file_id, &dataset);
				exit(EXIT_FAILURE);
			}
		}
	}
	
	/* Latitude of Observation Point
	 * Longitude of Observation Point */
	{
		dimsize[0] = scan_size;
		dimsize[1] = AM2_DEF_SNUM_LO;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_LATLON_L2_LO, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_setLatLon(file_id, dataset.p_latlon, scan_start, scan_end,
			AM2_LATLON_L2_LO);
		if (0 > ret)
		{
			E_MSG("AMTK_setLatLon() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Pixel Data Quality */
	{
		dimsize[0] = GEO_DATA_LAYER_NUM; /* 1 ... 3 */
		dimsize[1] = scan_size;
		dimsize[2] = AM2_DEF_SNUM_LO;    /* binary (1byte) */
		ret = AMTK_setDimSize(file_id, AM2_PIX_QUAL, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_SwathUChar(file_id, dataset.p_pixel_quality,
			scan_start, scan_end, AM2_PIX_QUAL);
		if (0 > ret)
		{
			E_MSG("AMTK_set_SwathUChar() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* termination */
	terminate(file_id, &dataset);
	
	printf("L2(Low) product created\n");
	
	exit(EXIT_SUCCESS);
}


/** Initialization.
 * @param scan_size [IN]  Number of scans
 * @param p_dataset [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(int scan_size, DATASET_MODEL *p_dataset)
{
	int i = 0;
	
	/* memory allocation */
	
	/* Scan Time
	 * (type size * Number of scans) */
	p_dataset->p_scan_time = (AM2_COMMON_SCANTIME *) malloc(
		sizeof(AM2_COMMON_SCANTIME) * scan_size);
	if (NULL == p_dataset->p_scan_time)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Position in Orbit
	 * (type size * Number of scans) */
	p_dataset->p_pos_orbit = (double *) malloc(sizeof(double) * scan_size);
	if (NULL == p_dataset->p_pos_orbit)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Geophysical Data (Layer: 1...3)
	 * (type size * Number of scans * 243) */
	for (i = 0; i < GEO_DATA_LAYER_NUM; i++)
	{
		p_dataset->p_geo_data[i] = (float *) malloc(sizeof(float) * scan_size
			* AM2_DEF_SNUM_LO);
		if (NULL == p_dataset->p_geo_data[i])
		{
			E_MSG("malloc() error.\n");
			return RET_ERROR;
		}
	}
	
	/* Latitude of Observation Point
	 * Longitude of Observation Point
	 * (type size * Number of scans * 243) */
	p_dataset->p_latlon = (AM2_COMMON_LATLON *) malloc(
		sizeof(AM2_COMMON_LATLON) * scan_size * AM2_DEF_SNUM_LO);
	if (NULL == p_dataset->p_latlon)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Pixel Data Quality
	 * (type size * Number of Geophysical Data[1...3] * Numbef of scans * 243) */
	p_dataset->p_pixel_quality = (unsigned char *) malloc(
		GEO_DATA_LAYER_NUM * scan_size * AM2_DEF_SNUM_LO);
	if (NULL == p_dataset->p_pixel_quality)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	return RET_NORMAL;
}


/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset)
{
	int i = 0;
	int ret = 0;
	
	if (0 <= file_id)
	{
		/* HDF product file close (write mode) */
		ret = AMTK_closeH5_Write(file_id);
		if (0 > ret)
		{
			E_MSG("AMTK_closeH5_Write() error.[%d]\n", ret);
		}
	}
	
	/* free memory */
	free(p_dataset->p_scan_time);
	p_dataset->p_scan_time = NULL;
	
	free(p_dataset->p_pos_orbit);
	p_dataset->p_pos_orbit = NULL;
	
	for (i = 0; i < GEO_DATA_LAYER_NUM; i++)
	{
		free(p_dataset->p_geo_data[i]);
		p_dataset->p_geo_data[i] = NULL;
	}
	
	free(p_dataset->p_latlon);
	p_dataset->p_latlon = NULL;
	
	free(p_dataset->p_pixel_quality);
	p_dataset->p_pixel_quality = NULL;
	
	return;
}


/** Set dataset value.
 * @param scan_size [IN]  Number of scans
 * @param p_dataset [OUT] Dataset
 */
static void set_value(int scan_size, DATASET_MODEL *p_dataset)
{
	int i = 0;
	int scan_num = 0;
	int pixel_num = 0;
	
	AM2_COMMON_SCANTIME *p_scan_time = NULL;
	
	/* Scan loop */
	for (scan_num = 0; scan_num < scan_size; scan_num++)
	{
		/* Scan Time */
		p_scan_time = p_dataset->p_scan_time + scan_num;
		p_scan_time->year = 2010;
		p_scan_time->month = scan_num % 12;
		if (p_scan_time->month == 0) p_scan_time->month = 1;
		p_scan_time->day = scan_num % 28;
		if (p_scan_time->day == 0) p_scan_time->day = 1;
		p_scan_time->hour= scan_num % 24;
		p_scan_time->minute = scan_num % 60;
		p_scan_time->second = scan_num % 60;
		p_scan_time->ms = scan_num % 100;
		
		/* Position in Orbit */
		*(p_dataset->p_pos_orbit + scan_num) = scan_num * 0.001;
		
		for (pixel_num = 0; pixel_num < AM2_DEF_SNUM_LO; pixel_num++)
		{
			for (i = 0; i < GEO_DATA_LAYER_NUM; i++)
			{
				/* Geophysical Data */
				*(p_dataset->p_geo_data[i] + (scan_num * AM2_DEF_SNUM_LO)
					+ pixel_num) = (scan_num + pixel_num + i) * 0.001;
				
				/* Pixel Data Quality */
				*(p_dataset->p_pixel_quality
					+ (i * scan_size * AM2_DEF_SNUM_LO)
					+ (scan_num * AM2_DEF_SNUM_LO)
					+ pixel_num)
					= (scan_num + pixel_num + i) % 0xFF;
			}
			
			/* Latitude of Observation Point
			 * Longitude of Observation Point */
			(p_dataset->p_latlon + (AM2_DEF_SNUM_LO * scan_num)
				+ pixel_num)->lat = (pixel_num - 90) % 91;
			(p_dataset->p_latlon + (AM2_DEF_SNUM_LO * scan_num)
				+ pixel_num)->lon = (pixel_num - 180) % 181;
		}
	}
	
	return;
}

