/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2010 All rights reserved.                  */
/*  File Name    : sample5_make_L3Dproduct.c                                 */
/*  Abstract     : I/O toolkit sample program                                */
/*  Function List                                                            */
/*      main()                       : Sample program main                   */
/*      initialize()                 : Initialization                        */
/*      terminate()                  : Termination                           */
/*      set_value()                  : Set dataset value                     */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2011/01/26                                                    */
/*  Version    1.08                                                          */
/*  Notice                                                                   */
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/** Include AMTK.h */
#ifdef __cplusplus
extern "C" {
#endif
#include "AMTK.h"
#ifdef __cplusplus
}
#endif

#include "sample_common.h"

/*---------------------------------------------------------------------------*
 * Common definition                                                         *
 *---------------------------------------------------------------------------*/
 
/** Sample L3(Day) product file name. */
#define FN_SAMPLE_PRODUCT_L3D ("../data/sample5_IOToolKit_l3d.h5")
/** Metadata definition. */
#define FN_SAMPLE_META_L3D    ("../data/metadata_sample_L3_day.txt")

/** Number of Geophysical Data (1...3). */
#define GEO_DATA_LAYER_NUM    (1)

/** Structure of dataset model. */
typedef struct
{
	/* Geophysical Data (Layer: 1...3) */
	float *p_geo_data[GEO_DATA_LAYER_NUM];
	/* Time Infomation */
	int *p_time_info;
	
	/* Number of pixels in X direction(longitude) */
	int x_pixsize;
	/* Number of pixels in Y direction(latitude) */
	int y_pixsize;
	
} DATASET_MODEL;


/** Initialization.
 * @param p_dataset [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(DATASET_MODEL *p_dataset);

/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset);

/** Set dataset value.
 * @param p_dataset [OUT] Dataset
 */
static void set_value(DATASET_MODEL *p_dataset);


/*---------------------------------------------------------------------------*
 * Main program                                                              *
 *---------------------------------------------------------------------------*/
 
/** AMSR2 I/O toolkit sample program.
 * sample5: output L3(Day) proudct.
 * 
 * GeophysicalName: Cloud Liquid Water
 * Projection     : EQR
 * Resolution     : 0.25deg
 * 
 * @param  argc [IN] Argument count
 * @param  argv [IN] Argument value
 * @return EXIT_SUCCESS
 * @return EXIT_FAILURE
 */
int main(int argc, char *argv[])
{
	hid_t file_id = 0;
	
	int i = 0;
	int meta_num = 0;
	int dimsize[3] = {0};
	int ret = 0;
	
	/* sample meta data read */
	char meta_name[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
	char meta_value[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
	
	/* Access label: Geophysical Data */
	const int geo_label[] = {AM2_GRID_GEO1, AM2_GRID_GEO2, AM2_GRID_GEO3};
	
	/* Dataset. */
	DATASET_MODEL dataset = {{0}};
	
	/* size: EQR/0.25deg */
	dataset.x_pixsize = AM2_DEF_L3L_EQ_X; /* 1440 */
	dataset.y_pixsize = AM2_DEF_L3L_EQ_Y; /*  720 */
	
	
	/* memory allocation */
	ret = initialize(&dataset);
	if (ret != RET_NORMAL)
	{
		E_MSG("initialize() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* set dataset value */
	set_value(&dataset);
	
	/*------------------------------------------------------------------------*
	 * output data                                                            *
	 *------------------------------------------------------------------------*/
	/* HDF product file open (write mode) */
	file_id = AMTK_openH5_Write(FN_SAMPLE_PRODUCT_L3D, AM2_CREATE_MODE);
	if (0 > file_id)
	{
		E_MSG("AMTK_openH5_Write() error.[%d]\n", file_id);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Read Metadata from Metadata file */
	if (RET_NORMAL != get_metadata_sample(FN_SAMPLE_META_L3D, meta_name,
		meta_value, &meta_num))
	{
		E_MSG("get_metadata_sample() error.\n");
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Metadata write */
	for (i = 0; i < meta_num; i++) 
	{
		ret = AMTK_setMetaDataName(file_id, &meta_name[i][0],
			&meta_value[i][0]);
		if (0 > ret)
		{
			E_MSG("AMTK_setMetaDataName() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Data set write */
	
	/* Geophysical Data (Layer: 1...3) */
	{
		dimsize[0] = dataset.x_pixsize;
		dimsize[1] = dataset.y_pixsize;
		dimsize[2] = GEO_DATA_LAYER_NUM;  /* 1 ... 3 */
		
		/* Layer: ALL -> AM2_GRID_GEOA */
		ret = AMTK_setDimSize(file_id, AM2_GRID_GEOA, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		/* Layer: 1...3 -> AM2_GRID_GEO1 ... AM2_GRID_GEO3 */
		for (i = 0; i < GEO_DATA_LAYER_NUM; i++)
		{
			ret = AMTK_set_GridFloat(file_id, dataset.p_geo_data[i],
				geo_label[i]);
			if (0 > ret)
			{
				E_MSG("AMTK_set_GridFloat() error.[%d]\n", ret);
				terminate(file_id, &dataset);
				exit(EXIT_FAILURE);
			}
		}
	}
	
	/* Time Infomation */
	{
		dimsize[0] = dataset.x_pixsize;
		dimsize[1] = dataset.y_pixsize;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_GRID_TIME, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_GridInt(file_id, dataset.p_time_info, AM2_GRID_TIME);
		if (0 > ret)
		{
			E_MSG("AMTK_set_GridInt() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* termination */
	terminate(file_id, &dataset);
	
	printf("L3(Day) product created\n");
	
	exit(EXIT_SUCCESS);
}


/** Initialization.
 * @param p_dataset [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(DATASET_MODEL *p_dataset)
{
	int i = 0;
	
	/* memory allocation */
	
	/* Geophysical Data (Layer: 1...3)
	 * (type size * X pixel * Y pixel ) */
	for (i = 0; i < GEO_DATA_LAYER_NUM; i++)
	{
		p_dataset->p_geo_data[i] = (float *) malloc(sizeof(float)
			* p_dataset->x_pixsize * p_dataset->y_pixsize);
		if (NULL == p_dataset->p_geo_data[i])
		{
			E_MSG("malloc() error.\n");
			return RET_ERROR;
		}
	}
	
	
	/* Time Infomation
	 * (type size * X pixel * Y pixel ) */
	p_dataset->p_time_info = (int *) malloc(sizeof(int)
			* p_dataset->x_pixsize * p_dataset->y_pixsize);
	if (NULL == p_dataset->p_time_info)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	return RET_NORMAL;
}


/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset)
{
	int i = 0;
	int ret = 0;
	
	if (0 <= file_id)
	{
		/* HDF product file close (write mode) */
		ret = AMTK_closeH5_Write(file_id);
		if (0 > ret)
		{
			E_MSG("AMTK_closeH5_Write() error.[%d]\n", ret);
		}
	}
	
	/* free memory */
	for (i = 0; i < GEO_DATA_LAYER_NUM; i++)
	{
		free(p_dataset->p_geo_data[i]);
		p_dataset->p_geo_data[i] = NULL;
	}
	
	free(p_dataset->p_time_info);
	p_dataset->p_time_info = NULL;
	
	return;
}


/** Set dataset value.
 * @param p_dataset [OUT] Dataset
 */
static void set_value(DATASET_MODEL *p_dataset)
{
	int x = 0;
	int y = 0;
	int i = 0;
	
	for (x = 0; x < p_dataset->x_pixsize; x++)
	{
		for (y = 0; y < p_dataset->y_pixsize; y++)
		{
			/* Geophysical Data */
			for (i = 0; i < GEO_DATA_LAYER_NUM; i++)
			{
				*(p_dataset->p_geo_data[i] + (x * p_dataset->y_pixsize) + y)
					= (x + y + i) * 0.001;
			}
			
			/* Time Infomation */
			*(p_dataset->p_time_info + (x * p_dataset->y_pixsize) + y)
				= (x + y);
		}
	}
	return;
}

