/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2010 All rights reserved.                  */
/*  File Name    : sample6_make_L3Mproduct.c                                 */
/*  Abstract     : I/O toolkit sample program                                */
/*  Function List                                                            */
/*      main()                       : Sample program main                   */
/*      initialize()                 : Initialization                        */
/*      terminate()                  : Termination                           */
/*      set_value()                  : Set dataset value                     */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2011/01/26                                                    */
/*  Version    1.08                                                          */
/*  Notice                                                                   */
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/** Include AMTK.h */
#ifdef __cplusplus
extern "C" {
#endif
#include "AMTK.h"
#ifdef __cplusplus
}
#endif

#include "sample_common.h"

/*---------------------------------------------------------------------------*
 * Common definition                                                         *
 *---------------------------------------------------------------------------*/
 
/** Sample L3(Month) product file name. */
#define FN_SAMPLE_PRODUCT_L3M ("../data/sample6_IOToolKit_l3m.h5")
/** Metadata definition. */
#define FN_SAMPLE_META_L3M    ("../data/metadata_sample_L3_month.txt")

/** Structure of dataset model. */
typedef struct
{
	/* Brightness Temperature (V) */
	float *p_tb_v;
	/* Brightness Temperature (H) */
	float *p_tb_h;
	/* Standard Deviation (V) */
	float *p_std_dev_v;
	/* Standard Deviation (H) */
	float *p_std_dev_h;
	/* Average Number (V) */
	int *p_avg_v;
	/* Average Number (H) */
	int *p_avg_h;
	/* Total Number (V) */
	int *p_total_v;
	/* Total Number (H) */
	int *p_total_h;
	
	/* Number of pixels in X direction(longitude) */
	int x_pixsize;
	/* Number of pixels in Y direction(latitude) */
	int y_pixsize;
	
} DATASET_MODEL;


/** Initialization.
 * @param p_dataset [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(DATASET_MODEL *p_dataset);

/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset);

/** Set dataset value.
 * @param p_dataset [OUT] Dataset
 */
static void set_value(DATASET_MODEL *p_dataset);


/*---------------------------------------------------------------------------*
 * Main program                                                              *
 *---------------------------------------------------------------------------*/
 
/** AMSR2 I/O toolkit sample program.
 * sample6: output L3(Month) proudct.
 * 
 * GeophysicalName: Brightness Temperature (89GHz)
 * Projection     : PS-N
 * Resolution     : 10km
 * 
 * @param  argc [IN] Argument count
 * @param  argv [IN] Argument value
 * @return EXIT_SUCCESS
 * @return EXIT_FAILURE
 */
int main(int argc, char *argv[])
{
	hid_t file_id = 0;
	
	int i = 0;
	int meta_num = 0;
	int dimsize[3] = {0};
	int ret = 0;
	
	/* sample meta data read */
	char meta_name[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
	char meta_value[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
	
	/* Dataset. */
	DATASET_MODEL dataset = {0};
	
	/* size: PS-N/10km */
	dataset.x_pixsize = AM2_DEF_L3H_PN1_X; /*  760 */
	dataset.y_pixsize = AM2_DEF_L3H_PN1_Y; /* 1120 */
	
	
	/* memory allocation */
	ret = initialize(&dataset);
	if (ret != RET_NORMAL)
	{
		E_MSG("initialize() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* set dataset value */
	set_value(&dataset);
	
	/*------------------------------------------------------------------------*
	 * output data                                                            *
	 *------------------------------------------------------------------------*/
	/* HDF product file open (write mode) */
	file_id = AMTK_openH5_Write(FN_SAMPLE_PRODUCT_L3M, AM2_CREATE_MODE);
	if (0 > file_id)
	{
		E_MSG("AMTK_openH5_Write() error.[%d]\n", file_id);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Read Metadata from Metadata file */
	if (RET_NORMAL != get_metadata_sample(FN_SAMPLE_META_L3M, meta_name,
		meta_value, &meta_num))
	{
		E_MSG("get_metadata_sample() error.\n");
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Metadata write */
	for (i = 0; i < meta_num; i++) 
	{
		ret = AMTK_setMetaDataName(file_id, &meta_name[i][0],
			&meta_value[i][0]);
		if (0 > ret)
		{
			E_MSG("AMTK_setMetaDataName() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Data set write */
	
	/* Brightness Temperature (V) */
	{
		dimsize[0] = dataset.x_pixsize;
		dimsize[1] = dataset.y_pixsize;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_GRID_TBV, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_GridFloat(file_id, dataset.p_tb_v, AM2_GRID_TBV);
		if (0 > ret)
		{
			E_MSG("AMTK_set_GridFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	/* Brightness Temperature (H) */
	{
		dimsize[0] = dataset.x_pixsize;
		dimsize[1] = dataset.y_pixsize;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_GRID_TBH, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_GridFloat(file_id, dataset.p_tb_h, AM2_GRID_TBH);
		if (0 > ret)
		{
			E_MSG("AMTK_set_GridFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Standard Deviation (V) */
	{
		dimsize[0] = dataset.x_pixsize;
		dimsize[1] = dataset.y_pixsize;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_GRID_TBV_STD, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_GridFloat(file_id, dataset.p_std_dev_v, AM2_GRID_TBV_STD);
		if (0 > ret)
		{
			E_MSG("AMTK_set_GridFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	/* Standard Deviation (H) */
	{
		dimsize[0] = dataset.x_pixsize;
		dimsize[1] = dataset.y_pixsize;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_GRID_TBH_STD, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_GridFloat(file_id, dataset.p_std_dev_h,
			AM2_GRID_TBH_STD);
		if (0 > ret)
		{
			E_MSG("AMTK_set_GridFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Average Number (V) */
	{
		dimsize[0] = dataset.x_pixsize;
		dimsize[1] = dataset.y_pixsize;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_GRID_TBV_ANUM, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_GridInt(file_id, dataset.p_avg_v, AM2_GRID_TBV_ANUM);
		if (0 > ret)
		{
			E_MSG("AMTK_set_GridFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	/* Average Number (H) */
	{
		dimsize[0] = dataset.x_pixsize;
		dimsize[1] = dataset.y_pixsize;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_GRID_TBH_ANUM, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_GridInt(file_id, dataset.p_avg_h, AM2_GRID_TBH_ANUM);
		if (0 > ret)
		{
			E_MSG("AMTK_set_GridFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Total Number (V) */
	{
		dimsize[0] = dataset.x_pixsize;
		dimsize[1] = dataset.y_pixsize;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_GRID_TBV_TNUM, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_GridInt(file_id, dataset.p_total_v, AM2_GRID_TBV_TNUM);
		if (0 > ret)
		{
			E_MSG("AMTK_set_GridFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	/* Total Number (H) */
	{
		dimsize[0] = dataset.x_pixsize;
		dimsize[1] = dataset.y_pixsize;
		dimsize[2] = 0;
		ret = AMTK_setDimSize(file_id, AM2_GRID_TBH_TNUM, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_setDimSize() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		
		ret = AMTK_set_GridInt(file_id, dataset.p_total_h, AM2_GRID_TBH_TNUM);
		if (0 > ret)
		{
			E_MSG("AMTK_set_GridFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* termination */
	terminate(file_id, &dataset);
	
	printf("L3(Month) product created\n");
	
	exit(EXIT_SUCCESS);
}


/** Initialization.
 * @param p_dataset [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(DATASET_MODEL *p_dataset)
{
	/* memory allocation */
	
	/* Brightness Temperature (V)
	 * (type size * X pixel * Y pixel ) */
	p_dataset->p_tb_v = (float *) malloc(sizeof(float) * p_dataset->x_pixsize
		* p_dataset->y_pixsize);
	if (NULL == p_dataset->p_tb_v)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	/* Brightness Temperature (H)
	 * (type size * X pixel * Y pixel ) */
	p_dataset->p_tb_h = (float *) malloc(sizeof(float) * p_dataset->x_pixsize
		* p_dataset->y_pixsize);
	if (NULL == p_dataset->p_tb_h)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
		
	/* Standard Deviation (V)
	 * (type size * X pixel * Y pixel ) */
	p_dataset->p_std_dev_v = (float *) malloc(sizeof(float)
		* p_dataset->x_pixsize * p_dataset->y_pixsize);
	if (NULL == p_dataset->p_std_dev_v)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	/* Standard Deviation (H)
	 * (type size * X pixel * Y pixel ) */
	p_dataset->p_std_dev_h = (float *) malloc(sizeof(float)
		* p_dataset->x_pixsize * p_dataset->y_pixsize);
	if (NULL == p_dataset->p_std_dev_h)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Average Number (V)
	 * (type size * X pixel * Y pixel ) */
	p_dataset->p_avg_v = (int *) malloc(sizeof(int) * p_dataset->x_pixsize
		* p_dataset->y_pixsize);
	if (NULL == p_dataset->p_avg_v)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	/* Average Number (H)
	 * (type size * X pixel * Y pixel ) */
	p_dataset->p_avg_h = (int *) malloc(sizeof(int) * p_dataset->x_pixsize
		* p_dataset->y_pixsize);
	if (NULL == p_dataset->p_avg_h)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Total Number (V)
	 * (type size * X pixel * Y pixel ) */
	p_dataset->p_total_v = (int *) malloc(sizeof(int) * p_dataset->x_pixsize
		* p_dataset->y_pixsize);
	if (NULL == p_dataset->p_total_v)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	/* Total Number (H)
	 * (type size * X pixel * Y pixel ) */
	p_dataset->p_total_h = (int *) malloc(sizeof(int) * p_dataset->x_pixsize
		* p_dataset->y_pixsize);
	if (NULL == p_dataset->p_total_h)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	return RET_NORMAL;
}


/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset)
{
	int ret = 0;
	
	if (0 <= file_id)
	{
		/* HDF product file close (write mode) */
		ret = AMTK_closeH5_Write(file_id);
		if (0 > ret)
		{
			E_MSG("AMTK_closeH5_Write() error.[%d]\n", ret);
		}
	}
	
	/* free memory */
	free(p_dataset->p_tb_v);
	p_dataset->p_tb_v = NULL;
	
	free(p_dataset->p_tb_h);
	p_dataset->p_tb_h = NULL;
	
	free(p_dataset->p_std_dev_v);
	p_dataset->p_std_dev_v = NULL;
	
	free(p_dataset->p_std_dev_h);
	p_dataset->p_std_dev_h = NULL;
	
	free(p_dataset->p_avg_v);
	p_dataset->p_avg_v = NULL;
	
	free(p_dataset->p_avg_h);
	p_dataset->p_avg_h = NULL;
	
	free(p_dataset->p_total_v);
	p_dataset->p_total_v = NULL;
	
	free(p_dataset->p_total_h);
	p_dataset->p_total_h = NULL;
	
	return;
}


/** Set dataset value.
 * @param p_dataset [OUT] Dataset
 */
static void set_value(DATASET_MODEL *p_dataset)
{
	int x = 0;
	int y = 0;
	
	for (x = 0; x < p_dataset->x_pixsize; x++)
	{
		for (y = 0; y < p_dataset->y_pixsize; y++)
		{
			/* Brightness Temperature (V) */
			*(p_dataset->p_tb_v + (x * p_dataset->y_pixsize) + y)
				= (x + y) * 0.01;
			/* Brightness Temperature (H) */
			*(p_dataset->p_tb_h + (x * p_dataset->y_pixsize) + y)
				= (x + y) * 0.01;
			
			/* Standard Deviation (V) */
			*(p_dataset->p_std_dev_v + (x * p_dataset->y_pixsize) + y)
				= (x + y) * 0.01;
			/* Standard Deviation (H) */
			*(p_dataset->p_std_dev_h + (x * p_dataset->y_pixsize) + y)
				= (x + y) * 0.01;
			
			/* Average Number (V) */
			*(p_dataset->p_avg_v + (x * p_dataset->y_pixsize) + y) = x + y;
			/* Average Number (H) */
			*(p_dataset->p_avg_h + (x * p_dataset->y_pixsize) + y) = x + y;
			
			/* Total Number (V) */
			*(p_dataset->p_total_v + (x * p_dataset->y_pixsize) + y) = x + y;
			/* Total Number (H) */
			*(p_dataset->p_total_h + (x * p_dataset->y_pixsize) + y) = x + y;
			
		}
	}
	
	return;
}

