/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2010 All rights reserved.                  */
/*  File Name    : sample7_read_L1Bproduct.c                                 */
/*  Abstract     : I/O toolkit sample program                                */
/*  Function List                                                            */
/*      main()                       : Sample program main                   */
/*      initialize()                 : Initialization                        */
/*      terminate()                  : Termination                           */
/*      get_value()                  : Get dataset value                     */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2012/12/03                                                    */
/*  Version    1.11                                                          */
/*  Notice                                                                   */
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/** Include AMTK.h */
#ifdef __cplusplus
extern "C" {
#endif
#include "AMTK.h"
#ifdef __cplusplus
}
#endif

#include "sample_common.h"

/*---------------------------------------------------------------------------*
 * Common definition                                                         *
 *---------------------------------------------------------------------------*/
 
/* PRINT_SCAN_NUM defines print data number */
#define PRINT_SCAN_NUM        (2)

/** Sample L1R product file name. */
#define FN_SAMPLE_PRODUCT_L1B ("../data/sample2_IOToolKit_l1b.h5")
/** Metadata definition. */
#define FN_SAMPLE_META_L1B    ("../data/metadata_sample_L1B.txt")

/** Number of scans. */
#define META_NUMSCAN_NAME     ("NumberOfScans")
/** Overlap scans. */
#define META_OVERLAP_NAME     ("OverlapScans")


/** Brightness Temperature data model. */
typedef struct
{
	/* Access Label. */
	const int access_label;
	/* Brightness Tempertature Data. */
	float *p_data;
	/* Dataset Name. */
	const char *p_dataset_name;
	
} TB_DATA;


/** Structure of dataset model. */
typedef struct
{
	/* Scan Time */
	AM2_COMMON_SCANTIME *p_scan_time;
	
	/* Position in Orbit */
	double *p_pos_orbit;
	/* Navigation Data */
	float *p_navi;
	/* Attitude Data */
	float *p_att;
	
	/* BrightHotness Temperature <Low resolution> */
	TB_DATA *p_low_tb_table;
	int low_tb_num;
	/* Brightness Temperature <High resolution>  */
	TB_DATA *p_high_tb_table;
	int high_tb_num;
	
	/* Hot Load Count 6 to 36 */
	int *p_hts_low;
	/* Hot Load Count 89 */
	int *p_hts_high;
	/* Cold Sky Mirror Count 6 to 36 */
	int *p_csm_low;
	/* Cold Sky Mirror Count 89 */
	int *p_csm_high;
	/* Rx Offset_Gain Count */
	int *p_rx_offset_gain;
	
	/* Latitude of Observation Point for 89A
	 * Longitude of Observation Point for 89A */
	AM2_COMMON_LATLON *p_latlon89A;
	/* Latitude of Observation Point for 89B
	 * Longitude of Observation Point for 89B */
	AM2_COMMON_LATLON *p_latlon89B;
	
	/* Sun Azimuth */
	float *p_sun_azimuth;
	/* Sun Elevation */
	float *p_sun_elevation;
	/* Earth Incidencet */
	float *p_earth_incidence;
	/* Earth Azimuth */
	float *p_earth_azimuth;
	/* Land_Ocean Flag 6 to 36 */
	int *p_lof_low;
	/* Land_Ocean Flag 89 */
	int *p_lof_high;
	/* Observation Supplement */
	unsigned char *p_obs_spl;
	/* SPC Temperature Count */
	float *p_spc_temp;
	/* SPS Temperature Count */
	float *p_sps_temp;
	/* PCD Data */
	unsigned char *p_pcd_data;
	
	/* Scan Data Quality */
	AMTK_SCAN_DATA_QUALITY *p_scan_quality;
	
	/* Pixel Data Quality 6 to 36 */
	unsigned char *p_pixel_quality_low;
	/* Pixel Data Quality 89 */
	unsigned char *p_pixel_quality_high;
	/* Interpolation Flag 6 to 36 */
	unsigned char *p_intpol_low;
	/* Interpolation Flag 89 */
	unsigned char *p_intpol_high;
	
} DATASET_MODEL;


/** Initialization.
 * @param file_id   [IN]  Product file id
 * @param scan_size [IN]  Number of scans
 * @param p_dataset [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(hid_t file_id, int scan_size, DATASET_MODEL *p_dataset);

/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset);

/** Get dataset value.
 * @param scan_size [IN] Number of scans
 * @param p_dataset [IN] Dataset
 */
static void get_value(int scan_size, const DATASET_MODEL *p_dataset);


/*---------------------------------------------------------------------------*
 * Main program                                                              *
 *---------------------------------------------------------------------------*/
 
/** AMSR2 I/O toolkit sample program.
 * sample7: input L1B proudct.
 * 
 * @param  argc [IN] Argument count
 * @param  argv [IN] Argument value
 * @return EXIT_SUCCESS
 * @return EXIT_FAILURE
 */
int main(int argc, char *argv[])
{
	hid_t file_id = 0;
	
	int i = 0;
	int meta_num = 0;
	int ret = 0;
	
	/* sample meta data read */
	char meta_name[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
	char meta_value[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
	
	char buffer[META_DATA_LEN_MAX] = "";
	char *p_value = NULL;
	
	/* Number of Scans */
	int num_scans = 0;
	int num_overlap_scans = 0;
	int scan_size = 0;
	int scan_start = 0;
	int scan_end = 0;
	
	/* Dataset. */
	DATASET_MODEL dataset = {0};
	
	
	/* Brightness Temperature <Low resolution> */
	TB_DATA low_tb_table[] =
	{
		{AM2_TB06V, NULL, "Brightness Temperature (6.9GHz,V)"},
		{AM2_TB06H, NULL, "Brightness Temperature (6.9GHz,H)"},
		{AM2_TB07V, NULL, "Brightness Temperature (7.3GHz,V)"},
		{AM2_TB07H, NULL, "Brightness Temperature (7.3GHz,H)"},
		{AM2_TB10V, NULL, "Brightness Temperature (10.7GHz,V)"},
		{AM2_TB10H, NULL, "Brightness Temperature (10.7GHz,H)"},
		{AM2_TB18V, NULL, "Brightness Temperature (18.7GHz,V)"},
		{AM2_TB18H, NULL, "Brightness Temperature (18.7GHz,H)"},
		{AM2_TB23V, NULL, "Brightness Temperature (23.8GHz,V)"},
		{AM2_TB23H, NULL, "Brightness Temperature (23.8GHz,H)"},
		{AM2_TB36V, NULL, "Brightness Temperature (36.5GHz,V)"},
		{AM2_TB36H, NULL, "Brightness Temperature (36.5GHz,H)"},
	};
	int low_tb_num = sizeof(low_tb_table) / sizeof(low_tb_table[0]);
	
	/* Brightness Temperature <High resolution> */
	TB_DATA high_tb_table[] =
	{
		{AM2_TB89AV, NULL, "Brightness Temperature (89.0GHz-A,V)"},
		{AM2_TB89AH, NULL, "Brightness Temperature (89.0GHz-A,H)"},
		{AM2_TB89BV, NULL, "Brightness Temperature (89.0GHz-B,V)"},
		{AM2_TB89BH, NULL, "Brightness Temperature (89.0GHz-B,H)"},
	};
	int high_tb_num = sizeof(high_tb_table) / sizeof(high_tb_table[0]);
	
	/* Set Brightness Temperature data */
	dataset.p_low_tb_table = low_tb_table;
	dataset.low_tb_num = low_tb_num;
	
	dataset.p_high_tb_table = high_tb_table;
	dataset.high_tb_num = high_tb_num;
	
	
	/* HDF product file open */
	file_id = AMTK_openH5(FN_SAMPLE_PRODUCT_L1B);
	if (0 > file_id)
	{
		if (-501 == file_id)
		{
			printf("Product file not found.[%s]\n", FN_SAMPLE_PRODUCT_L1B);
		}
		else
		{
			E_MSG("AMTK_openH5() error.[%d]\n", ret);
		}
		exit(EXIT_FAILURE);
	}
	
	/*------------------------------------------------------------------------*
	 * memory allocation                                                      *
	 *------------------------------------------------------------------------*/
	p_value = buffer;
	
	/* Get "NumberOfScans" */
	ret = AMTK_getMetaDataName(file_id, META_NUMSCAN_NAME, &p_value);
	if (0 > ret)
	{
		E_MSG("AMTK_getMetaDataName() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	num_scans = atoi(buffer);
	
	/* Get "OverlapScans" */
	ret = AMTK_getMetaDataName(file_id, META_OVERLAP_NAME, &p_value);
	if (0 > ret)
	{
		E_MSG("AMTK_getMetaDataName() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	num_overlap_scans = atoi(buffer);
	
	/* Get scan size. */
	scan_size = num_scans + num_overlap_scans * 2;
	scan_start = 1 - num_overlap_scans;
	scan_end = num_scans + num_overlap_scans;
	
	printf("Scan size = %d, Scan Start = %d, Scan End = %d\n", scan_size,
		scan_start, scan_end);
	
	/* memory allocation */
	ret = initialize(file_id, scan_size, &dataset);
	if (ret != RET_NORMAL)
	{
		E_MSG("initialize() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/*------------------------------------------------------------------------*
	 * input data                                                             *
	 *------------------------------------------------------------------------*/
	/* Read Metadata from Metadata file */
	if (RET_NORMAL != get_metadata_sample(FN_SAMPLE_META_L1B, meta_name,
		meta_value, &meta_num))
	{
		E_MSG("get_metadata_sample() error.\n");
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* meta data read */
	for (i = 0; i < meta_num; i++)
	{
		p_value = &meta_value[i][0];
		ret = AMTK_getMetaDataName(file_id, &meta_name[i][0], &p_value);
		if (0 > ret)
		{
			E_MSG("AMTK_getMetaDataName() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		printf("No=%d\tName=%s\tValue=%s\n", i, &meta_name[i][0], p_value);
	}
	
	/* Data set read */
	/* Scan Time */
	ret = AMTK_getScanTime(file_id, scan_start, scan_end, &dataset.p_scan_time);
	if (0 > ret)
	{
		E_MSG("AMTK_getScanTime() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Position in Orbit */
	ret = AMTK_get_SwathDouble(file_id, &dataset.p_pos_orbit, scan_start,
		scan_end, AM2_POS_ORBIT);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathDouble() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Navigation Data */
	ret = AMTK_get_SwathFloat(file_id, &dataset.p_navi, scan_start, scan_end,
		AM2_NAVI);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Attitude Data */
	ret = AMTK_get_SwathFloat(file_id, &dataset.p_att, scan_start, scan_end,
		AM2_ATT);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Brightness Temperature (6.9GHz,V) ... (36.5GHz,H) */
	for (i = 0; i < low_tb_num; i++)
	{
		ret = AMTK_get_SwathFloat(file_id, &(low_tb_table[i].p_data),
			scan_start, scan_end, low_tb_table[i].access_label);
		if (0 > ret)
		{
			E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Brightness Temperature (89.0GHz-A,V) ... (89.0GHz-B,H) */
	for (i = 0; i < high_tb_num; i++)
	{
		ret = AMTK_get_SwathFloat(file_id, &(high_tb_table[i].p_data),
			scan_start, scan_end, high_tb_table[i].access_label);
		if (0 > ret)
		{
			E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Hot Load Count 6 to 36 */
	ret = AMTK_get_SwathInt(file_id, &dataset.p_hts_low, scan_start, scan_end,
		AM2_HTS_LO);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathInt() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Hot Load Count 89 */
	ret = AMTK_get_SwathInt(file_id, &dataset.p_hts_high, scan_start, scan_end,
		AM2_HTS_HI);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathInt() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Cold Sky Mirror Count 6 to 36 */
	ret = AMTK_get_SwathInt(file_id, &dataset.p_csm_low, scan_start, scan_end,
		AM2_CSM_LO);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathInt() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Cold Sky Mirror Count 89 */
	ret = AMTK_get_SwathInt(file_id, &dataset.p_csm_high, scan_start, scan_end,
		AM2_CSM_HI);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathInt() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Rx Offset_Gain Count */
	ret = AMTK_get_SwathInt(file_id, &dataset.p_rx_offset_gain, scan_start,
		scan_end, AM2_OFF_GAIN);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathInt() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Latitude of Observation Point for 89A
	 * Longitude of Observation Point for 89A */
	ret = AMTK_getLatLon(file_id, &dataset.p_latlon89A, scan_start, scan_end,
		AM2_LATLON_89A);
	if (0 > ret)
	{
		E_MSG("AMTK_getLatLon() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
		
	/* Latitude of Observation Point for 89B
	 * Longitude of Observation Point for 89B */
	ret = AMTK_getLatLon(file_id, &dataset.p_latlon89B, scan_start, scan_end,
		AM2_LATLON_89B);
	if (0 > ret)
	{
		E_MSG("AMTK_getLatLon() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Sun Azimuth */
	ret = AMTK_get_SwathFloat(file_id, &dataset.p_sun_azimuth, scan_start,
		scan_end, AM2_SUN_AZ);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Sun Elevation */
	ret = AMTK_get_SwathFloat(file_id, &dataset.p_sun_elevation, scan_start,
		scan_end, AM2_SUN_EL);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Earth Incidence */
	ret = AMTK_get_SwathFloat(file_id, &dataset.p_earth_incidence, scan_start,
		scan_end, AM2_EARTH_INC);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Earth Azimuth */
	ret = AMTK_get_SwathFloat(file_id, &dataset.p_earth_azimuth, scan_start,
		scan_end, AM2_EARTH_AZ);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Land_Ocean Flag 6 to 36 */
	ret = AMTK_get_SwathInt(file_id, &dataset.p_lof_low, scan_start, scan_end,
		AM2_LOF_LO);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathInt() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Land_Ocean Flag 89 */
	ret = AMTK_get_SwathInt(file_id, &dataset.p_lof_high, scan_start, scan_end,
		AM2_LOF_HI);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathInt() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Observation Supplement */
	ret = AMTK_get_SwathUChar(file_id, &dataset.p_obs_spl, scan_start, scan_end,
		AM2_OB_SPL);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathUChar() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* SPC Temperature Count */
	ret = AMTK_get_SwathFloat(file_id, &dataset.p_spc_temp, scan_start, scan_end,
		AM2_SPC_TEMP);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* SPS Temperature Count */
	ret = AMTK_get_SwathFloat(file_id, &dataset.p_sps_temp, scan_start, scan_end,
		AM2_SPS_TEMP);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* PCD Data */
	ret = AMTK_get_SwathUChar(file_id, &dataset.p_pcd_data, scan_start, scan_end,
		AM2_PCD);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathUChar() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Scan Data Quality */
	ret = AMTK_get_ScanDataQuality(file_id, &dataset.p_scan_quality, scan_start,
		scan_end, AM2_SCAN_QUAL);
	if (0 > ret)
	{
		E_MSG("AMTK_get_ScanDataQuality() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Pixel Data Quality 6 to 36 */
	ret = AMTK_get_SwathUChar(file_id, &dataset.p_pixel_quality_low, scan_start,
		scan_end, AM2_PIX_QUAL_LO);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathUChar() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Pixel Data Quality 89 */
	ret = AMTK_get_SwathUChar(file_id, &dataset.p_pixel_quality_high,
		scan_start, scan_end, AM2_PIX_QUAL_HI);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathUChar() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Interpolation Flag 6 to 36 */
	ret = AMTK_get_SwathUChar(file_id, &dataset.p_intpol_low, scan_start,
		scan_end, AM2_INTPL_LO);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathUChar() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Interpolation Flag 89 */
	ret = AMTK_get_SwathUChar(file_id, &dataset.p_intpol_high, scan_start,
		scan_end, AM2_INTPL_HI);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathUChar() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* get dataset value */
	get_value(scan_size, &dataset);
	
	/* termination */
	terminate(file_id, &dataset);
	
	printf("L1B product read end\n");
	
	exit(EXIT_SUCCESS);
}


/** Initialization.
 * @param file_id   [IN]  Product file id
 * @param scan_size [IN]  Number of scans
 * @param p_dataset [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(hid_t file_id, int scan_size, DATASET_MODEL *p_dataset)
{
	int ret = 0;
	int dimsize[3] = {0};
	int i = 0;

	/* memory allocation */
	
	/* Scan Time
	 * (type size * Number of scans) */
	ret = AMTK_getDimSize(file_id, AM2_SCAN_TIME_DEF, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_scan_time = (AM2_COMMON_SCANTIME *) malloc(
		sizeof(AM2_COMMON_SCANTIME) * get_memory_size(dimsize));
	if (NULL == p_dataset->p_scan_time)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Position in Orbit
	 * (type size * Number of scans) */
	ret = AMTK_getDimSize(file_id, AM2_POS_ORBIT, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_pos_orbit = (double *) malloc(sizeof(double)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_pos_orbit)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Navigation Data
	 * (type size * Number of scans * 6) */
	ret = AMTK_getDimSize(file_id, AM2_NAVI, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_navi = (float *) malloc(sizeof(float)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_navi)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Attitude Data
	 * (type size * Number of scans * 3) */
	ret = AMTK_getDimSize(file_id, AM2_ATT, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_att = (float *) malloc(sizeof(float)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_att)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Brightness Temperature (6.9GHz,V) ... (36.5GHz,H)
	 * (type size * Number of scans * 243) */
	for (i = 0; i < p_dataset->low_tb_num; i++)
	{
		ret = AMTK_getDimSize(file_id,
			p_dataset->p_low_tb_table[i].access_label, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
			return RET_ERROR;
		}
		p_dataset->p_low_tb_table[i].p_data = (float *) malloc(sizeof(float)
			* get_memory_size(dimsize));
		if (NULL == p_dataset->p_low_tb_table[i].p_data)
		{
			E_MSG("malloc() error.\n");
			return RET_ERROR;
		}
	}
	
	/* Brightness Temperature (89.0GHz-A,V) ... (89.0GHz-B,H)
	 * (type size * Number of scans * 486) */
	for (i = 0; i < p_dataset->high_tb_num; i++)
	{
		ret = AMTK_getDimSize(file_id,
			p_dataset->p_high_tb_table[i].access_label, dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
			return RET_ERROR;
		}
		p_dataset->p_high_tb_table[i].p_data = (float *) malloc(sizeof(float)
			* get_memory_size(dimsize));
		if (NULL == p_dataset->p_high_tb_table[i].p_data)
		{
			E_MSG("malloc() error.\n");
			return RET_ERROR;
		}
	}
	
	/* Hot Load Count 6 to 36
	 * (type size * 12 * Number of scans * 16) */
	ret = AMTK_getDimSize(file_id, AM2_HTS_LO, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_hts_low = (int *) malloc(sizeof(int)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_hts_low)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Hot Load Count 89
	 * (type size * 4 * Number of scans * 32) */
	ret = AMTK_getDimSize(file_id, AM2_HTS_HI, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_hts_high = (int *) malloc(sizeof(int)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_hts_high)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Cold Sky Mirror Count 6 to 36
	 * (type size * 12 * Number of scans * 16) */
	ret = AMTK_getDimSize(file_id, AM2_CSM_LO, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_csm_low = (int *) malloc(sizeof(int)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_csm_low)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Cold Sky Mirror Count 89
	 * (type size * 4 * Number of scans * 32) */
	ret = AMTK_getDimSize(file_id, AM2_CSM_HI, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_csm_high = (int *) malloc(sizeof(int)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_csm_high)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Rx Offset_Gain Count
	 * (type size * Number of scans * 32) */
	ret = AMTK_getDimSize(file_id, AM2_OFF_GAIN, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_rx_offset_gain = (int *) malloc(sizeof(int)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_rx_offset_gain)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Latitude of Observation Point for 89A
	 * Longitude of Observation Point for 89A
	 * (type size * Number of scans * 486) */
	ret = AMTK_getDimSize(file_id, AM2_LATLON_89A, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_latlon89A = (AM2_COMMON_LATLON *) malloc(
		sizeof(AM2_COMMON_LATLON) * get_memory_size(dimsize));
	if (NULL == p_dataset->p_latlon89A)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Latitude of Observation Point for 89B
	 * Longitude of Observation Point for 89B
	 * (type size * Number of scans * 486) */
	ret = AMTK_getDimSize(file_id, AM2_LATLON_89B, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_latlon89B = (AM2_COMMON_LATLON *) malloc(
		sizeof(AM2_COMMON_LATLON) * get_memory_size(dimsize));
	if (NULL == p_dataset->p_latlon89B)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Sun Azimuth
	 * (type size * Number of scans * 243) */
	ret = AMTK_getDimSize(file_id, AM2_SUN_AZ, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_sun_azimuth = (float *) malloc(sizeof(float)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_sun_azimuth)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Sun Elevation
	 * (type size * Number of scans * 243) */
	ret = AMTK_getDimSize(file_id, AM2_SUN_EL, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_sun_elevation = (float *) malloc(sizeof(float)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_sun_elevation)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Earth Incidence
	 * (type size * Number of scans * 243) */
	ret = AMTK_getDimSize(file_id, AM2_EARTH_INC, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_earth_incidence = (float *) malloc(sizeof(float)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_earth_incidence)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Earth Azimuth
	 * (type size * Number of scans * 243) */
	ret = AMTK_getDimSize(file_id, AM2_EARTH_AZ, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_earth_azimuth = (float *) malloc(sizeof(float)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_earth_azimuth)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Land_Ocean Flag 6 to 36
	 * (type size * 6 * Number of scans * 243) */
	ret = AMTK_getDimSize(file_id, AM2_LOF_LO, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_lof_low = (int *) malloc(sizeof(int)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_lof_low)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Land_Ocean Flag 89
	 * (type size * 2 * Number of scans * 486) */
	ret = AMTK_getDimSize(file_id, AM2_LOF_HI, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_lof_high = (int *) malloc(sizeof(int)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_lof_high)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Observation Supplement
	 * (type size * Number of scans * 248[124 * binary 2byte]) */
	ret = AMTK_getDimSize(file_id, AM2_OB_SPL, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_obs_spl = (unsigned char *) malloc(get_memory_size(dimsize));
	if (NULL == p_dataset->p_obs_spl)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* SPC Temperature Count
	 * (type size * Number of scans * 34) */
	ret = AMTK_getDimSize(file_id, AM2_SPC_TEMP, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_spc_temp = (float *) malloc(sizeof(float)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_spc_temp)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* SPS Temperature Count
	 * (type size * Number of scans * 46) */
	ret = AMTK_getDimSize(file_id, AM2_SPS_TEMP, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_sps_temp = (float *) malloc(sizeof(float)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_sps_temp)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* PCD Data
	 * (type size * Number of scans * 64) */
	ret = AMTK_getDimSize(file_id, AM2_PCD, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_pcd_data = (unsigned char *) malloc(get_memory_size(dimsize));
	if (NULL == p_dataset->p_pcd_data)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Scan Data Quality
	 * (type size * Number of scans) */
	ret = AMTK_getDimSize(file_id, AM2_SCAN_QUAL, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_scan_quality = (AMTK_SCAN_DATA_QUALITY *) malloc(
		sizeof(AMTK_SCAN_DATA_QUALITY) * dimsize[0]);
	if (NULL == p_dataset->p_scan_quality)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Pixel Data Quality 6 to 36
	 * (type size * Numbef of scans * 486[243 * binary 2byte]) */
	ret = AMTK_getDimSize(file_id, AM2_PIX_QUAL_LO, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_pixel_quality_low = (unsigned char *) malloc(
		get_memory_size(dimsize));
	if (NULL == p_dataset->p_pixel_quality_low)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}

	/* Pixel Data Quality 89
	 * (type size * Numbef of scans * 486[486 * binary 1byte]) */
	ret = AMTK_getDimSize(file_id, AM2_PIX_QUAL_HI, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_pixel_quality_high = (unsigned char *) malloc(
		get_memory_size(dimsize));
	if (NULL == p_dataset->p_pixel_quality_high)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Interpolation Flag 6 to 36
	 * (type size * 12 * Numbef of scans * 16) */
	ret = AMTK_getDimSize(file_id, AM2_INTPL_LO, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_intpol_low = (unsigned char *) malloc(
		get_memory_size(dimsize));
	if (NULL == p_dataset->p_intpol_low)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Interpolation Flag 89
	 * (type size * 4 * Numbef of scans * 32) */
	ret = AMTK_getDimSize(file_id, AM2_INTPL_HI, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_intpol_high = (unsigned char *) malloc(
		get_memory_size(dimsize));
	if (NULL == p_dataset->p_intpol_high)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	return RET_NORMAL;
}


/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset)
{
	int i = 0;
	int ret = 0;
	
	if (0 <= file_id)
	{
		/* HDF product file close */
		ret = AMTK_closeH5(file_id);
		if (0 > ret)
		{
			E_MSG("AMTK_closeH5() error.[%d]\n", ret);
		}
	}
	
	/* free memory */
	free(p_dataset->p_scan_time);
	p_dataset->p_scan_time = NULL;
	
	free(p_dataset->p_pos_orbit);
	p_dataset->p_pos_orbit = NULL;
	
	free(p_dataset->p_navi);
	p_dataset->p_navi = NULL;
	
	free(p_dataset->p_att);
	p_dataset->p_att = NULL;
	
	for (i = 0; i < p_dataset->low_tb_num; i++)
	{
		free(p_dataset->p_low_tb_table[i].p_data);
		p_dataset->p_low_tb_table[i].p_data = NULL;
	}
	for (i = 0; i < p_dataset->high_tb_num; i++)
	{
		free(p_dataset->p_high_tb_table[i].p_data);
		p_dataset->p_high_tb_table[i].p_data = NULL;
	}
	
	free(p_dataset->p_hts_low);
	p_dataset->p_hts_low = NULL;
	
	free(p_dataset->p_hts_high);
	p_dataset->p_hts_high = NULL;
	
	free(p_dataset->p_csm_low);
	p_dataset->p_csm_low = NULL;
	
	free(p_dataset->p_csm_high);
	p_dataset->p_csm_high = NULL;
	
	free(p_dataset->p_rx_offset_gain);
	p_dataset->p_rx_offset_gain = NULL;
	
	free(p_dataset->p_latlon89A);
	p_dataset->p_latlon89A = NULL;
	
	free(p_dataset->p_latlon89B);
	p_dataset->p_latlon89B = NULL;
	
	free(p_dataset->p_sun_azimuth);
	p_dataset->p_sun_azimuth = NULL;
	
	free(p_dataset->p_sun_elevation);
	p_dataset->p_sun_elevation = NULL;
	
	free(p_dataset->p_earth_incidence);
	p_dataset->p_earth_incidence = NULL;
	
	free(p_dataset->p_earth_azimuth);
	p_dataset->p_earth_azimuth = NULL;
	
	free(p_dataset->p_lof_low);
	p_dataset->p_lof_low = NULL;
	
	free(p_dataset->p_lof_high);
	p_dataset->p_lof_high = NULL;
	
	free(p_dataset->p_obs_spl);
	p_dataset->p_obs_spl = NULL;
	
	free(p_dataset->p_spc_temp);
	p_dataset->p_spc_temp = NULL;
	
	free(p_dataset->p_sps_temp);
	p_dataset->p_sps_temp = NULL;
	
	free(p_dataset->p_pcd_data);
	p_dataset->p_pcd_data = NULL;
	
	free(p_dataset->p_scan_quality);
	p_dataset->p_scan_quality = NULL;
	
	free(p_dataset->p_pixel_quality_low);
	p_dataset->p_pixel_quality_low = NULL;
	
	free(p_dataset->p_pixel_quality_high);
	p_dataset->p_pixel_quality_high = NULL;
	
	free(p_dataset->p_intpol_low);
	p_dataset->p_intpol_low = NULL;
	
	free(p_dataset->p_intpol_high);
	p_dataset->p_intpol_high = NULL;
	
	return;
}


/** Get dataset value.
 * @param scan_size [IN] Number of scans
 * @param p_dataset [IN] Dataset
 */
static void get_value(int scan_size, const DATASET_MODEL *p_dataset)
{
	int i = 0;
	int scan_num = 0;
	int pixel_num = 0;
	
	for (scan_num = 0; scan_num < PRINT_SCAN_NUM; scan_num++)
	{
		/* Scan Time */
		printf("Scan Time No=%d [%d]   [%02d/%02d]  %02d:%02d:%02d.%03d\n",
			scan_num,
			(p_dataset->p_scan_time + scan_num)->year,
			(p_dataset->p_scan_time + scan_num)->month,
			(p_dataset->p_scan_time + scan_num)->day,
			(p_dataset->p_scan_time + scan_num)->hour,
			(p_dataset->p_scan_time + scan_num)->minute,
			(p_dataset->p_scan_time + scan_num)->second,
			(p_dataset->p_scan_time + scan_num)->ms);
		
		/* Brightness Temperature (6.9GHz,V) ... (36.5GHz,H) */
		for (i = 0; i < p_dataset->low_tb_num; i++)
		{
			/* first pixel */
			pixel_num = 0;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Brightness Temperature %s = %8.2f  \n",
				p_dataset->p_low_tb_table[i].p_dataset_name,
				*(p_dataset->p_low_tb_table[i].p_data
					+ (AM2_DEF_SNUM_LO * scan_num)
					+ pixel_num));
			
			/* last pixel */
			pixel_num = AM2_DEF_SNUM_LO - 1;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Brightness Temperature %s = %8.2f  \n",
				p_dataset->p_low_tb_table[i].p_dataset_name,
				*(p_dataset->p_low_tb_table[i].p_data
					+ (AM2_DEF_SNUM_LO * scan_num)
					+ pixel_num));
		}
		
		/* Brightness Temperature (89.0GHz-A,V) ... (89.0GHz-B,H) */
		for (i = 0; i < p_dataset->high_tb_num; i++)
		{
			/* first pixel */
			pixel_num = 0;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Brightness Temperature %s = %8.2f  \n",
				p_dataset->p_high_tb_table[i].p_dataset_name,
				*(p_dataset->p_high_tb_table[i].p_data
					+ (AM2_DEF_SNUM_HI * scan_num)
					+ pixel_num));
			
			/* last pixel */
			pixel_num = AM2_DEF_SNUM_HI - 1;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Brightness Temperature %s = %8.2f  \n",
				p_dataset->p_high_tb_table[i].p_dataset_name,
				*(p_dataset->p_high_tb_table[i].p_data
					+ (AM2_DEF_SNUM_HI * scan_num)
					+ pixel_num));
		}
		
		
		/* Latitude/Longitude of Observation Point for 89A */
		printf("Latitude/Longitude of Observation Point for 89A\n" );
		/* first pixel */
		pixel_num = 0;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Latitude  = %8.2f  \n",
			(p_dataset->p_latlon89A + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lat);
		printf("Longitude = %8.2f  \n",
			(p_dataset->p_latlon89A + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lon);
		
		/* last pixel */
		pixel_num = AM2_DEF_SNUM_HI - 1;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Latitude  = %8.2f  \n",
			(p_dataset->p_latlon89A + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lat);
		printf("Longitude = %8.2f  \n",
			(p_dataset->p_latlon89A + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lon);
		
		/* Latitude/Longitude of Observation Point for 89B */
		printf("Latitude/Longitude of Observation Point for 89B\n" );
		/* first pixel */
		pixel_num = 0;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Latitude  = %8.2f  \n",
			(p_dataset->p_latlon89B + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lat);
		printf("Longitude = %8.2f  \n",
			(p_dataset->p_latlon89B + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lon);
		
		/* last pixel */
		pixel_num = AM2_DEF_SNUM_HI - 1;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Latitude  = %8.2f  \n",
			(p_dataset->p_latlon89B + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lat);
		printf("Longitude = %8.2f  \n",
			(p_dataset->p_latlon89B + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lon);
		
		/* Land_Ocean Flag 6 to 36 */
		printf("Land_Ocean Flag 6 to 36\n" );
		pixel_num = 0;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Land_Ocean Flag 06GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (0 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		printf("Land_Ocean Flag 07GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (1 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		printf("Land_Ocean Flag 10GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (2 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		printf("Land_Ocean Flag 18GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (3 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		printf("Land_Ocean Flag 23GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (4 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		printf("Land_Ocean Flag 36GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (5 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		
		/* last pixel */
		pixel_num = AM2_DEF_SNUM_LO - 1;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Land_Ocean Flag 06GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (0 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		printf("Land_Ocean Flag 07GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (1 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		printf("Land_Ocean Flag 10GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (2 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		printf("Land_Ocean Flag 18GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (3 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		printf("Land_Ocean Flag 23GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (4 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		printf("Land_Ocean Flag 36GHz = 0x%02x\n",
			*(p_dataset->p_lof_low + (5 * scan_size * AM2_DEF_SNUM_LO) +
				(scan_num * AM2_DEF_SNUM_LO) + pixel_num));
		
		/* Land_Ocean Flag 89 */
		/* first pixel */
		pixel_num = 0;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Land_Ocean Flag 89AGHz = 0x%02x\n",
			*(p_dataset->p_lof_high + (0 * scan_size * AM2_DEF_SNUM_HI) +
				(scan_num * AM2_DEF_SNUM_HI) + pixel_num));
		printf("Land_Ocean Flag 89BGHz = 0x%02x\n",
			*(p_dataset->p_lof_high + (1 * scan_size * AM2_DEF_SNUM_HI) +
				(scan_num * AM2_DEF_SNUM_HI) + pixel_num));
		
		/* last pixel */
		pixel_num = AM2_DEF_SNUM_HI - 1;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Land_Ocean Flag 89AGHz = 0x%02x\n",
			*(p_dataset->p_lof_high + (0 * scan_size * AM2_DEF_SNUM_HI) +
				(scan_num * AM2_DEF_SNUM_HI) + pixel_num));
		printf("Land_Ocean Flag 89BGHz = 0x%02x\n",
			*(p_dataset->p_lof_high + (1 * scan_size * AM2_DEF_SNUM_HI) +
				(scan_num * AM2_DEF_SNUM_HI) + pixel_num));
	}
	
	/* The following data will not print, so each function use same memory area.*/
	/* Just show as access example */
	
	return;
}

