/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2010 All rights reserved.                  */
/*  File Name    : sample8_read_L2Lproduct.c                                 */
/*  Abstract     : I/O toolkit sample program                                */
/*  Function List                                                            */
/*      main()                       : Sample program main                   */
/*      initialize()                 : Initialization                        */
/*      terminate()                  : Termination                           */
/*      get_value()                  : Get dataset value                     */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2011/01/26                                                    */
/*  Version    1.08                                                          */
/*  Notice                                                                   */
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/** Include AMTK.h */
#ifdef __cplusplus
extern "C" {
#endif
#include "AMTK.h"
#ifdef __cplusplus
}
#endif

#include "sample_common.h"

/*---------------------------------------------------------------------------*
 * Common definition                                                         *
 *---------------------------------------------------------------------------*/
 
/* PRINT_SCAN_NUM defines print data number */
#define PRINT_SCAN_NUM        (2)
/** Number of Geophysical Data. */
#define MAX_GEO_NUM           (3)

/** Sample L1R product file name. */
#define FN_SAMPLE_PRODUCT_L2L ("../data/sample3_IOToolKit_l2l.h5")
/** Metadata definition. */
#define FN_SAMPLE_META_L2L    ("../data/metadata_sample_L2L.txt")

/** Number of scans. */
#define META_NUMSCAN_NAME     ("NumberOfScans")
/** Overlap scans. */
#define META_OVERLAP_NAME     ("OverlapScans")


/** Structure of dataset model. */
typedef struct
{
	/* Scan Time */
	AM2_COMMON_SCANTIME *p_scan_time;
	
	/* Position in Orbit */
	double *p_pos_orbit;
	
	/* Geophysical Data (Layer: 1...3) */
	float *p_geo_data[MAX_GEO_NUM];
	
	/* Number of Geophysical Data. */
	int geo_data_num;
	
	/* Latitude of Observation Point
	 * Longitude of Observation Point */
	AM2_COMMON_LATLON *p_latlon;
	
	/* Pixel Data Quality */
	unsigned char *p_pixel_quality;
	
} DATASET_MODEL;


/** Initialization.
 * @param file_id   [IN]  Product file id
 * @param scan_size [IN]  Number of scans
 * @param p_dataset [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(hid_t file_id, int scan_size, DATASET_MODEL *p_dataset);

/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset);

/** Get dataset value.
 * @param scan_size [IN] Number of scans
 * @param p_dataset [IN] Dataset
 */
static void get_value(int scan_size, const DATASET_MODEL *p_dataset);


/*---------------------------------------------------------------------------*
 * Main program                                                              *
 *---------------------------------------------------------------------------*/
 
/** AMSR2 I/O toolkit sample program.
 * sample8: input L2(Low) proudct.
 * 
 * @param  argc [IN] Argument count
 * @param  argv [IN] Argument value
 * @return EXIT_SUCCESS
 * @return EXIT_FAILURE
 */
int main(int argc, char *argv[])
{
	hid_t file_id = 0;
	
	int i = 0;
	int meta_num = 0;
	int ret = 0;
	
	/* sample meta data read */
	char meta_name[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
	char meta_value[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
	
	char buffer[META_DATA_LEN_MAX] = "";
	char *p_value = NULL;
	
	/* Number of Scans */
	int num_scans = 0;
	int num_overlap_scans = 0;
	int scan_size = 0;
	int scan_start = 0;
	int scan_end = 0;
	
	/* Access label: Geophysical Data */
	const int geo_data_label[] = {AM2_SWATH_GEO1, AM2_SWATH_GEO2, AM2_SWATH_GEO3};
	
	/* Dataset. */
	DATASET_MODEL dataset = {0};
	
	
	/* HDF product file open */
	file_id = AMTK_openH5(FN_SAMPLE_PRODUCT_L2L);
	if (0 > file_id)
	{
		if (-501 == file_id)
		{
			printf("Product file not found.[%s]\n", FN_SAMPLE_PRODUCT_L2L);
		}
		else
		{
			E_MSG("AMTK_openH5() error.[%d]\n", ret);
		}
		exit(EXIT_FAILURE);
	}
	
	/*------------------------------------------------------------------------*
	 * memory allocation                                                      *
	 *------------------------------------------------------------------------*/
	p_value = buffer;
	
	/* Get "NumberOfScans" */
	ret = AMTK_getMetaDataName(file_id, META_NUMSCAN_NAME, &p_value);
	if (0 > ret)
	{
		E_MSG("AMTK_getMetaDataName() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	num_scans = atoi(buffer);
	
	/* Get "OverlapScans" */
	ret = AMTK_getMetaDataName(file_id, META_OVERLAP_NAME, &p_value);
	if (0 > ret)
	{
		E_MSG("AMTK_getMetaDataName() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	num_overlap_scans = atoi(buffer);
	
	/* Get scan size. */
	scan_size = num_scans + num_overlap_scans * 2;
	scan_start = 1 - num_overlap_scans;
	scan_end =  num_scans + num_overlap_scans;
	
	printf("Scan size = %d, Scan Start = %d, Scan End = %d\n", scan_size,
		scan_start, scan_end);
	
	/* memory allocation */
	ret = initialize(file_id, scan_size, &dataset);
	if (ret != RET_NORMAL)
	{
		E_MSG("initialize() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/*------------------------------------------------------------------------*
	 * input data                                                             *
	 *------------------------------------------------------------------------*/
	/* Read Metadata from Metadata file */
	if (RET_NORMAL != get_metadata_sample(FN_SAMPLE_META_L2L, meta_name,
		meta_value, &meta_num))
	{
		E_MSG("get_metadata_sample() error.\n");
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* meta data read */
	for (i = 0; i < meta_num; i++)
	{
		p_value = &meta_value[i][0];
		ret = AMTK_getMetaDataName(file_id, &meta_name[i][0], &p_value);
		if (0 > ret)
		{
			E_MSG("AMTK_getMetaDataName() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		printf("No=%d\tName=%s\tValue=%s\n", i, &meta_name[i][0], p_value);
	}
	
	/* Data set read */
	/* Scan Time */
	ret = AMTK_getScanTime(file_id, scan_start, scan_end, &dataset.p_scan_time);
	if (0 > ret)
	{
		E_MSG("AMTK_getScanTime() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Position in Orbit */
	ret = AMTK_get_SwathDouble(file_id, &dataset.p_pos_orbit, scan_start,
		scan_end, AM2_POS_ORBIT);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathDouble() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Geophysical Data (Layer: 1...3) */
	for (i = 0; i < dataset.geo_data_num; i++)
	{
		ret = AMTK_get_SwathFloat(file_id, &(dataset.p_geo_data[i]),
			scan_start, scan_end, geo_data_label[i]);
		if (0 > ret)
		{
			E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Latitude of Observation Point
	 * Longitude of Observation Point */
	ret = AMTK_getLatLon(file_id, &dataset.p_latlon, scan_start, scan_end,
		AM2_LATLON_L2_LO);
	if (0 > ret)
	{
		E_MSG("AMTK_getLatLon() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Pixel Data Quality */
	ret = AMTK_get_SwathUChar(file_id, &dataset.p_pixel_quality, scan_start,
		scan_end, AM2_PIX_QUAL);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathUChar() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* get dataset value */
	get_value(scan_size, &dataset);
	
	/* termination */
	terminate(file_id, &dataset);
	
	printf("L2(Low) product read end\n");
	
	exit(EXIT_SUCCESS);
}


/** Initialization.
 * @param file_id   [IN]  Product file id
 * @param scan_size [IN]  Number of scans
 * @param p_dataset [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(hid_t file_id, int scan_size, DATASET_MODEL *p_dataset)
{
	int ret = 0;
	int dimsize[3] = {0};
	int i = 0;
	
	/* Access label: Geophysical Data */
	const int geo_data_label[] = {AM2_SWATH_GEO1, AM2_SWATH_GEO2, AM2_SWATH_GEO3};
	
	/* memory allocation */
	
	/* Scan Time
	 * (type size * Number of scans) */
	/* get dimension size */
	ret = AMTK_getDimSize(file_id, AM2_SCAN_TIME_DEF, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_scan_time = (AM2_COMMON_SCANTIME *) malloc(
		sizeof(AM2_COMMON_SCANTIME) * get_memory_size(dimsize));
	if (NULL == p_dataset->p_scan_time)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Position in Orbit
	 * (type size * Number of scans) */
	ret = AMTK_getDimSize(file_id, AM2_POS_ORBIT, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_pos_orbit = (double *) malloc(sizeof(double)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_pos_orbit)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Geophysical Data (1...3) */
	
	/* Layer: ALL -> AM2_SWATH_GEOA
	 * (type size * Number of scans * 243 * Number of Geophysical Data[1...3]) */
	ret = AMTK_getDimSize(file_id, AM2_SWATH_GEOA, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	/*
	 * get Number of Geophysical Data
	 * # dimsize[0] : Number of Scans
	 * # dimsize[1] : Number of Pixel
	 * # dimsize[2] : Number of Geophysical Data
	 */
	p_dataset->geo_data_num = dimsize[2];
	if ((0 > p_dataset->geo_data_num) || (MAX_GEO_NUM < p_dataset->geo_data_num))
	{
		/* out of bounds(MAX:3) */
		E_MSG("Illegal error: Number of Geophysical Data:%d\n",
			p_dataset->geo_data_num);
		return RET_ERROR;
	}
	for (i = 0; i < p_dataset->geo_data_num; i++)
	{
		/*  Layer: 1...3 -> AM2_SWATH_GEO1 ... AM2_SWATH_GEO3
		 * (type size * Number of scans * 243) */
		ret = AMTK_getDimSize(file_id, geo_data_label[i], dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
			return RET_ERROR;
		}
		p_dataset->p_geo_data[i] = (float *) malloc(sizeof(float)
			* get_memory_size(dimsize));
		if (NULL == p_dataset->p_geo_data[i])
		{
			E_MSG("malloc() error.\n");
			return RET_ERROR;
		}
	}
	
	/* Latitude of Observation Point
	 * Longitude of Observation Point
	 * (type size * Number of scans * 243) */
	ret = AMTK_getDimSize(file_id, AM2_LATLON_L2_LO, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_latlon = (AM2_COMMON_LATLON *) malloc(
		sizeof(AM2_COMMON_LATLON) * get_memory_size(dimsize));
	if (NULL == p_dataset->p_latlon)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Pixel Data Quality
	 * (type size * Number of Geophysical Data[1...3] * Numbef of scans * 243) */
	ret = AMTK_getDimSize(file_id, AM2_PIX_QUAL, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_pixel_quality = (unsigned char *) malloc(
		get_memory_size(dimsize));
	if (NULL == p_dataset->p_pixel_quality)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	
	return RET_NORMAL;
}


/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset)
{
	int i = 0;
	int ret = 0;
	
	if (0 <= file_id)
	{
		/* HDF product file close */
		ret = AMTK_closeH5(file_id);
		if (0 > ret)
		{
			E_MSG("AMTK_closeH5() error.[%d]\n", ret);
		}
	}
	
	/* free memory */
	free(p_dataset->p_scan_time);
	p_dataset->p_scan_time = NULL;
	
	free(p_dataset->p_pos_orbit);
	p_dataset->p_pos_orbit = NULL;
	
	for (i = 0; i < p_dataset->geo_data_num; i++)
	{
		free(p_dataset->p_geo_data[i]);
		p_dataset->p_geo_data[i] = NULL;
	}
	
	free(p_dataset->p_latlon);
	p_dataset->p_latlon = NULL;
	
	free(p_dataset->p_pixel_quality);
	p_dataset->p_pixel_quality = NULL;
	
	return;
}


/** Get dataset value.
 * @param scan_size [IN] Number of scans
 * @param p_dataset [IN] Dataset
 */
static void get_value(int scan_size, const DATASET_MODEL *p_dataset)
{
	int i = 0;
	int scan_num = 0;
	int pixel_num = 0;
	
	for (scan_num = 0; scan_num < PRINT_SCAN_NUM; scan_num++)
	{
		/* Scan Time */
		printf("Scan Time No=%d [%d]   [%02d/%02d]  %02d:%02d:%02d.%03d\n",
			scan_num,
			(p_dataset->p_scan_time + scan_num)->year,
			(p_dataset->p_scan_time + scan_num)->month,
			(p_dataset->p_scan_time + scan_num)->day,
			(p_dataset->p_scan_time + scan_num)->hour,
			(p_dataset->p_scan_time + scan_num)->minute,
			(p_dataset->p_scan_time + scan_num)->second,
			(p_dataset->p_scan_time + scan_num)->ms);
		
		/* Geophysical Data */
		for (i = 0; i < p_dataset->geo_data_num; i++)
		{
			/* first pixel */
			pixel_num = 0;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Geophysical Data (Layer:%d) = %8.4f\n", i + 1,
				*(p_dataset->p_geo_data[i]
					+ (AM2_DEF_SNUM_LO * scan_num)
					+ pixel_num));
			
			/* last pixel */
			pixel_num = AM2_DEF_SNUM_LO - 1;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Geophysical Data (Layer:%d) = %8.4f\n", i + 1,
				*(p_dataset->p_geo_data[i]
					+ (AM2_DEF_SNUM_LO * scan_num)
					+ pixel_num));
		}
		
		/* Latitude/Longitude of Observation Point */
		printf("Latitude/Longitude of Observation Point\n" );
		/* first pixel */
		pixel_num = 0;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Latitude  = %8.2f\n",
			(p_dataset->p_latlon + (AM2_DEF_SNUM_LO * scan_num) +
				pixel_num)->lat);
		printf("Longitude = %8.2f\n",
			(p_dataset->p_latlon + (AM2_DEF_SNUM_LO * scan_num) +
				pixel_num)->lon);
		
		/* last pixel */
		pixel_num = AM2_DEF_SNUM_LO - 1;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Latitude  = %8.2f\n",
			(p_dataset->p_latlon + (AM2_DEF_SNUM_LO * scan_num) +
				pixel_num)->lat);
		printf("Longitude = %8.2f\n",
			(p_dataset->p_latlon + (AM2_DEF_SNUM_LO * scan_num) +
				pixel_num)->lon);
		
		/* Pixel Data Quality */
		for (i = 0; i < p_dataset->geo_data_num; i++)
		{
			/* first pixel */
			pixel_num = 0;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Pixel Data Quality (Layer:%d) = 0x%02x\n", i + 1,
				*(p_dataset->p_pixel_quality
					+ (i * scan_size * AM2_DEF_SNUM_LO)
					+ (scan_num * AM2_DEF_SNUM_LO)
					+ pixel_num));
			
			/* last pixel */
			pixel_num = AM2_DEF_SNUM_LO - 1;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Pixel Data Quality (Layer:%d) = 0x%02x\n", i + 1,
				*(p_dataset->p_pixel_quality
					+ (i * scan_size * AM2_DEF_SNUM_LO)
					+ (scan_num * AM2_DEF_SNUM_LO)
					+ pixel_num));
		}
	}
	
	/* The following data will not print, so each function use same memory area.*/
	/* Just show as access example */
	
	return;
}

