/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2010 All rights reserved.                  */
/*  File Name    : sample9_read_L2Hproduct.c                                 */
/*  Abstract     : I/O toolkit sample program                                */
/*  Function List                                                            */
/*      main()                       : Sample program main                   */
/*      initialize()                 : Initialization                        */
/*      terminate()                  : Termination                           */
/*      get_value()                  : Get dataset value                     */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2011/01/26                                                    */
/*  Version    1.08                                                          */
/*  Notice                                                                   */
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/** Include AMTK.h */
#ifdef __cplusplus
extern "C" {
#endif
#include "AMTK.h"
#ifdef __cplusplus
}
#endif

#include "sample_common.h"

/*---------------------------------------------------------------------------*
 * Common definition                                                         *
 *---------------------------------------------------------------------------*/
 
/** PRINT_SCAN_NUM defines print data number. */
#define PRINT_SCAN_NUM        (2)

/** Number of Geophysical Data. */
#define MAX_GEO_NUM           (3)

/** Sample L1R product file name. */
#define FN_SAMPLE_PRODUCT_L2H ("../data/sample4_IOToolKit_l2h.h5")
/** Metadata definition. */
#define FN_SAMPLE_META_L2H    ("../data/metadata_sample_L2H.txt")

/** Number of scans. */
#define META_NUMSCAN_NAME     ("NumberOfScans")
/** Overlap scans. */
#define META_OVERLAP_NAME     ("OverlapScans")


/** Structure of dataset model. */
typedef struct
{
	/* Scan Time */
	AM2_COMMON_SCANTIME *p_scan_time;
	
	/* Position in Orbit */
	double *p_pos_orbit;
	
	/* Geophysical Data 89A (Layer: 1...3) */
	float *p_geo_data_89A[MAX_GEO_NUM];
	/* Number of Geophysical Data for 89A. */
	int geo_89A_num;
	
	/* Geophysical Data 89B (Layer: 1...3) */
	float *p_geo_data_89B[MAX_GEO_NUM];
	/* Number of Geophysical Data for 89A. */
	int geo_89B_num;
	
	/* Latitude of Observation Point for 89A
	 * Longitude of Observation Point for 89A */
	AM2_COMMON_LATLON *p_latlon_89A;
	/* Latitude of Observation Point for 89B
	 * Longitude of Observation Point for 89B */
	AM2_COMMON_LATLON *p_latlon_89B;
	
	/* Pixel Data Quality for 89A */
	unsigned char *p_pixel_quality_89A;
	/* Number of Geophysical Data(Pixel Data Quality for 89A). */
	int pix_89A_num;
	
	/* Pixel Data Quality for 89B */
	unsigned char *p_pixel_quality_89B;
	/* Number of Geophysical Data(Pixel Data Quality for 89B). */
	int pix_89B_num;
	
} DATASET_MODEL;


/** Initialization.
 * @param file_id   [IN]  Product file id
 * @param scan_size [IN]  Number of scans
 * @param p_dataset [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(hid_t file_id, int scan_size, DATASET_MODEL *p_dataset);

/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset);

/** Get dataset value.
 * @param scan_size [IN] Number of scans
 * @param p_dataset [IN] Dataset
 */
static void get_value(int scan_size, const DATASET_MODEL *p_dataset);


/*---------------------------------------------------------------------------*
 * Main program                                                              *
 *---------------------------------------------------------------------------*/
 
/** AMSR2 I/O toolkit sample program.
 * sample9: input L2(High) proudct.
 * 
 * @param  argc [IN] Argument count
 * @param  argv [IN] Argument value
 * @return EXIT_SUCCESS
 * @return EXIT_FAILURE
 */
int main(int argc, char *argv[])
{
	hid_t file_id = 0;
	
	int i = 0;
	int meta_num = 0;
	int ret = 0;
	
	/* sample meta data read */
	char meta_name[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
	char meta_value[META_DATA_NUM_MAX][META_DATA_LEN_MAX] = {{0}};
	
	char buffer[META_DATA_LEN_MAX] = "";
	char *p_value = NULL;
	
	/* Number of Scans */
	int num_scans = 0;
	int num_overlap_scans = 0;
	int scan_size = 0;
	int scan_start = 0;
	int scan_end = 0;
	
	/* Access label: Geophysical Data */
	const int geo_89A_label[] = {AM2_SWATHA_GEO1, AM2_SWATHA_GEO2, AM2_SWATHA_GEO3};
	const int geo_89B_label[] = {AM2_SWATHB_GEO1, AM2_SWATHB_GEO2, AM2_SWATHB_GEO3};
	
	/* Dataset. */
	DATASET_MODEL dataset = {0};
	
	
	/* HDF product file open */
	file_id = AMTK_openH5(FN_SAMPLE_PRODUCT_L2H);
	if (0 > file_id)
	{
		if (-501 == file_id)
		{
			printf("Product file not found.[%s]\n", FN_SAMPLE_PRODUCT_L2H);
		}
		else
		{
			E_MSG("AMTK_openH5() error.[%d]\n", ret);
		}
		exit(EXIT_FAILURE);
	}
	
	/*------------------------------------------------------------------------*
	 * memory allocation                                                      *
	 *------------------------------------------------------------------------*/
	p_value = buffer;
	
	/* Get "NumberOfScans" */
	ret = AMTK_getMetaDataName(file_id, META_NUMSCAN_NAME, &p_value);
	if (0 > ret)
	{
		E_MSG("AMTK_getMetaDataName() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	num_scans = atoi(buffer);
	
	/* Get "OverlapScans" */
	ret = AMTK_getMetaDataName(file_id, META_OVERLAP_NAME, &p_value);
	if (0 > ret)
	{
		E_MSG("AMTK_getMetaDataName() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	num_overlap_scans = atoi(buffer);
	
	/* Get scan size. */
	scan_size = num_scans + num_overlap_scans * 2;
	scan_start = 1 - num_overlap_scans;
	scan_end =  num_scans + num_overlap_scans;
	
	printf("Scan size = %d, Scan Start = %d, Scan End = %d\n", scan_size,
		scan_start, scan_end);
	
	/* memory allocation */
	ret = initialize(file_id, scan_size, &dataset);
	if (ret != RET_NORMAL)
	{
		E_MSG("initialize() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/*------------------------------------------------------------------------*
	 * input data                                                             *
	 *------------------------------------------------------------------------*/
	/* Read Metadata from Metadata file */
	if (RET_NORMAL != get_metadata_sample(FN_SAMPLE_META_L2H, meta_name,
		meta_value, &meta_num))
	{
		E_MSG("get_metadata_sample() error.\n");
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* meta data read */
	for (i = 0; i < meta_num; i++)
	{
		p_value = &meta_value[i][0];
		ret = AMTK_getMetaDataName(file_id, &meta_name[i][0], &p_value);
		if (0 > ret)
		{
			E_MSG("AMTK_getMetaDataName() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
		printf("No=%d\tName=%s\tValue=%s\n", i, &meta_name[i][0], p_value);
	}
	
	/* Data set read */
	/* Scan Time */
	ret = AMTK_getScanTime(file_id, scan_start, scan_end, &dataset.p_scan_time);
	if (0 > ret)
	{
		E_MSG("AMTK_getScanTime() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Position in Orbit */
	ret = AMTK_get_SwathDouble(file_id, &dataset.p_pos_orbit, scan_start,
		scan_end, AM2_POS_ORBIT);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathDouble() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Geophysical Data for 89A (Layer: 1...3) */
	for (i = 0; i < dataset.geo_89A_num; i++)
	{
		ret = AMTK_get_SwathFloat(file_id, &(dataset.p_geo_data_89A[i]),
			scan_start, scan_end, geo_89A_label[i]);
		if (0 > ret)
		{
			E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Geophysical Data for 89B (Layer: 1...3) */
	for (i = 0; i < dataset.geo_89B_num; i++)
	{
		ret = AMTK_get_SwathFloat(file_id, &(dataset.p_geo_data_89B[i]),
			scan_start, scan_end, geo_89B_label[i]);
		if (0 > ret)
		{
			E_MSG("AMTK_get_SwathFloat() error.[%d]\n", ret);
			terminate(file_id, &dataset);
			exit(EXIT_FAILURE);
		}
	}
	
	/* Latitude of Observation Point for 89A
	 * Longitude of Observation Point for 89A */
	ret = AMTK_getLatLon(file_id, &dataset.p_latlon_89A, scan_start, scan_end,
		AM2_LATLON_L2_89A);
	if (0 > ret)
	{
		E_MSG("AMTK_getLatLon() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Latitude of Observation Point for 89B
	 * Longitude of Observation Point for 89B */
	ret = AMTK_getLatLon(file_id, &dataset.p_latlon_89B, scan_start, scan_end,
		AM2_LATLON_L2_89B);
	if (0 > ret)
	{
		E_MSG("AMTK_getLatLon() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Pixel Data Quality for 89A */
	ret = AMTK_get_SwathUChar(file_id, &dataset.p_pixel_quality_89A, scan_start,
		scan_end, AM2_PIX_QUAL_A);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathUChar() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* Pixel Data Quality for 89B */
	ret = AMTK_get_SwathUChar(file_id, &dataset.p_pixel_quality_89B, scan_start,
		scan_end, AM2_PIX_QUAL_B);
	if (0 > ret)
	{
		E_MSG("AMTK_get_SwathUChar() error.[%d]\n", ret);
		terminate(file_id, &dataset);
		exit(EXIT_FAILURE);
	}
	
	/* get dataset value */
	get_value(scan_size, &dataset);
	
	/* termination */
	terminate(file_id, &dataset);
	
	printf("L2(High) product read end\n");
	
	exit(EXIT_SUCCESS);
}


/** Initialization.
 * @param file_id   [IN]  Product file id
 * @param scan_size [IN]  Number of scans
 * @param p_dataset [OUT] Dataset
 * @return RET_NORMAL
 * @return RET_ERROR
 */
static int initialize(hid_t file_id, int scan_size, DATASET_MODEL *p_dataset)
{
	int ret = 0;
	int dimsize[3] = {0};
	int i = 0;
	
	/* Access label: Geophysical Data */
	const int geo_89A_label[] = {AM2_SWATHA_GEO1, AM2_SWATHA_GEO2, AM2_SWATHA_GEO3};
	const int geo_89B_label[] = {AM2_SWATHB_GEO1, AM2_SWATHB_GEO2, AM2_SWATHB_GEO3};
	
	/* memory allocation */
	
	/* Scan Time
	 * (type size * Number of scans) */
	/* get dimension size */
	ret = AMTK_getDimSize(file_id, AM2_SCAN_TIME_DEF, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_scan_time = (AM2_COMMON_SCANTIME *) malloc(
		sizeof(AM2_COMMON_SCANTIME) * get_memory_size(dimsize));
	if (NULL == p_dataset->p_scan_time)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Position in Orbit
	 * (type size * Number of scans) */
	ret = AMTK_getDimSize(file_id, AM2_POS_ORBIT, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_pos_orbit = (double *) malloc(sizeof(double)
		* get_memory_size(dimsize));
	if (NULL == p_dataset->p_pos_orbit)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Geophysical Data for 89A (1...3) */
	
	/* Layer: ALL -> AM2_SWATHA_GEOA
	 * (type size * Number of scans * 486 * Number of Geophysical Data[1...3]) */
	ret = AMTK_getDimSize(file_id, AM2_SWATHA_GEOA, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	/*
	 * get Number of Geophysical Data
	 * # dimsize[0] : Number of Scans
	 * # dimsize[1] : Number of Pixel
	 * # dimsize[2] : Number of Geophysical Data
	 */
	p_dataset->geo_89A_num = dimsize[2];
	if ((0 > p_dataset->geo_89A_num) || (MAX_GEO_NUM < p_dataset->geo_89A_num))
	{
		/* out of bounds(MAX:3) */
		E_MSG("Illegal error: Number of Geophysical Data for 89A:%d\n",
			p_dataset->geo_89A_num);
		return RET_ERROR;
	}
	for (i = 0; i < p_dataset->geo_89A_num; i++)
	{
		/* Layer: 1...3 -> AM2_SWATHA_GEO1 ... AM2_SWATHA_GEO3
		 * (type size * Number of scans * 486) */
		ret = AMTK_getDimSize(file_id, geo_89A_label[i], dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
			return RET_ERROR;
		}
		p_dataset->p_geo_data_89A[i] = (float *) malloc(sizeof(float)
			* get_memory_size(dimsize));
		if (NULL == p_dataset->p_geo_data_89A[i])
		{
			E_MSG("malloc() error.\n");
			return RET_ERROR;
		}
	}
	
	/* Geophysical Data for 89B (1...3) */
	
	/* Layer: ALL -> AM2_SWATHB_GEOA
	 * (type size * Number of scans * 486 * Number of Geophysical Data[1...3]) */
	ret = AMTK_getDimSize(file_id, AM2_SWATHB_GEOA, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	/*
	 * get Number of Geophysical Data
	 * # dimsize[0] : Number of Scans
	 * # dimsize[1] : Number of Pixel
	 * # dimsize[2] : Number of Geophysical Data
	 */
	p_dataset->geo_89B_num = dimsize[2];
	if ((0 > p_dataset->geo_89B_num) || (MAX_GEO_NUM < p_dataset->geo_89B_num))
	{
		/* out of bounds(MAX:3) */
		E_MSG("Illegal error: Number of Geophysical Data for 89B:%d\n",
			p_dataset->geo_89B_num);
		return RET_ERROR;
	}
	for (i = 0; i < p_dataset->geo_89B_num; i++)
	{
		/* Layer: 1...3 -> AM2_SWATHB_GEO1 ... AM2_SWATHB_GEO3
		 * (type size * Number of scans * 486) */
		ret = AMTK_getDimSize(file_id, geo_89B_label[i], dimsize);
		if (0 > ret)
		{
			E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
			return RET_ERROR;
		}
		p_dataset->p_geo_data_89B[i] = (float *) malloc(sizeof(float)
			* get_memory_size(dimsize));
		if (NULL == p_dataset->p_geo_data_89B[i])
		{
			E_MSG("malloc() error.\n");
			return RET_ERROR;
		}
	}
	
	/* Latitude of Observation Point for 89A
	 * Longitude of Observation Point for 89A
	 * (type size * Number of scans * 486) */
	ret = AMTK_getDimSize(file_id, AM2_LATLON_L2_89A, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_latlon_89A = (AM2_COMMON_LATLON *) malloc(
		sizeof(AM2_COMMON_LATLON) * get_memory_size(dimsize));
	if (NULL == p_dataset->p_latlon_89A)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	
	/* Latitude of Observation Point for 89B
	 * Longitude of Observation Point for 89B
	 * (type size * Number of scans * 486) */
	ret = AMTK_getDimSize(file_id, AM2_LATLON_L2_89B, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	p_dataset->p_latlon_89B = (AM2_COMMON_LATLON *) malloc(
		sizeof(AM2_COMMON_LATLON) * get_memory_size(dimsize));
	if (NULL == p_dataset->p_latlon_89B)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Pixel Data Quality for 89A
	 * (type size * Number of Geophysical Data[1...3] * Numbef of scans * 486) */
	ret = AMTK_getDimSize(file_id, AM2_PIX_QUAL_A, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	/*
	 * get Number of Pixel Data Quality for 89A
	 * # dimsize[0] : Number of Geophysical Data
	 * # dimsize[1] : Number of Scans
	 * # dimsize[2] : Number of Pixel
	 */
	p_dataset->pix_89A_num = dimsize[0];
	
	p_dataset->p_pixel_quality_89A = (unsigned char *) malloc(
		get_memory_size(dimsize));
	if (NULL == p_dataset->p_pixel_quality_89A)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	/* Pixel Data Quality for 89B
	 * (type size * Number of Geophysical Data[1...3] * Numbef of scans * 486) */
	ret = AMTK_getDimSize(file_id, AM2_PIX_QUAL_B, dimsize);
	if (0 > ret)
	{
		E_MSG("AMTK_getDimSize() error.[%d]\n", ret);
		return RET_ERROR;
	}
	/*
	 * get Number of Pixel Data Quality for 89B
	 * # dimsize[0] : Number of Geophysical Data
	 * # dimsize[1] : Number of Scans
	 * # dimsize[2] : Number of Pixel
	 */
	p_dataset->pix_89B_num = dimsize[0];
	
	p_dataset->p_pixel_quality_89B = (unsigned char *) malloc(
		get_memory_size(dimsize));
	if (NULL == p_dataset->p_pixel_quality_89B)
	{
		E_MSG("malloc() error.\n");
		return RET_ERROR;
	}
	
	return RET_NORMAL;
}


/** Termination.
 * @param file_id   [IN]  Product file id
 * @param p_dataset [OUT] Dataset
 */
static void terminate(hid_t file_id, DATASET_MODEL *p_dataset)
{
	int i = 0;
	int ret = 0;
	
	if (0 <= file_id)
	{
		/* HDF product file close */
		ret = AMTK_closeH5(file_id);
		if (0 > ret)
		{
			E_MSG("AMTK_closeH5() error.[%d]\n", ret);
		}
	}
	
	/* free memory */
	free(p_dataset->p_scan_time);
	p_dataset->p_scan_time = NULL;
	
	free(p_dataset->p_pos_orbit);
	p_dataset->p_pos_orbit = NULL;
	
	for (i = 0; i < p_dataset->geo_89A_num; i++)
	{
		free(p_dataset->p_geo_data_89A[i]);
		p_dataset->p_geo_data_89A[i] = NULL;
	}
	
	for (i = 0; i < p_dataset->geo_89B_num; i++)
	{
		free(p_dataset->p_geo_data_89B[i]);
		p_dataset->p_geo_data_89B[i] = NULL;
	}
	
	free(p_dataset->p_latlon_89A);
	p_dataset->p_latlon_89A = NULL;
	
	free(p_dataset->p_latlon_89B);
	p_dataset->p_latlon_89B = NULL;
	
	free(p_dataset->p_pixel_quality_89A);
	p_dataset->p_pixel_quality_89A = NULL;
	
	free(p_dataset->p_pixel_quality_89B);
	p_dataset->p_pixel_quality_89B = NULL;
	
	return;
}


/** Get dataset value.
 * @param scan_size [IN] Number of scans
 * @param p_dataset [IN] Dataset
 */
static void get_value(int scan_size, const DATASET_MODEL *p_dataset)
{
	int i = 0;
	int scan_num = 0;
	int pixel_num = 0;
	
	for (scan_num = 0; scan_num < PRINT_SCAN_NUM; scan_num++)
	{
		/* Scan Time */
		printf("Scan Time No=%d [%d]   [%02d/%02d]  %02d:%02d:%02d.%03d\n",
			scan_num,
			(p_dataset->p_scan_time + scan_num)->year,
			(p_dataset->p_scan_time + scan_num)->month,
			(p_dataset->p_scan_time + scan_num)->day,
			(p_dataset->p_scan_time + scan_num)->hour,
			(p_dataset->p_scan_time + scan_num)->minute,
			(p_dataset->p_scan_time + scan_num)->second,
			(p_dataset->p_scan_time + scan_num)->ms);
		
		/* Geophysical Data for 89A */
		for (i = 0; i < p_dataset->geo_89A_num; i++)
		{
			/* first pixel */
			pixel_num = 0;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Geophysical Data (Layer:%d) = %8.4f\n", i + 1,
				*(p_dataset->p_geo_data_89A[i]
					+ (AM2_DEF_SNUM_HI * scan_num)
					+ pixel_num));
			
			/* last pixel */
			pixel_num = AM2_DEF_SNUM_HI - 1;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Geophysical Data (Layer:%d) = %8.4f\n", i + 1,
				*(p_dataset->p_geo_data_89A[i]
					+ (AM2_DEF_SNUM_HI * scan_num)
					+ pixel_num));
		}
		
		/* Geophysical Data for 89B */
		for (i = 0; i < p_dataset->geo_89B_num; i++)
		{
			/* first pixel */
			pixel_num = 0;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Geophysical Data (Layer:%d) = %8.4f\n", i + 1,
				*(p_dataset->p_geo_data_89B[i]
					+ (AM2_DEF_SNUM_HI * scan_num)
					+ pixel_num));
			
			/* last pixel */
			pixel_num = AM2_DEF_SNUM_HI - 1;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Geophysical Data (Layer:%d) = %8.4f\n", i + 1,
				*(p_dataset->p_geo_data_89B[i]
					+ (AM2_DEF_SNUM_HI * scan_num)
					+ pixel_num));
		}
		
		/* Latitude of Observation Point for 89A
		 * Longitude of Observation Point for 89A */
		printf("Latitude/Longitude of Observation Point for 89A\n" );
		/* first pixel */
		pixel_num = 0;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Latitude  = %8.2f\n",
			(p_dataset->p_latlon_89A + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lat);
		printf("Longitude = %8.2f\n",
			(p_dataset->p_latlon_89A + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lon);
		
		/* last pixel */
		pixel_num = AM2_DEF_SNUM_HI - 1;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Latitude  = %8.2f\n",
			(p_dataset->p_latlon_89A + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lat);
		printf("Longitude = %8.2f\n",
			(p_dataset->p_latlon_89A + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lon);
		
		/* Latitude of Observation Point for 89B
		 * Longitude of Observation Point for 89B */
		printf("Latitude/Longitude of Observation Point for 89B\n" );
		/* first pixel */
		pixel_num = 0;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Latitude  = %8.2f\n",
			(p_dataset->p_latlon_89B + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lat);
		printf("Longitude = %8.2f\n",
			(p_dataset->p_latlon_89B + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lon);
		
		/* last pixel */
		pixel_num = AM2_DEF_SNUM_HI - 1;
		printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
		printf("Latitude  = %8.2f\n",
			(p_dataset->p_latlon_89B + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lat);
		printf("Longitude = %8.2f\n",
			(p_dataset->p_latlon_89B + (AM2_DEF_SNUM_HI * scan_num) +
				pixel_num)->lon);
		
		/* Pixel Data Quality for 89A */
		for (i = 0; i < p_dataset->pix_89A_num; i++)
		{
			/* first pixel */
			pixel_num = 0;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Pixel Data Quality for 89A (Layer:%d) = 0x%02x\n", i + 1,
				*(p_dataset->p_pixel_quality_89A
					+ (i * scan_size * AM2_DEF_SNUM_HI)
					+ (scan_num * AM2_DEF_SNUM_HI)
					+ pixel_num));
			
			/* last pixel */
			pixel_num = AM2_DEF_SNUM_HI - 1;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Pixel Data Quality for 89A (Layer:%d) = 0x%02x\n", i + 1,
				*(p_dataset->p_pixel_quality_89A
					+ (i * scan_size * AM2_DEF_SNUM_HI)
					+ (scan_num * AM2_DEF_SNUM_HI)
					+ pixel_num));
		}
		
		/* Pixel Data Quality for 89B */
		for (i = 0; i < p_dataset->pix_89B_num; i++)
		{
			/* first pixel */
			pixel_num = 0;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Pixel Data Quality for 89B (Layer:%d) = 0x%02x\n", i + 1,
				*(p_dataset->p_pixel_quality_89B
					+ (i * scan_size * AM2_DEF_SNUM_HI)
					+ (scan_num * AM2_DEF_SNUM_HI)
					+ pixel_num));
			
			/* last pixel */
			pixel_num = AM2_DEF_SNUM_HI - 1;
			printf("Scan = %d Pixel = %d\n", scan_num, pixel_num);
			printf("Pixel Data Quality for 89B (Layer:%d) = 0x%02x\n", i + 1,
				*(p_dataset->p_pixel_quality_89B
					+ (i * scan_size * AM2_DEF_SNUM_HI)
					+ (scan_num * AM2_DEF_SNUM_HI)
					+ pixel_num));
		}
	}
	
	/* The following data will not print, so each function use same memory area.*/
	/* Just show as access example */
	
	return;
}

