/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2010 All rights reserved.                  */
/*  File Name    : IOTK_common.c                                             */
/*  Abstract     : I/O toolkit get function                                  */
/*  Function List                                                            */
/*      IOTK_printf()                : IOToolKit DEBUG printf                */
/*      string_check_fortran()       : String check for Fortran              */
/*      IOTK_DebugCheck()            : IOToolKit DEBUG flag check            */
/*      DateToSec()                  : IOToolKit date to sec                 */
/*      SecToDate()                  : IOToolKit sec to Date                 */
/*      checkScanTimeData()          : IOToolKit check scantime data         */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2010/06/30                                                    */
/*  Version    1.000                                                         */
/*             1.100 2015/01/05 Add handling Scantime data for AMSR-E        */
/*  Notice                                                                   */
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <math.h>
#include "./AMTK_PLUS.h"
#include "./IOTK_common.h"

/* DEBUG FLAG */
char    IOTK_dbgflg = IOTK_OFF;

/** @file IOTK_common.c
 * IOToolKit DEBUG printf
 *
 * @author     NEC
 * @date       2010/06/30
 */
int IOTK_printf(const char *form, ...)
{
	int     ret=0;
	va_list args;

	if(IOTK_dbgflg == IOTK_ON)
	{
		va_start(args,form);
		ret = vfprintf(stdout, form,args);
		va_end(args);
	}
	return ret;
}

/** @file IOTK_common.c
 * String check for Fortran
 *
 * @author     NEC
 * @date       2010/06/30
 */
char *string_check_fortran(char* original, int length)
{
    int i = 0;
    char* p_string = NULL;

    p_string = (char*) malloc( length+1 );
    memset( p_string, 0x00, length+1);
    strncpy( p_string, original, length);
    for( i=length ; i>0 ; i--)
    {
        if( *(p_string+i) == '\0' )
        {
            continue;
        }
        if( *(p_string+i) != ' ' )
        {
            break;
        }
        *(p_string+i)='\0';
    }
    return p_string;
}

/** @file IOTK_common.c
 * IOToolKit DEBUG flag check
 *
 * @author     NEC
 * @date       2010/06/30
 */
void IOTK_DebugCheck(void)
{
	char	*p_str = NULL;

    p_str = getenv("IOTOOLKIT_DEBUG");
    if(p_str != NULL)
    {    
        IOTK_dbgflg = IOTK_ON;
    }    
    else 
    {    
        IOTK_dbgflg = IOTK_OFF;
    }    
}

/** @file IOTK_common.c
 * IOToolKit date to sec
 *
 * @author     NEC
 * @date       2010/06/30
 */
int DateToSec(AM2_COMMON_SCANTIME date, int *total_sec)
{
	int		temp_y = 0;
	int		temp_m = 0;

	int		d_year = 0;
	int		d_ly = 0;
	int		d_month = 0;
	int		total_day = 0;
	int		s_date = 0;
	int		s_time = 0;
	int		ret = 0;


	/*******************/
	/* Check arguments */
	/*******************/
	ret = checkScanTimeData(&date);
	if(ret != SUCCESS)
	{
		return ret;
	}

	/****************************************/
	/* Calculate the number of elapsed days */
	/****************************************/

	/* If the argument is Jan or Feb,              */
	/* process to the 13 or 14 month of last year. */
	if (date.month <= 2)
	{
		temp_y = date.year - 1;
		temp_m = date.month + 12;
	}
	else
	{
		temp_y = date.year;
		temp_m = date.month;
	}

	d_year = 365 * (temp_y - 1);
	d_ly = temp_y/4 - temp_y/100 + temp_y/400;
	d_month = (temp_m * 979 - 1033)/32;
	total_day = d_year + d_ly + d_month + date.day - 1;

	/* move to 1993.01.01 from 1970.01.01 */
	total_day = total_day - 727563;

	/****************************************/
	/* Calculate the number of elapsed Sec */
	/****************************************/
	s_date = total_day * 86400;
	s_time = ((date.hour * 60) + date.minute) * 60;

	*total_sec = s_date + s_time + date.second;

	return SUCCESS;
}

/** @file IOTK_common.c
 * IOToolKit sec to Date
 *
 * @author     NEC
 * @date       2010/06/30
 */
int SecToDate(int total_sec,AM2_COMMON_SCANTIME *date)
{
	int		t_second=0;
	int		t_day   =0;
	int		cnt     =0;
	int		ret     =0;
	const int month_end[] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
	const int ly_month_end[] = { 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

	/*******************/
	/* Check arguments */
	/*******************/

	if(total_sec < 0)
	{
		IOTK_printf("SecToDate Error : Argument[MinusValue(total_sec)]\n");
		return ERROR_ILLEGAL_TOTALSEC;
	}
	
	/****************************************/
	/* Calculate the number of elapsed days */
	/****************************************/

	t_second = total_sec;

	date->second = t_second % 60;
	t_second = t_second / 60;

	date->minute = t_second % 60;
	t_second = t_second / 60;
	
	date->hour = t_second % 24;
	t_second = t_second / 24;
		
	t_day = t_second + 727563 + 1;

	cnt = 1;
	while(t_day > 365)
	{
		if((0 == cnt % 400) || ((0 != (cnt % 100)) && (0 == (cnt % 4))))
		{
			t_day = t_day - 366;
		}
		else
		{
			t_day = t_day - 365;
		}
		cnt++;
	}

	date->year = cnt;

	if((0 == date->year % 400) || ((0 != (date->year % 100)) && (0 == (date->year % 4))))
	{
		for(cnt=0; t_day > ly_month_end[cnt]; ++cnt)
		{
			t_day = t_day - ly_month_end[cnt];
		}
	}
	else
	{
		for(cnt=0; t_day > month_end[cnt]; ++cnt)
		{
			t_day = t_day - month_end[cnt];
		}
	}

	if( (cnt+1 == 1) && (t_day == 0))
	{
		date->year = date->year - 1;
		date->month = 12;
		date->day = 31;
	}

	else if(cnt == 12)
	{
		date->year = date->year + 1;
		date->month = 1;
		date->day = t_day - month_end[0];
	}

	else 
	{
		date->month = cnt+1;
		date->day = t_day;
	}

	/*********************/
	/* Check Result Data */
	/*********************/
	ret = checkScanTimeData(date);
	if(ret != SUCCESS)
	{
		return ret;
	}

	return SUCCESS;

}

/** @file IOTK_common.c
 * IOToolKit check scantime data
 *
 * @author     NEC
 * @date       2010/06/30
 */
int checkScanTimeData(AM2_COMMON_SCANTIME *date)
{
	if(0 == date->year && 0 == date->month && 0 == date->day && 0 == date->hour && 0 == date->minute && 0 == date->second && 0 == date->ms)
	{
		/* No action. */
		/* Handling to read "Error scan_time data" of AMSR-E. */
		
	}
	else
	{

		const int month_end[] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

		if((date->month < 1) || (date->day < 1) || (date->hour < 0) || (date->minute < 0) || (date->second < 0))
		{
			IOTK_printf("SecToDate Error : Result[MinusValue]\n");
			return ERROR_ILLEGAL_DATE;
		}


		if(date->year < 1993)
		{
			IOTK_printf("SecToDate Error : Result[date->year]\n");
			return ERROR_ILLEGAL_YEAR;
		}


		if(date->month > 12)
		{
			IOTK_printf("SecToDate Error : Result[date->month]\n");
			return ERROR_ILLEGAL_MONTH;
		}


		if((0 == date->year % 400) || ((0 != (date->year % 100)) && (0 == (date->year % 4))))
		{

			if((date->month == 2) && (date->day > 29))
			{
				IOTK_printf("SecToDate Error : Result[date->day(LeapYear : Feb)]\n");
				return ERROR_ILLEGAL_DAY;
			}
			else if((date->month != 2) && (date->day > month_end[date->month-1]))
			{
				IOTK_printf("SecToDate Error : Result[date->day(LeapYear : other)]\n");
				return ERROR_ILLEGAL_DAY;
			}
		}

		else if( date->day > month_end[date->month-1])
		{
			IOTK_printf("SecToDate Error : Result[date->day]\n");
			return ERROR_ILLEGAL_DAY;
		}


		if( date->hour > 23)
		{
			IOTK_printf("SecToDate Error : Result[date->hour]\n");
			return ERROR_ILLEGAL_HOUR;
		}
	

		if(date->minute > 59)
		{
			IOTK_printf("SecToDate Error : Result[date->minute]\n");
			return ERROR_ILLEGAL_MIN;
		}
	
		if((date->hour==23) && (date->minute==59))
		{

			if( date->second > 60)
			{
				IOTK_printf("SecToDate Error : Result[date->minute(LeapSecond)]\n");
				return ERROR_ILLEGAL_SEC;
			}
		}

		else if( date->second > 59)
		{
			IOTK_printf("SecToDate Error : Result[date->minute]\n");
			return ERROR_ILLEGAL_SEC;
		}
	}

	return SUCCESS;
}


