/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2010 All rights reserved.                  */
/*  File Name    : amtk_get.c                                                */
/*  Abstract     : I/O toolkit get function                                  */
/*  Function List                                                            */
/*      AMTK_openH5()                : HDF file open funtion for read        */
/*      AMTK_closeH5()               : HDF file close funtion for read       */
/*      AMTK_getDimSize()            : Get HDF dataset dimension size        */
/*      AMTK_getMetaDataName()       : Get HDF Meta Data by Name             */
/*      AMTK_getMetaData()           : Get HDF Meta Data                     */
/*      AMTK_get_SwathInt()          : Get HDF dataset integer value         */
/*      AMTK_get_SwathFloat()        : Get HDF dataset float value           */
/*      AMTK_get_SwathDouble()       : Get HDF dataset double value          */
/*      AMTK_get_SwathUChar()        : Get HDF dataset unsigned char value   */
/*      AMTK_get_GridInt()           : Get HDF dataset integer value in grid */
/*      AMTK_get_GridFloat()         : Get HDF dataset float value in grid   */
/*      AMTK_get_Grid03D()           : Get HDF 3 days average                */
/*      AMTK_get_ScanDataQuality()   : Get HDF Scan Data Quality Structure   */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2011/06/13                                                    */
/*  Version    1.10                                                          */
/*  Notice                                                                   */
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "./IOTK_common.h"
#include "./AMTK.h"
#include "./AMTK_PLUS.h"

extern char **name_table[];
extern char *latlon_name_lo[];
extern char *latlon_name_89a[];
extern char *latlon_name_89b[];

/** @file amtk_get.c
 * HDF file open funtion for read
 *
 * @author     NEC
 * @date       2010/06/30
 */
 hid_t AMTK_openH5(char *file_name)
{
	hid_t   file_id = 0;

	/******* Mute HDF error  ******/
	hid_t error_stack = H5E_DEFAULT;
	H5Eset_auto2(error_stack, NULL, NULL);
	/******* Mute HDF error  end******/

	if(file_name == NULL)
	{
		return ERROR_HDF_FILE_OPEN;
	}

	if(strlen(file_name) <= 0)
	{
		return ERROR_HDF_FILE_OPEN;
	}

	/* DEBUG FLAG */
	IOTK_DebugCheck();
	/* DEBUG FLAG */

	file_id = H5Fopen(file_name, H5F_ACC_RDONLY, H5P_DEFAULT);
	if(file_id == FAIL)
	{
		IOTK_printf("File open error@H5Fopen\n");
		return ERROR_HDF_FILE_OPEN;
	}
	return file_id;
}
hid_t amtk_openh5_(char *file_name, int length)
{
    hid_t hDF_file_id = 0;
    char *p_string = NULL;

    p_string = string_check_fortran(file_name, length);
    hDF_file_id =  AMTK_openH5(p_string);
    free(p_string);
    return hDF_file_id;
}
/* For g77 entry */
hid_t amtk_openh5__(char *file_name, int length)
{
    return amtk_openh5_(file_name, length);
}

/** @file amtk_get.c
 * HDF file close funtion for read
 *
 * @author     NEC
 * @date       2010/06/30
 */
int AMTK_closeH5(hid_t HDF_file_id)
{
	if( H5Fclose(HDF_file_id)==FAIL )
	{
       return ERROR_HDF_FILE_CLOSE;
	}
    return SUCCESS;
}
int amtk_closeh5_(hid_t *HDF_file_id)
{
    return AMTK_closeH5(*HDF_file_id);
}
/* For g77 entry */
int amtk_closeh5__(hid_t *HDF_file_id)
{
    return amtk_closeh5_(HDF_file_id);
}

/** @file amtk_get.c
 * Get HDF dataset dimension size
 *
 * @author     NEC
 * @date       2011/06/13
 */
 int AMTK_getDimSize(hid_t HDF_file_id,int dataset_no , int *size)
{
	int				status = 0;
	int				rank = 0;
	int				ret = 0;
	int				table_num = 0;
	hid_t			dataset = 0;
	hid_t			space = 0;
	hsize_t			size1[3] = {0,0,0};
	hsize_t			size2[3] = {0,0,0};
	ACCESS_LABEL	*p_table=NULL;
	char			setName[513];
	GET_HDFDATA 	hdfdata;

	status = CommonPrmCheck(HDF_file_id, AM2_SCANNUM_NOCHK, AM2_SCANNUM_NOCHK, dataset_no);
	if(status != SUCCESS)
	{
		return status;
	}

	p_table = search_table(HDF_file_id, dataset_no, &table_num);
	if(p_table == NULL)
	{
		return ERROR_NOT_ACCESSLABEL;
	}

	memset(&setName[0], 0x00, sizeof(setName));

	switch(dataset_no)
	{
		case AM2_LATLON_L2_LO:
			strcpy(&setName[0], latlon_name_lo[0]);
			break;
		case AM2_LATLON_89A:
		case AM2_LATLON_RS_89A:
		case AM2_LATLON_L2_89A:
		case AM2_LATLON_06:
		case AM2_LATLON_07:
		case AM2_LATLON_10:
		case AM2_LATLON_18:
		case AM2_LATLON_23:
		case AM2_LATLON_36:
		case AM2_LATLON_LO:
		case AM2_LATLON_RS_LO:
			strcpy(&setName[0], latlon_name_89a[0]);
			break;
		case AM2_LATLON_89B:
		case AM2_LATLON_RS_89B:
		case AM2_LATLON_L2_89B:
			strcpy(&setName[0], latlon_name_89b[0]);
			break;
		case AM2_GRID_LATLON:
			ret = getL3LatLonFileName(HDF_file_id, &setName[0]);
			if(ret < 0)
			{
				return ret;
			}
			rank = getL3LatLonSize(&setName[0], size);
			return rank;
		case AM2_GRID_GEO1:
		case AM2_GRID_GEO2:
		case AM2_GRID_GEO3:
			p_table = search_table(HDF_file_id, AM2_GRID_GEOA, &table_num);
			if(p_table == NULL)
			{
				return ERROR_NOT_ACCESSLABEL;
			}
			strcpy(&setName[0], name_table[table_num][p_table->dataset_no]);
			break;
		case AM2_GRID_GEO1_STD:
		case AM2_GRID_GEO2_STD:
		case AM2_GRID_GEO3_STD:
			p_table = search_table(HDF_file_id, AM2_GRID_GEOA_STD, &table_num);
			if(p_table == NULL)
			{
				return ERROR_NOT_ACCESSLABEL;
			}
			strcpy(&setName[0], name_table[table_num][p_table->dataset_no]);
			break;
		case AM2_GRID_GEO1_ANUM:
		case AM2_GRID_GEO2_ANUM:
		case AM2_GRID_GEO3_ANUM:
			p_table = search_table(HDF_file_id, AM2_GRID_GEOA_ANUM, &table_num);
			if(p_table == NULL)
			{
				return ERROR_NOT_ACCESSLABEL;
			}
			strcpy(&setName[0], name_table[table_num][p_table->dataset_no]);
			break;
		case AM2_GRID_GEO1_TNUM:
		case AM2_GRID_GEO2_TNUM:
		case AM2_GRID_GEO3_TNUM:
			p_table = search_table(HDF_file_id, AM2_GRID_GEOA_TNUM, &table_num);
			if(p_table == NULL)
			{
				return ERROR_NOT_ACCESSLABEL;
			}
			strcpy(&setName[0], name_table[table_num][p_table->dataset_no]);
			break;
		case AM2_SWATH_GEO1:
		case AM2_SWATH_GEO2:
		case AM2_SWATH_GEO3:
			p_table = search_table(HDF_file_id, AM2_SWATH_GEOA, &table_num);
			if(p_table == NULL)
			{
				return ERROR_NOT_ACCESSLABEL;
			}
			strcpy(&setName[0], name_table[table_num][p_table->dataset_no]);
			break;
		case AM2_SWATHA_GEO1:
		case AM2_SWATHA_GEO2:
		case AM2_SWATHA_GEO3:
			p_table = search_table(HDF_file_id, AM2_SWATHA_GEOA, &table_num);
			if(p_table == NULL)
			{
				return ERROR_NOT_ACCESSLABEL;
			}
			strcpy(&setName[0], name_table[table_num][p_table->dataset_no]);
			break;
		case AM2_SWATHB_GEO1:
		case AM2_SWATHB_GEO2:
		case AM2_SWATHB_GEO3:
			p_table = search_table(HDF_file_id, AM2_SWATHB_GEOA, &table_num);
			if(p_table == NULL)
			{
				return ERROR_NOT_ACCESSLABEL;
			}
			strcpy(&setName[0], name_table[table_num][p_table->dataset_no]);
			break;
		default:
			/* WriteNG */
			ret = checkWriteNG(dataset_no);
			if(ret == ERROR_ACCESSLABEL_WRITE_NG)
			{
				checkAccessLabel(dataset_no, &hdfdata);
				p_table = search_table(HDF_file_id, hdfdata.data[0].dataset_no, &table_num);
				if(p_table == NULL)
				{
					return ERROR_NOT_ACCESSLABEL;
				}
				strcpy(&setName[0], name_table[table_num][p_table->dataset_no]);
			}
			else
			{
				strcpy(&setName[0], name_table[table_num][p_table->dataset_no]);
			}
			break;
	}

	dataset  = H5Dopen (HDF_file_id, &setName[0], H5P_DEFAULT);
	if(dataset < 0)
	{
		IOTK_printf("getDimSize ERROR - 1[%s]\n",setName);
		return ERROR_HDF_DATASET_OPEN;
	}

	space  = H5Dget_space(dataset);
	
	/* close dataset */
	H5Dclose(dataset);
	
	if(space < 0)
	{
		IOTK_printf("getDimSize ERROR - 2\n");
		return ERROR_HDF_DATASET_GET_SPACE;
	}

	rank   = H5Sget_simple_extent_ndims(space);
	if(rank < 0)
	{
		IOTK_printf("getDimSize ERROR - 3\n");
		/* close dataspace */
		H5Sclose(space);
		return ERROR_HDF_DATASET_GET_RANK;
	}
	status = H5Sget_simple_extent_dims(space, (hsize_t *)&size1, (hsize_t *)&size2);
	
	/* close dataspace */
	H5Sclose(space);

	if(status < 0)
	{
		IOTK_printf("getDimSize ERROR - 4\n");
		return ERROR_HDF_DATASET_GET_DIMS;
	}
	size[0] = (int)size1[0];
	size[1] = (int)size1[1];
	size[2] = (int)size1[2];
	
	switch(dataset_no)
	{
		case AM2_GRID_GEO1:
		case AM2_GRID_GEO2:
		case AM2_GRID_GEO3:
			size[2] = 0;
			rank = 2;
			break;
		case AM2_GRID_GEO1_STD:
		case AM2_GRID_GEO2_STD:
		case AM2_GRID_GEO3_STD:
			size[2] = 0;
			rank = 2;
			break;
		case AM2_GRID_GEO1_ANUM:
		case AM2_GRID_GEO2_ANUM:
		case AM2_GRID_GEO3_ANUM:
			size[2] = 0;
			rank = 2;
			break;
		case AM2_GRID_GEO1_TNUM:
		case AM2_GRID_GEO2_TNUM:
		case AM2_GRID_GEO3_TNUM:
			size[2] = 0;
			rank = 2;
			break;
		case AM2_SWATH_GEO1:
		case AM2_SWATH_GEO2:
		case AM2_SWATH_GEO3:
			size[2] = 0;
			rank = 2;
			break;
		case AM2_SWATHA_GEO1:
		case AM2_SWATHA_GEO2:
		case AM2_SWATHA_GEO3:
			size[2] = 0;
			rank = 2;
			break;
		case AM2_SWATHB_GEO1:
		case AM2_SWATHB_GEO2:
		case AM2_SWATHB_GEO3:
			size[2] = 0;
			rank = 2;
			break;
		case AM2_LATLON_LO:
		case AM2_LATLON_06:
		case AM2_LATLON_07:
		case AM2_LATLON_10:
		case AM2_LATLON_18:
		case AM2_LATLON_23:
		case AM2_LATLON_36:
		case AM2_LATLON_RS_LO:
			size[1] = size[1]/2;
			break;
		default:
			/* WriteNG */
			ret = checkWriteNG(dataset_no);
			if(ret == ERROR_ACCESSLABEL_WRITE_NG)
			{
				if(rank == 1)
				{
					size[1] = hdfdata.datacnt;
					rank = 2;
				}
				else if(rank == 2)
				{
					size[2] = hdfdata.datacnt;
					rank = 3;
				}
			}
			break;
	}

	return rank;
}
int amtk_getdimsize_(hid_t *HDF_file_id,int *dataset_no , int *size)
{
    return AMTK_getDimSize(*HDF_file_id, *dataset_no ,  size);
}
/* For g77 entry */
int amtk_getdimsize__(hid_t *HDF_file_id,int *dataset_no , int *size)
{
    return amtk_getdimsize_(HDF_file_id, dataset_no , size);
}

/** @file amtk_get.c
 * Get HDF Meta Data by Name
 *
 * @author     NEC
 * @date       2010/06/30
 */
int AMTK_getMetaDataName(hid_t HDF_file_id, char *name, char **value)
{
	hid_t	attr = 0;
	herr_t	ret = 0;
	
	if(HDF_file_id == 0)
	{
		return ERROR_ILLEGAL_HDF_FILEID;
	}
	
	if(name == NULL || value == NULL)
	{
		return ERROR_NULL_ADDRESS;
	}
	
    /*  Open the attribute using its name.  */
	attr = H5Aopen_name(HDF_file_id, name);
	if(attr < 0)
	{
		return ERROR_HDF_ATTR_NAME_OPEN;
	}
	
	/* Get Attribute. */
	ret = getAttribute(attr, value);
	if (SUCCESS != ret)
	{
		/* Close Attribute. */
		H5Aclose(attr);
		return ret;
	}
	
	/* Close Attribute. */
	H5Aclose(attr);
	return strlen(*value);
}
int amtk_getmetadataname_(hid_t *HDF_file_id, char *name_org, char *value_org, int name_length, int value_length)
{
    char* p_name = NULL;
    char* p_value = NULL;
    int length = 0;

    p_name = string_check_fortran(name_org, name_length);
    p_value = (char *) malloc(value_length + 1);
    if (p_value == NULL)
    {
        return ERROR_MALLOC;
    }
    else
    {
        /* init buffer and outdata */
        memset(p_value, '\0', value_length + 1);
        memset(value_org, '\0', value_length);
    }
    length = AMTK_getMetaDataName(*HDF_file_id, p_name, &p_value);
    free(p_name);
    if (length > 0)
    {
        strncpy(value_org, p_value, length);
    }
    free(p_value);
    return length;
}
/* For g77 entry */
int amtk_getmetadataname__(hid_t *HDF_file_id, char *name_org, char *value_org, int name_length, int value_length)
{
    return amtk_getmetadataname_(HDF_file_id, name_org, value_org, name_length, value_length);
}

/** @file amtk_get.c
 * Get HDF Meta Data
 *
 * @author     NEC
 * @date       2010/06/30
 */
int AMTK_getMetaData(hid_t HDF_file_id, int index, char **value)
{
	hid_t	attr = 0;
	herr_t	ret = 0;
	int		num = 0;
	
	if(HDF_file_id == 0)
	{
		return ERROR_ILLEGAL_HDF_FILEID;
	}
	
	if(index <= 0)
	{
		return ERROR_ILLEGAL_INDEX;
	}
	
	if(value == NULL)
	{
		return ERROR_NULL_ADDRESS;
	}
	
	num = index - 1;
	
	/* Open the attribute using its name.  */
	attr = H5Aopen_idx(HDF_file_id, num);
	if( attr < 0)
	{
		IOTK_printf("getMetaData Open ERROR \n");
		return ERROR_HDF_ATTR_IDX_OPEN;
	}
	
	/* Get Attribute. */
	ret = getAttribute(attr, value);
	if (SUCCESS != ret)
	{
		/* Close Attribute. */
		H5Aclose(attr);
		return ret;
	}
	
	/* Close Attribute. */
	H5Aclose(attr);
	return strlen(*value);
}
int amtk_getmetadata_(hid_t *HDF_file_id, int *index, char *value, int value_length)
{
    char *p_value = NULL;
    int length = 0;
    
    p_value = (char *) malloc(value_length + 1);
    if (p_value == NULL)
    {
        return ERROR_MALLOC;
    }
    else
    {
        /* init buffer and outdata */
        memset(p_value, '\0', value_length + 1);
        memset(value, '\0', value_length);
    }
    length = AMTK_getMetaData(*HDF_file_id, *index, &p_value);
    if (length > 0)
    {
        strncpy(value, p_value, length);
    }
    free(p_value);
    return length;
}
/* For g77 entry */
int amtk_getmetadata__(hid_t *HDF_file_id, int *index, char *value, int value_length)
{
    return amtk_getmetadata_(HDF_file_id, index, value, value_length);
}

/** @file amtk_get.c
 * Get HDF dataset integer value
 *
 * @author     NEC
 * @date       2010/06/30
 */
int AMTK_get_SwathInt(hid_t HDF_file_id, int **data, int from_scan, int to_scan, int dataset_no)
{
	int	ret = 0;
	int datatype = 0;

	ret = CommonPrmCheck(HDF_file_id, from_scan, to_scan, dataset_no);
	if(ret != SUCCESS)
	{
		return ret;
	}
	datatype = AM2_DTYPE_INT;
	ret = getDataSet(HDF_file_id, (void **)data, from_scan, to_scan, dataset_no, datatype);
	if(ret <= 0)
	{
		return ret;
	}
	return ret;
}
int amtk_get_swathint_(hid_t *HDF_file_id, int *data, int *from_scan, int *to_scan, int *dataset_no)
{
    return AMTK_get_SwathInt(*HDF_file_id, &data, *from_scan, *to_scan, *dataset_no);
}
/* For g77 entry */
int amtk_get_swathint__(hid_t *HDF_file_id, int *data, int *from_scan, int *to_scan, int *dataset_no)
{
    return amtk_get_swathint_(HDF_file_id, data, from_scan, to_scan, dataset_no);
}

/** @file amtk_get.c
 * Get HDF float value
 *
 * @author     NEC
 * @date       2010/06/30
 */
int AMTK_get_SwathFloat(hid_t HDF_file_id, float **data, int from_scan, int to_scan, int dataset_no)
{
	int	ret = 0;
	int datatype = 0;

	ret = CommonPrmCheck(HDF_file_id, from_scan, to_scan, dataset_no);
	if(ret != SUCCESS)
	{
		return ret;
	}
	datatype = AM2_DTYPE_FLOAT;
	ret = getDataSet(HDF_file_id, (void **)data, from_scan, to_scan, dataset_no, datatype);
	if(ret <= 0)
	{
		return ret;
	}
	return ret;
}
int amtk_get_swathfloat_(hid_t *HDF_file_id, float *data, int *from_scan, int *to_scan, int *dataset_no)
{
    return AMTK_get_SwathFloat(*HDF_file_id, &data, *from_scan, *to_scan, *dataset_no);
}
/* For g77 entry */
int amtk_get_swathfloat__(hid_t *HDF_file_id, float *data, int *from_scan, int *to_scan, int *dataset_no)
{
    return amtk_get_swathfloat_(HDF_file_id, data, from_scan, to_scan, dataset_no);
}

/** @file amtk_get.c
 * Get HDF double value
 *
 * @author     NEC
 * @date       2010/06/30
 */
int AMTK_get_SwathDouble(hid_t HDF_file_id, double **data, int from_scan, int to_scan, int dataset_no)
{
	int	ret = 0;
	int datatype = 0;

	ret = CommonPrmCheck(HDF_file_id, from_scan, to_scan, dataset_no);
	if(ret != SUCCESS)
	{
		return ret;
	}
	datatype = AM2_DTYPE_DOUBLE;
	ret = getDataSet(HDF_file_id, (void **)data, from_scan, to_scan, dataset_no, datatype);
	if(ret <= 0)
	{
		return ret;
	}
	return ret;
}
int amtk_get_swathdouble_(hid_t *HDF_file_id, double *data, int *from_scan, int *to_scan, int *dataset_no)
{
    return AMTK_get_SwathDouble(*HDF_file_id, &data, *from_scan, *to_scan, *dataset_no);
}
/* For g77 entry */
int amtk_get_swathdouble__(hid_t *HDF_file_id, double *data, int *from_scan, int *to_scan, int *dataset_no)
{
    return amtk_get_swathdouble_(HDF_file_id, data, from_scan, to_scan, dataset_no);
}

/** @file amtk_get.c
 * Get HDF unsigned charcter value
 *
 * @author     NEC
 * @date       2010/06/30
 */
int AMTK_get_SwathUChar(hid_t HDF_file_id, unsigned char **data, int from_scan, int to_scan, int dataset_no)
{
	int	ret = 0;
	int datatype = 0;

	ret = CommonPrmCheck(HDF_file_id, from_scan, to_scan, dataset_no);
	if(ret != SUCCESS)
	{
		return ret;
	}
	datatype = AM2_DTYPE_UCHAR;
	ret = getDataSet(HDF_file_id, (void **)data, from_scan, to_scan, dataset_no, datatype);
	if(ret <= 0)
	{
		return ret;
	}
	return ret;
}
int amtk_get_swathuchar_(hid_t *HDF_file_id, unsigned char *data, int *from_scan, int *to_scan, int *dataset_no)
{
    return AMTK_get_SwathUChar(*HDF_file_id, &data, *from_scan, *to_scan, *dataset_no);
}
/* For g77 entry */
int amtk_get_swathuchar__(hid_t *HDF_file_id, unsigned char *data, int *from_scan, int *to_scan, int *dataset_no)
{
    return amtk_get_swathuchar_(HDF_file_id, data, from_scan, to_scan, dataset_no);
}

/** @file amtk_get.c
 * Get HDF Integer value in grid
 *
 * @author     NEC
 * @date       2010/06/30
 */
int AMTK_get_GridInt(hid_t HDF_file_id, int **data, int dataset_no)
{
	int	ret = 0;
	int	gridFlg = IOTK_OFF;

	ret = CommonPrmCheck(HDF_file_id, AM2_SCANNUM_NOCHK, AM2_SCANNUM_NOCHK, dataset_no);
	if(ret != SUCCESS)
	{
		return ret;
	}
	switch(dataset_no)
	{
		case AM2_GRID_TBV_ANUM:
		case AM2_GRID_TBH_ANUM:
		case AM2_GRID_TB_ANUM:
		case AM2_GRID_TBV_TNUM:
		case AM2_GRID_TBH_TNUM:
		case AM2_GRID_TB_TNUM:
		case AM2_GRID_TIME:
			ret = SUCCESS;
			break;
		case AM2_GRID_GEO1_ANUM:
		case AM2_GRID_GEO2_ANUM:
		case AM2_GRID_GEO3_ANUM:
		case AM2_GRID_GEOA_ANUM:
		case AM2_GRID_GEO1_TNUM:
		case AM2_GRID_GEO2_TNUM:
		case AM2_GRID_GEO3_TNUM:
		case AM2_GRID_GEOA_TNUM:
			gridFlg = IOTK_ON;
			ret = SUCCESS;
			break;
		default:
			ret = ERROR_ACCESSLABEL_READ_NG;
			break;
	}
	if(ret == ERROR_ACCESSLABEL_READ_NG)
	{
		return ret;
	}
	if(gridFlg == IOTK_OFF)
	{
		ret = getDataSet(HDF_file_id, (void **)data, AM2_START_SCANNUM_ALL, AM2_END_SCANNUM_ALL, dataset_no, AM2_DTYPE_INT);
	}
	else
	{
		ret = getGridDataSet(HDF_file_id, (void **)data, AM2_START_SCANNUM_ALL, AM2_END_SCANNUM_ALL, dataset_no, AM2_DTYPE_INT);
	}
	if(ret <= 0)
	{
		return ret;
	}
	return ret;
}
int amtk_get_gridint_(hid_t *HDF_file_id, int *data, int *dataset_no)
{
    return AMTK_get_GridInt(*HDF_file_id, &data, *dataset_no);
}
/* For g77 entry */
int amtk_get_gridint__(hid_t *HDF_file_id, int *data, int *dataset_no)
{
    return amtk_get_gridint_(HDF_file_id, data, dataset_no);
}

/** @file amtk_get.c
 * Get HDF float value in grid
 *
 * @author     NEC
 * @date       2010/06/30
 */
int AMTK_get_GridFloat(hid_t HDF_file_id, float **data, int dataset_no)
{
	int	ret = 0;
	int	gridFlg = IOTK_OFF;

	ret = CommonPrmCheck(HDF_file_id, AM2_SCANNUM_NOCHK, AM2_SCANNUM_NOCHK, dataset_no);
	if(ret != SUCCESS)
	{
		return ret;
	}
	switch(dataset_no)
	{
		case AM2_GRID_TBV:
		case AM2_GRID_TBH:
		case AM2_GRID_TB:
		case AM2_GRID_TBV_STD:
		case AM2_GRID_TBH_STD:
		case AM2_GRID_TB_STD:
			ret = SUCCESS;
			break;
		case AM2_GRID_GEO1:
		case AM2_GRID_GEO2:
		case AM2_GRID_GEO3:
		case AM2_GRID_GEOA:
		case AM2_GRID_GEO1_STD:
		case AM2_GRID_GEO2_STD:
		case AM2_GRID_GEO3_STD:
		case AM2_GRID_GEOA_STD:
			gridFlg = IOTK_ON;
			ret = SUCCESS;
			break;
		default:
			ret = ERROR_ACCESSLABEL_READ_NG;
			break;
	}
	if(ret == ERROR_ACCESSLABEL_READ_NG)
	{
		return ret;
	}
	if(gridFlg == IOTK_OFF)
	{
		ret = getDataSet(HDF_file_id, (void **)data, AM2_START_SCANNUM_ALL, AM2_END_SCANNUM_ALL, dataset_no, AM2_DTYPE_FLOAT);
	}
	else
	{
		ret = getGridDataSet(HDF_file_id, (void **)data, AM2_START_SCANNUM_ALL, AM2_END_SCANNUM_ALL, dataset_no, AM2_DTYPE_FLOAT);
	}
	return ret;
}
int amtk_get_gridfloat_(hid_t *HDF_file_id, float *data, int *dataset_no)
{
    return AMTK_get_GridFloat(*HDF_file_id, &data, *dataset_no);
}
/* For g77 entry */
int amtk_get_gridfloat__(hid_t *HDF_file_id, float *data, int *dataset_no)
{
    return amtk_get_gridfloat_(HDF_file_id, data, dataset_no);
}

/** @file amtk_get.c
 * Get HDF 3 days average
 *
 * @author     NEC
 * @date       2010/06/30
 */
int AMTK_get_Grid03D(float **average, char *name_table)
{
	hid_t	hDF_file_id = 0;
	int		rank = 0;
	int		size[3]={0,0,0};
	int		chksize[3]={0,0,0};
	float	*p_data=NULL;
	void	*p_dataBuf = NULL;
	int		getmemsize = 0;
	int		mgetflg=IOTK_OFF;
	int		cnt = 0;
	int		cnt1 = 0;
	int		ret = 0;
	long	lnRet = 0L;
	int		dataset_no = 0;
	char	buf[256];

	float	*p_dmyadr = NULL;
	int		readcnt = 0;
	AM2_COMMON_SCANTIME chktime[6];
	int 	total_sec[3];
	char    *p_dmybuf = NULL;
	char	fname[6][512];
	float	scale_dummy = 0.0 ;
	char	unit_dummy[11];
	char	chk_geo[41];


	if(name_table == NULL)
	{
		return ERROR_ILLEGAL_FILENAME;
	}
	memcpy(&fname[0], name_table, sizeof(fname));

	for(cnt = 0; cnt < 6; cnt++)
	{
		lnRet = strlen(fname[cnt]);
		if(lnRet <= 0)
		{
			return ERROR_ILLEGAL_FILENAME;
		}
	}

	for(cnt = 0; cnt < 6; cnt++)
	{
		/* Open file */
		hDF_file_id = AMTK_openH5(fname[cnt]);
		if(hDF_file_id < 0)
		{
			return hDF_file_id;
		}
		memset(&buf[0], 0x00, sizeof(buf));
		p_dmybuf = &buf[0];
		
		/* Get meta data (AMTK inside) */
		ret = getMetaDataInside(hDF_file_id, AM2_DEF_META_GEOPHYSICALNAME, &p_dmybuf,
			ERROR_HDF_METAOP_GEONAME, ERROR_HDF_METANG_GEONAME);
		
		/* Close file */
		AMTK_closeH5(hDF_file_id);
		hDF_file_id = 0;
		
		if(ret < 0)
		{
			return ret;
		}

		ret = getGeophysicalInfo(p_dmybuf,  &scale_dummy, unit_dummy);
		if(ret < 0)
		{
			return ret;
		}
		if(cnt == 0)
		{
			strcpy( chk_geo , p_dmybuf) ;
		}
		else 
		{
			ret = strcmp( chk_geo , p_dmybuf);
			if(ret != 0)
			{
				/* GeophysicalName error. */
				return ERROR_GRID03D_GEONAME;
			}
		}
	}
	for(cnt = 0; cnt < 6; cnt++)
	{
		/* Open file */
		hDF_file_id = AMTK_openH5(fname[cnt]);
		p_dmybuf = &buf[0];
		
		/* Get meta data (AMTK inside) */
		ret = getMetaDataInside(hDF_file_id, AM2_DEF_META_ORBITDIRECTION, &p_dmybuf,
			ERROR_HDF_METAOP_ORBITDIR, ERROR_HDF_METANG_ORBITDIR);
		
		/* Close file */
		AMTK_closeH5(hDF_file_id);
		hDF_file_id = 0;
		
		if(ret < 0)
		{
			return ret;
		}
		if((cnt == 0) || (cnt == 2) || (cnt == 4))
		{
			ret = strncmp("Ascending", buf, 9);
			if(ret != 0)
			{
				break;
			}
		}
		else
		{
			ret = strncmp("Descending", buf, 10);
			if(ret != 0)
			{
				break;
			}
		}
	}
	if(cnt != 6)
	{
		/* OrbitDirection error. */
		return ERROR_GRID03D_ORBITDIR;
	}

	for(cnt = 0; cnt < 6; cnt++)
	{
		/* Open file */
		hDF_file_id = AMTK_openH5(fname[cnt]);
		if(hDF_file_id < 0)
		{
			return hDF_file_id;
		}
		memset(&buf[0], 0x00, sizeof(buf));
		p_dmybuf = &buf[0];
		
		/* Get meta data (AMTK inside) */
		ret = getMetaDataInside(hDF_file_id, AM2_DEF_META_PRODUCTIONDATETIME, &p_dmybuf,
			ERROR_HDF_METAOP_PRDCTDTM, ERROR_HDF_METANG_PRDCTDTM);
		
		/* Close file */
		AMTK_closeH5(hDF_file_id);
		hDF_file_id = 0;
		
		if(ret < 0)
		{
			return ret;
		}
		ret = sscanf(&buf[0], "%hd-%hd-%hdT",&chktime[cnt].year,&chktime[cnt].month,&chktime[cnt].day);
		if (ret != 3)
		{
			/* ProductionDateTime error. */
			return ERROR_GRID03D_PRDCTDATE;
		}
		chktime[cnt].hour = 0;
		chktime[cnt].minute = 0;
		chktime[cnt].second = 0;
	}
	for(cnt = 0; cnt < 6; cnt+=2)
	{
		if( (chktime[0+cnt].year == chktime[1+cnt].year)
			&& (chktime[0+cnt].month == chktime[1+cnt].month)
			&& (chktime[0+cnt].day == chktime[1+cnt].day))
		{
			;
		}
		else
		{
			/* ProductionDateTime error. */
			return ERROR_GRID03D_PRDCTDATE;
		}
	}
	DateToSec(chktime[0], &total_sec[0]);
	DateToSec(chktime[2], &total_sec[1]);
	DateToSec(chktime[4], &total_sec[2]);

	if((total_sec[0]-total_sec[1]) != 24*60*60)
	{
		/* ProductionDateTime error. */
		return ERROR_GRID03D_PRDCTDATE;
	}
	if((total_sec[1]-total_sec[2]) != 24*60*60)
	{
		/* ProductionDateTime error. */
		return ERROR_GRID03D_PRDCTDATE;
	}

	/* Open file */
	hDF_file_id = AMTK_openH5(fname[0]);
	if(hDF_file_id < 0)
	{
		return hDF_file_id;
	}
	dataset_no = AM2_GRID_GEOA;
	rank = AMTK_getDimSize(hDF_file_id,  dataset_no, &chksize[0]);
	
	/* Close file */
	AMTK_closeH5(hDF_file_id);
	hDF_file_id = 0;
		
	if(rank < 0)
	{
		return rank;
	}
	
	for(cnt = 1; cnt < 6; cnt++)
	{
		/* Open file */
		hDF_file_id = AMTK_openH5(fname[cnt]);
		if(hDF_file_id < 0)
		{
			return hDF_file_id;
		}
		
		rank = AMTK_getDimSize(hDF_file_id,  dataset_no, &size[0]);
		
		/* Close file */
		AMTK_closeH5(hDF_file_id);
		hDF_file_id = 0;
		
		if(rank < 0)
		{
			return rank;
		}
		if( (chksize[0] != size[0])
			|| (chksize[1] != size[1])
			|| (chksize[2] != size[2]) )
		{
			/* GeophysicalData error. */
			return ERROR_GRID03D_GEOSIZE;
		}
	}

	for(cnt = 0; cnt < 6; cnt++)
	{
		/* Open file */
		hDF_file_id = AMTK_openH5(fname[cnt]);
		if(hDF_file_id < 0)
		{
			return hDF_file_id;
		}

		dataset_no = AM2_GRID_GEOA;
		rank = AMTK_getDimSize(hDF_file_id,  dataset_no, &size[0]);
		if(rank < 0)
		{
			AMTK_closeH5(hDF_file_id);
			return rank;
		}

		if(p_data == NULL){
			p_data  = (float *)getMemoryAlloc(size, rank, sizeof(float), AM2_START_SCANNUM_ALL, AM2_END_SCANNUM_ALL, &getmemsize);
			if(p_data == NULL)
			{
				AMTK_closeH5(hDF_file_id);
				return	ERROR_MALLOC;
			}
		}

		if(*average == NULL)
		{
			*average  = (float *)getMemoryAlloc(size, rank, sizeof(float), AM2_START_SCANNUM_ALL, AM2_END_SCANNUM_ALL, &getmemsize);
			if(*average == NULL)
			{
				free(p_data);
				AMTK_closeH5(hDF_file_id);
				return	ERROR_MALLOC;
			}
			mgetflg = IOTK_ON;
		}
		if(cnt == 0)
		{
			p_dmyadr = *average;
			readcnt = (getmemsize/sizeof(float));
		    for(cnt1 = 0; cnt1 < readcnt; cnt1++)
		    {
			    *(p_dmyadr+cnt1) = 0.0;
		    }
		}
		p_dataBuf = p_data;
		ret = getGridDataSet(hDF_file_id, &p_dataBuf, AM2_START_SCANNUM_ALL, AM2_END_SCANNUM_ALL, AM2_GRID_GEOA, AM2_DTYPE_FLOAT);
		if(ret <= 0)
		{
			AMTK_closeH5(hDF_file_id);
			free(p_data);
			if(mgetflg == IOTK_ON)
			{
				free(average);
				average = NULL;
			}
			return ret;
		}
		p_dmyadr = *average;
		readcnt = (getmemsize/sizeof(float));
		for(cnt1 = 0; cnt1 < readcnt; cnt1++)
		{
			*(p_dmyadr+cnt1) += *(p_data+cnt1);
		}
		/* Close file */
		AMTK_closeH5(hDF_file_id);
		hDF_file_id = 0;
	}
	p_dmyadr = *average;
	readcnt = (getmemsize/sizeof(float));
	for(cnt = 0; cnt < readcnt; cnt++)
	{
		*(p_dmyadr+cnt) /= 6;
	}
	free(p_data);
	return readcnt;
}
int amtk_get_grid03d_(float *average, char *name_table, int name_table_length)
{
	char	*p_fname = NULL;
    char	*p_string = NULL;
	int		ret = 0;

	p_fname = (char *)malloc(sizeof(char)*512*6);
	memset(&p_fname[0], 0x00, sizeof(char)*512*6);

    p_string = string_check_fortran(&name_table[0], name_table_length);
	memcpy(&p_fname[0], p_string, strlen(p_string));
	free(p_string);
	p_string = NULL;

    p_string = string_check_fortran(&name_table[name_table_length], name_table_length);
	memcpy(&p_fname[512], p_string, strlen(p_string));
	free(p_string);
	p_string = NULL;

    p_string = string_check_fortran(&name_table[name_table_length*2], name_table_length);
	memcpy(&p_fname[512*2], p_string, strlen(p_string)); 
	free(p_string);
	p_string = NULL;

    p_string = string_check_fortran(&name_table[name_table_length*3], name_table_length);
	memcpy(&p_fname[512*3], p_string, strlen(p_string));
	free(p_string);
	p_string = NULL;

    p_string = string_check_fortran(&name_table[name_table_length*4], name_table_length);
	memcpy(&p_fname[512*4], p_string, strlen(p_string));
	free(p_string);
	p_string = NULL;

    p_string = string_check_fortran(&name_table[name_table_length*5], name_table_length);
	memcpy(&p_fname[512*5], p_string, strlen(p_string));
	free(p_string);
	p_string = NULL;

    ret = AMTK_get_Grid03D(&average, &p_fname[0]);
	free(p_fname);
    return ret;
}
/* For g77 entry */
int amtk_get_grid03d__(float *average, char *name_table, int name_table_length)
{
    return amtk_get_grid03d_(average, name_table, name_table_length);
}

/** @file amtk_get.c
 * Get HDF Scan Data Quality Structure
 *
 * @author     NEC
 * @date       2011/06/13
 */
int AMTK_get_ScanDataQuality(hid_t HDF_file_id, AMTK_SCAN_DATA_QUALITY **quality, int from_scan, int to_scan, int dataset_no)
{
	ACCESS_LABEL	*p_table=NULL;
	int				size[3] = {0,0,0};
	int				size2[3] = {0,0,0};
	int				ret = 0;
	int				rank = 0;
	int				rank2 = 0;
	int				table_num = 0;
	int				dataset = 0;
	int				datasize = 0;
	int				getmemsize = 0;
	int				getmemsize1 = 0;
	int				overlap = 0;
	unsigned char	*p_r_data = NULL;
	hid_t			memspace = 0;
	hid_t			filespace = 0;
	int				readCnt = 0;
    int             cnt = 0;
    int             cnt1 = 0;
    int             cnt2 = 0;
	AMTK_SCAN_DATA_QUALITY *p_quality1 = NULL;
	int				isLittleEndian = 0;

	/* check system byte order */
	isLittleEndian = 1;
	isLittleEndian = *(char *)(&isLittleEndian);

	ret = CommonPrmCheck(HDF_file_id, from_scan, to_scan, dataset_no);
	if(ret != SUCCESS)
	{
		return ret;
	}

	p_table = search_table(HDF_file_id, dataset_no, &table_num);

	if(p_table == NULL)
	{
		return ERROR_NOT_ACCESSLABEL;
	}

	ret = getOverLapScans( HDF_file_id, &overlap);
	if(ret != SUCCESS)
	{
		return ret;
	}

	rank = AMTK_getDimSize( HDF_file_id, dataset_no , &size[0]);
	if(rank < 0)
	{
		return rank;
	}

	datasize = sizeof(unsigned char);

	p_r_data  = (unsigned char *)getMemoryAlloc(&size[0], rank, datasize, from_scan, to_scan, &getmemsize);
	if(p_r_data == NULL)
	{
		return ERROR_MALLOC;
	}

	dataset = H5Dopen (HDF_file_id, name_table[table_num][p_table->dataset_no], H5P_DEFAULT);
	if(dataset < 0)
	{
		free(p_r_data);
		return ERROR_HDF_DATASET_OPEN;
	}

	ret = setHyperslab(dataset, from_scan, to_scan, &size[0], rank, overlap, &memspace, &filespace);
	if(ret < 0)
	{
		free(p_r_data);
		H5Dclose(dataset);
		return ret;
	}

	ret = H5Dread(dataset,		    /* hid_t dset_id        */
				H5T_NATIVE_UCHAR,	/* hid_t mem_type_id    */
				memspace,			/* hid_t mem_space_id   */
				filespace,			/* hid_t file_space_id  */
				H5P_DEFAULT,		/* hid_t plist_id       */
				p_r_data);			/* void *buf   *out*    */
	
	/* close dataspace */
	H5Sclose(memspace);
	H5Sclose(filespace);
	
	if(ret < 0)
	{
		free(p_r_data);
		H5Dclose(dataset);
		return ERROR_HDF_DATASET_READ;
	}

	if(*quality == NULL)
	{
		datasize = sizeof(AMTK_SCAN_DATA_QUALITY);
		size2[0] = size[0];
		rank2 = 1;
		*quality = (AMTK_SCAN_DATA_QUALITY *)getMemoryAlloc(size2, rank2, datasize, from_scan, to_scan, &getmemsize1);
		if(*quality == NULL)
		{
			free(p_r_data);
			return ERROR_MALLOC;
		}
	}

	readCnt = getmemsize/sizeof(AMTK_SCAN_DATA_QUALITY);
	p_quality1 = *quality;
	for(cnt = 0; cnt < readCnt; cnt++)
	{
		cnt2 = 0;
		
		for(cnt1 = 0; cnt1 < 2; cnt1++)
		{
			checkByteOrder( (unsigned char*)(p_r_data+(512*cnt)+cnt2*4), 
                (unsigned char*)&((p_quality1+cnt)->angular_direction[cnt1]),  isLittleEndian);
			cnt2++;
		}
		checkByteOrder( (unsigned char*)(p_r_data+(512*cnt)+cnt2*4), 
                (unsigned char*)&((p_quality1+cnt)->quality_of_packet),  isLittleEndian);
		cnt2++;
		checkByteOrder( (unsigned char*)(p_r_data+(512*cnt)+cnt2*4), 
                (unsigned char*)&((p_quality1+cnt)->angular_velocity),  isLittleEndian);
		cnt2++;
		for(cnt1 = 0; cnt1 < 64; cnt1++)
		{
			checkByteOrder( (unsigned char*)(p_r_data+(512*cnt)+cnt2*4), 
                (unsigned char*)&((p_quality1+cnt)->calibration[cnt1]),  isLittleEndian);
			cnt2++;
		}
		checkByteOrder( (unsigned char*)(p_r_data+(512*cnt)+cnt2*4), 
                (unsigned char*)&((p_quality1+cnt)->spc_sps_error),  isLittleEndian);
		cnt2++;
		for(cnt1 = 0; cnt1 < 16; cnt1++)
		{
			checkByteOrder( (unsigned char*)(p_r_data+(512*cnt)+cnt2*4), 
                (unsigned char*)&((p_quality1+cnt)->hts_temperature[cnt1]),  isLittleEndian);
			cnt2++;
		}
		for(cnt1 = 0; cnt1 < 33; cnt1++)
		{
			checkByteOrder( (unsigned char*)(p_r_data+(512*cnt)+cnt2*4), 
                (unsigned char*)&((p_quality1+cnt)->parity_error[cnt1]),  isLittleEndian);
			cnt2++;
		}
		for(cnt1 = 0; cnt1 < 4; cnt1++)
		{
			checkByteOrder( (unsigned char*)(p_r_data+(512*cnt)+cnt2*4), 
                (unsigned char*)&((p_quality1+cnt)->quality_info[cnt1]),  isLittleEndian);
			cnt2++;
		}
		for(cnt1 = 0; cnt1 < 6; cnt1++)
		{
			checkByteOrder( (unsigned char*)(p_r_data+(512*cnt)+cnt2*4), 
                (unsigned char*)&((p_quality1+cnt)->spare[cnt1]),  isLittleEndian);
			cnt2++;
		}
	}

	free(p_r_data);

	H5Dclose(dataset);

	return readCnt;
}


int amtk_get_scandataquality_(hid_t *HDF_file_id, AMTK_SCAN_DATA_QUALITY *quality, int *from_scan, int *to_scan, int *dataset_no)
{
	return AMTK_get_ScanDataQuality(*HDF_file_id, &quality, *from_scan, *to_scan, *dataset_no);
}
/* For g77 entry */
int amtk_get_scandataquality__(hid_t *HDF_file_id, AMTK_SCAN_DATA_QUALITY *quality, int *from_scan, int *to_scan, int *dataset_no)
{
	return amtk_get_scandataquality_(HDF_file_id, quality, from_scan, to_scan, dataset_no);
}
/*********************************************************************************/
