/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2010 All rights reserved.                  */
/*  File Name    : amtk_latlon.c                                             */
/*  Abstract     : I/O toolkit latitude longitude function                   */
/*  Function List                                                            */
/*      get_a_value()          : Pick up A1 or A2 value of each channel      */
/*      get_coef()             : Get Coregistration A1, A2 value             */
/*      calc_outer_vector()    : Calculate outer-product                     */
/*      calc_inner_vector()    : Calculate inner-product                     */
/*      radian2latlon()        : Convert radian to latitude/longitude        */
/*      convRegistration()     : Calculate co registration                   */
/*      latlon2radian()        : Convert latitude/longitude to radian        */
/*      LatLon_Conversion_89G()   : Calculate Coregistration vale            */
/*      getRegistration()      : Get resistration                            */
/*      getLowlatlonAverage()  : Get latitude/longitude average for low freq */
/*      getResamplingPoint()   : Get L1R resampling point                    */
/*      getL3LatLonSize()      : Get L3 latitude/longitude size              */
/*      getL3LatLonFileName()  : Get L3 latitude/longitude file name         */
/*      getL3LatLon()          : Get L3 latitude/longitude                   */
/*      getLatLon()            : Get latitude/longitude                      */
/*      setLatLon()            : Set latitude/longitude                      */
/*      AMTK_getLatLon()       : Get latitude/longitude                      */
/*      AMTK_setLatLon         : Set latitude/longitude                      */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2012/02/13                                                    */
/*  Version    1.11                                                          */
/*  Notice                                                                   */
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <math.h>
#include <sys/stat.h>
#include "./AMTK_PLUS.h"
#include "./IOTK_common.h"

#define CHANNEL_NUM 7
#define L3_LATLON_NUM 9
#define REGISTRATION_NUM 6
#define		CONV_RAD_TO_DEG		(57.295779835)		/* 180.0/PAI(3.1415926359)	*/
#define		CONV_DEG_TO_RAD		(0.0174532924216)	/* PAI/180.0 */

extern char **name_table[];
extern char *latlon_name_lo[];
extern char *latlon_name_89a[];
extern char *latlon_name_89b[];

union unit1{
	unsigned char	a[4];
	float	b;
} ;

#define	L3_PROJECTION_EQ      0
#define	L3_PROJECTION_PN      1
#define	L3_PROJECTION_PS      2

char	*l3_projection[] = {"EQR_",               /* EQ:EQR  */
								 "POLARSTEREO_NORTH_", /* PN:PS-N */
								 "POLARSTEREO_SOUTH_", /* PS:PS-S */
								  ""};

#define	L3_LOW_RESO    ("25km.dat")
#define	L3_HIGH_RESO   ("10km.dat")

/** @file amtk_latlon.c
 * Pick up A1 or A2 value of each channel
 *
 * @author     NEC
 * @date       2012/02/09
 */
int get_a_value(char *value, double coef[])
{
	int	i = 0;
	int	bt_type = 0;
	char	buff[256];
	char	*p_file_data = NULL;
	char	*p_str = NULL;
	char	*bt_key[CHANNEL_NUM] = {
		"6G-", "7G-", "10G-", "18G-", "23G-", "36G-", ""};
	
	for (bt_type = 0; bt_type < CHANNEL_NUM; bt_type++)
	{
		p_file_data = value;
		p_str = strstr(p_file_data, bt_key[bt_type]);
		if (p_str == NULL)
		{
			/* Invalid value */
			return FAIL;
		}
		
		p_str += strlen(bt_key[bt_type]);
		for (i = 0; (*p_str != ',') && (*p_str != '\0'); i++, p_str++)
		{
			buff[i] = *p_str;
		}
		buff[i] = '\0';
		coef[bt_type] = atof(buff);
	}
	return SUCCESS;
}

/** @file amtk_latlon.c
 *  Get Coregistration A1, A2 value from Meta data.
 *  The each channel values are set in COEF[] by
 *  channel order. *
 *
 * @author     NEC
 * @date       2012/02/09
 */
int get_coef(hid_t *HDF_file_id, LATLON_COEF COEF[]){
	int		ret = 0;
	int		i = 0, j = 0;
	char	*p_dmybuf = NULL;

	char *name[] = 
	{
		AM2_DEF_META_COREGISTRATIONA1,
		AM2_DEF_META_COREGISTRATIONA2
	};
	char  value[256];
	double coef[CHANNEL_NUM];

	/****Get Coef A1 and A2  ********/
	for (i = 0; i < 2; i++)
	{
		p_dmybuf = &value[0];
		
		/* Get meta data (AMTK inside) */
		ret = getMetaDataInside(*HDF_file_id, name[i], &p_dmybuf,
			(i == 0 ? ERROR_HDF_METAOP_COREGA1 : ERROR_HDF_METAOP_COREGA2),
			(i == 0 ? ERROR_HDF_METANG_COREGA1 : ERROR_HDF_METANG_COREGA2));
		if (ret < 0)
		{
			return ret;
		}
		/* Get Coef string */
		ret = get_a_value(value, coef);
		if (ret < 0)
		{
			return (i == 0 ? ERROR_HDF_METAOP_COREGA1 : ERROR_HDF_METAOP_COREGA2);
		}
		
		for (j = 0; j < CHANNEL_NUM; j++)
		{
			if (i == 0)
			{
				COEF[j].a1 = coef[j];
			}
			else
			{
				COEF[j].a2 = coef[j];
			}
		}
	}
	return SUCCESS;
}

/** @file amtk_latlon.c
 *  Calculate outer-product.
 *
 * @author     NEC
 * @date       2010/06/30
 */
static void calc_outer_vector(	double *p1, double *p2, double *p)
{
	p[0] = p1[1]*p2[2] - p1[2]*p2[1];
	p[1] = p1[2]*p2[0] - p1[0]*p2[2];
	p[2] = p1[0]*p2[1] - p1[1]*p2[0];
	return;
}

/** @file amtk_latlon.c
 *  Calculate inner-product.
 *
 * @author     NEC
 * @date       2010/06/30
 */
static double calc_inner_vector(double *p1,double *p2)
{
	double  val = 0.0;
	double	len = 0.0;

	len = sqrt(p1[0]*p1[0]+p1[1]*p1[1]+p1[2]*p1[2]);
	p1[0] = p1[0]/len;
	p1[1] = p1[1]/len;
	p1[2] = p1[2]/len;

	len = sqrt(p2[0]*p2[0]+p2[1]*p2[1]+p2[2]*p2[2]);
	p2[0] = p2[0]/len;
	p2[1] = p2[1]/len;
	p2[2] = p2[2]/len;

	val = p1[0]*p2[0] + p1[1]*p2[1] + p1[2]*p2[2];

	return val;
}

/** @file amtk_latlon.c
 *  Convert radian to latitude/longitude
 *
 * @author     NEC
 * @date       2011/03/01
 */
static void radian2latlon(double *pos, AM2_COMMON_LATLON *latlon)
{
	double lat_work = 0.0;
	double lon_work = 0.0;
	
	lon_work = atan2(pos[1], pos[0]);
	if ( fabs(pos[0]) < DBL_EPSILON )
	{
		if ( fabs(pos[1]) < DBL_EPSILON )
		{
			if ( pos[2] > 0.0 )
			{
				lat_work = 90.0;
			}
			else
			{
				lat_work = -90.0;
			}
		}
		else
		{
			lat_work = (RADDEG(atan(pos[2]*sin(lon_work)/pos[1])));
		}
	}
	else
	{
		lat_work = (RADDEG(atan(pos[2]*cos(lon_work)/pos[0])));
	}
	lon_work = (RADDEG(lon_work));
	
	latlon->lat = (float) lat_work;
	latlon->lon = (float) lon_work;
	return;
}

/** @file amtk_latlon.c
 *  Calculate co registration.
 *
 * @author     NEC
 * @date       2010/06/30
 */
static void convRegistration(double	*pos89A_odd, double	*pos89A_even, LATLON_COEF info_coef, double *pos_low)
{
	double ex[3], ey[3], ez[3];	
	double	len = 0.0;
	double	theta = 0.0, cos_theta = 0.0;
	double	a1_theta = 0.0, a2_theta = 0.0;

	ex[0] = pos89A_odd[0];
	ex[1] = pos89A_odd[1];
	ex[2] = pos89A_odd[2];
	calc_outer_vector( pos89A_odd, pos89A_even, ez );
	len = sqrt(ez[0]*ez[0]+ez[1]*ez[1]+ez[2]*ez[2]);
	if ( len < 1.0e-6 )
	{
		pos_low[0] = pos89A_odd[0];
		pos_low[1] = pos89A_odd[1];
		pos_low[2] = pos89A_odd[2];
	}
	else
	{
		ez[0] = ez[0] / len; 
		ez[1] = ez[1] / len; 
		ez[2] = ez[2] / len; 
		calc_outer_vector( ez, ex, ey );

		cos_theta = calc_inner_vector( pos89A_odd, pos89A_even );
		theta = acos(cos_theta);
		a1_theta = info_coef.a1 * theta;
		a2_theta = info_coef.a2 * theta;

		pos_low[0] = cos(a2_theta)*( cos(a1_theta)*ex[0] + sin(a1_theta)*ey[0] ) + sin(a2_theta)*ez[0];
		pos_low[1] = cos(a2_theta)*( cos(a1_theta)*ex[1] + sin(a1_theta)*ey[1] ) + sin(a2_theta)*ez[1];
		pos_low[2] = cos(a2_theta)*( cos(a1_theta)*ex[2] + sin(a1_theta)*ey[2] ) + sin(a2_theta)*ez[2];
	}
	return;
}

/** @file amtk_latlon.c
 *  Convert latlon to radian
 *
 * @author     NEC
 * @date       2010/06/30
 */
static void	latlon2radian(	AM2_COMMON_LATLON	*latlon,
							double				*pos )
{
	double	gclat = 0.0, gclon = 0.0;	/* latitude and longitude */

	gclat = DEGRAD(latlon->lat);
	gclon = DEGRAD(latlon->lon);

	pos[0] = cos(gclat)*cos(gclon);
	pos[1] = cos(gclat)*sin(gclon);
	pos[2] = sin(gclat);

	return;
}

/** @file amtk_latlon.c
 *  Calculate Coregistration vale with A1, A2 value.
 *
 * @author     NEC
 * @date       2010/12/24
 */
static int LatLon_Conversion_89G(AM2_COMMON_LATLON	*info89A_odd,
								 AM2_COMMON_LATLON	*info89A_even,
								 LATLON_COEF		info_coef,
								 AM2_COMMON_LATLON	*info_low	)
{
	double pos89A_odd[3] = {0.0}, pos89A_even[3] = {0.0}, pos_low[3] = {0.0};
	
	/* Check abnormal value. */
	if (AM2_DEF_LATLON_ABNML >= info89A_odd->lat)
	{
		/* Return abnormal value. */
		info_low->lat = info89A_odd->lat;
		info_low->lon = info89A_odd->lat;
		return 0;
	}
	else if (AM2_DEF_LATLON_ABNML >= info89A_odd->lon)
	{
		/* Return abnormal value. */
		info_low->lat = info89A_odd->lon;
		info_low->lon = info89A_odd->lon;
		return 0;
	}
	else if (AM2_DEF_LATLON_ABNML >= info89A_even->lat)
	{
		/* Return abnormal value. */
		info_low->lat = info89A_even->lat;
		info_low->lon = info89A_even->lat;
		return 0;
	}
	else if (AM2_DEF_LATLON_ABNML >= info89A_even->lon)
	{
		/* Return abnormal value. */
		info_low->lat = info89A_even->lon;
		info_low->lon = info89A_even->lon;
		return 0;
	}
	
	/* Check illegal value. */
	if ( fabs(info89A_odd->lat)>90.0 )
	{
		return ERROR_ILLEGAL_LAT;
	}
	if ( fabs(info89A_odd->lon)>180.0 )
	{
		return ERROR_ILLEGAL_LON;
	}
	if ( fabs(info89A_even->lat)>90.0 )
	{
		return ERROR_ILLEGAL_LAT;
	}
	if ( fabs(info89A_even->lon)>180.0 )
	{
		return ERROR_ILLEGAL_LON;
	}

	/* Convert coordinates system from Earth-Centered Rotating to Polar */
	latlon2radian(info89A_odd , pos89A_odd ); /* latitude and longitude of No. 2m-1 */
	latlon2radian(info89A_even, pos89A_even); /* latitude and longitude of No. 2m */

	/* Gap calculation of latitude and longitude */
	convRegistration( pos89A_odd, pos89A_even, info_coef, pos_low );

	/* Convert coordinates system from Polar to Earth-Centered Rotating */
	radian2latlon(pos_low, info_low);

	return 0;
}

/** @file amtk_latlon.c
 *  Get resistration
  *
 * @author     NEC
 * @date       2010/12/24
 */
int getRegistration(hid_t HDF_file_id, AM2_COMMON_LATLON **latlon, int from_scan, int to_scan, int dataset_no){

	int				   size[3] = {0,0,0};
	int				   rank = 0;
	int				   getmemsize=0;
	int					readScanSize = 0;
	LATLON_COEF			cOEF[7] = {{0}};
	int					index_89A = 0;
	int					index_low = 0;
	int					pixel = 0, scan = 0;
	AM2_COMMON_LATLON	info_low;
	AM2_COMMON_LATLON	*p_ConvLatLon = NULL;
	int					datacnt = 0;
	int					pnt = 0;
	int					ret = 0;

	rank = AMTK_getDimSize( HDF_file_id, AM2_LATLON_89A , &size[0]);
	if(rank < 0)
	{
		return rank;
	}

	p_ConvLatLon  = (AM2_COMMON_LATLON *)getMemoryAlloc(&size[0], rank, 
                     sizeof(AM2_COMMON_LATLON), from_scan, to_scan, &getmemsize);
	if(p_ConvLatLon == NULL)
	{
		return	ERROR_MALLOC;
	}

	datacnt = getLatLon(HDF_file_id, &p_ConvLatLon, from_scan, to_scan, AM2_LATLON_89A);
	if(datacnt < 0)
	{
		free( p_ConvLatLon );
		p_ConvLatLon = NULL;
		return	datacnt;
	}

	switch(dataset_no)
	{
		case AM2_LATLON_06:
			pnt = 0;
			break;
		case AM2_LATLON_07:
			pnt = 1;
			break;
		case AM2_LATLON_10:
			pnt = 2;
			break;
		case AM2_LATLON_18:
			pnt = 3;
			break;
		case AM2_LATLON_23:
			pnt = 4;
			break;
		case AM2_LATLON_36:
			pnt = 5;
			break;
        default:
			pnt = 0;
			break;
	}
	ret = get_coef(&HDF_file_id, cOEF);
	if (0 > ret)
	{
		free( p_ConvLatLon );
		p_ConvLatLon = NULL;
		return ret;
	}
	
	/*
	 * Scan size = MemorySize / Pixel / datasize
	 * # Scan size != size[0] (from_scan and to_scan affects)
	 */
	readScanSize = getmemsize / size[1] / sizeof(AM2_COMMON_LATLON);
	
	for (scan = 0; scan < readScanSize; scan++)
	{
		for (pixel = 0 ; pixel < (size[1] - 1); pixel+=2)
		{
			/* Get array index. */
			index_89A = (scan * size[1]) + pixel;
			index_low = (scan * (size[1] / 2)) + (pixel / 2);
			
			/* Convert low res. */
			ret = LatLon_Conversion_89G(&p_ConvLatLon[index_89A],
				&p_ConvLatLon[index_89A + 1], cOEF[pnt], &info_low);
			if (ret < 0)
			{
				free(p_ConvLatLon);
				p_ConvLatLon = NULL;
				return ret;
			}
			
			/* Set low res. */
			((*latlon)[index_low]).lat = info_low.lat;
			((*latlon)[index_low]).lon = info_low.lon;
		}
	}
	
	free(p_ConvLatLon);
	p_ConvLatLon = NULL;
	
	return (datacnt / 2);
}

/** @file amtk_latlon.c
 *  Get latitude/longitude average for low frequency
  *
 * @author     NEC
 * @date       2010/12/24
 */
int getLowlatlonAverage(hid_t HDF_file_id, AM2_COMMON_LATLON **latlon, int from_scan, int to_scan, int dataset_no)
{
	static const int low_label[REGISTRATION_NUM] =
	{
		AM2_LATLON_06, /* 06GHz */
		AM2_LATLON_07, /* 07GHz */
		AM2_LATLON_10, /* 10GHz */
		AM2_LATLON_18, /* 18GHz */
		AM2_LATLON_23, /* 23GHz */
		AM2_LATLON_36  /* 36GHz */
	};
	
	int					cnt = 0;
	int					reg_num = 0;
	int					size[3] = {0,0,0};
	int					rank = 0;
	int					ret = 0;
	int					getmemsize=0;
	int					getmemsize1=0;
	int					datasize = 0;
	int					anscnt=0;
	char				memflg=IOTK_OFF;
	AM2_COMMON_LATLON	*p_rdata[REGISTRATION_NUM] = {0};
	AM2_COMMON_LATLON	*p_dmydata = NULL;
	int                 overlap = 0;
	int					i = 0, j = 0;
	int					is_sign_reversed = 0;
	float				lat = 0.0, lon = 0.0;

	dataset_no = AM2_LATLON_06;
	rank = AMTK_getDimSize( HDF_file_id, dataset_no , &size[0]);
	if(rank < 0)
	{
		return rank;
	}

	ret = getOverLapScans( HDF_file_id, &overlap);
	if(ret != SUCCESS)
	{
		return ret;
	}

	datasize = sizeof(AM2_COMMON_LATLON);
	
	/* Get work memory (6GHz ... 36GHz) */
	for (i = 0; i < REGISTRATION_NUM; i++)
	{
		p_rdata[i] = (AM2_COMMON_LATLON *) getMemoryAlloc(&size[0], rank, datasize, from_scan, to_scan, &getmemsize1);
		if (p_rdata[i] == NULL)
		{
			/* Free work memory */
			for (j = 0; j < REGISTRATION_NUM; j++)
			{
				free(p_rdata[j]);
				p_rdata[j] = NULL;
			}
			return ERROR_MALLOC;
		}
	}

	if(*latlon == NULL)
	{
		/* Get output memory */
		*latlon  = (AM2_COMMON_LATLON *) getMemoryAlloc(&size[0], rank, datasize, from_scan, to_scan, &getmemsize);
		if (*latlon == NULL)
		{
			/* Free work memory */
			for (j = 0; j < REGISTRATION_NUM; j++)
			{
				free(p_rdata[j]);
				p_rdata[j] = NULL;
			}
			return ERROR_MALLOC;
		}
		memflg = IOTK_ON;  
	}

	anscnt = getmemsize1/datasize;
	p_dmydata = *latlon;
	for(cnt = 0; cnt < anscnt; cnt++)
	{
		(p_dmydata+cnt)->lat = 0.0;
		(p_dmydata+cnt)->lon = 0.0;
	}
	
	/* Get regisitration */
	for (reg_num = 0; reg_num < REGISTRATION_NUM; reg_num++)
	{
		ret = getRegistration(HDF_file_id, &(p_rdata[reg_num]), from_scan, to_scan, low_label[reg_num]);
		if (ret < 0)
		{
			if (memflg == IOTK_ON)
			{
				free(*latlon);
				*latlon = NULL;
			}
			/* Free work memory */
			for (j = 0; j < REGISTRATION_NUM; j++)
			{
				free(p_rdata[j]);
				p_rdata[j] = NULL;
			}
			return ret;
		}
	}

	/* Get average */
	for (cnt = 0; cnt < anscnt; cnt++)
	{
		/* Check sign */
		for (reg_num = 0; reg_num < REGISTRATION_NUM; reg_num++)
		{
			is_sign_reversed = ((p_rdata[0] + cnt)->lon < 0) ^ ((p_rdata[reg_num] + cnt)->lon < 0);
			if (is_sign_reversed)
			{
				break;
			}
		}
		
		/* Get total number */
		for (reg_num = 0; reg_num < REGISTRATION_NUM; reg_num++)
		{
			lat = (p_rdata[reg_num] + cnt)->lat;
			lon = (p_rdata[reg_num] + cnt)->lon;
			
			(p_dmydata + cnt)->lat += lat;
			(p_dmydata + cnt)->lon += lon;
			
			/* Greater than 180 deg (180 ... -179) */
			if ((is_sign_reversed) &&
				(lon > AM2_DEF_LATLON_ABNML) &&
				(lon < -90.0)) /* exclude (0 ... -1) */
			{
				(p_dmydata + cnt)->lon += 360;
			}
		}
		/* Determining the average value */
		(p_dmydata + cnt)->lat /= REGISTRATION_NUM;
		(p_dmydata + cnt)->lon /= REGISTRATION_NUM;
		
		/* Greater than 180 deg */
		if (180 < (p_dmydata + cnt)->lon)
		{
			(p_dmydata + cnt)->lon -= 360;
		}
	}

	/* Free work memory */
	for (j = 0; j < REGISTRATION_NUM; j++)
	{
		free(p_rdata[j]);
		p_rdata[j] = NULL;
	}
	return anscnt;
}


/** @file amtk_latlon.c
 *  Get L1R Resampling point.
  *
 * @author     NEC
 * @date       2010/11/05
 */
int getResamplingPoint(hid_t HDF_file_id, AM2_COMMON_LATLON **latlon, int from_scan, int to_scan, int dataset_no)
{
	/** Dimension size. */
	int dimsize[3] = {0};
	/** Dimension rank. */
	int rank = 0;
	/** Memory allocation size. */
	int getmemsize = 0;
	/** NumberOfScans, Pixel */
	int pixel = 0, scan = 0;
	/** Index of 89AGHz, Low res */
	int index_89A = 0, index_low = 0;
	/** Max Data size. */
	int datasize = 0;
	/** Reading Scan size. */
	int readScanSize = 0; 
	/** 89AGHz Data. */
	AM2_COMMON_LATLON *p_89Adata = NULL;
	/** Out data. */
	AM2_COMMON_LATLON *p_outdata = NULL;
	
	/* Get 89AGHz Dimsension size. */
	rank = AMTK_getDimSize(HDF_file_id, AM2_LATLON_89A , dimsize);
	if (rank < 0)
	{
		/* error */
		return rank;
	}
	
	/* 89GHz data allocation. */
	p_89Adata = (AM2_COMMON_LATLON *) getMemoryAlloc(dimsize, rank,
		sizeof(AM2_COMMON_LATLON), from_scan, to_scan, &getmemsize);
	if (NULL == p_89Adata)
	{
		/* error */
		return ERROR_MALLOC;
	}
	
	/* Get 89AGHz data. */
	datasize = getLatLon(HDF_file_id, &p_89Adata, from_scan, to_scan,
		AM2_LATLON_89A);
	if (datasize < 0)
	{
		/* Free 89GHz data */
		free(p_89Adata);
		p_89Adata = NULL;
		return datasize;
	}
	
	/*
	 * Scan size = MemorySize / Pixel / datasize
	 * # Scan size != dimsize[0] (from_scan and to_scan affects)
	 */
	readScanSize = getmemsize / dimsize[1] / sizeof(AM2_COMMON_LATLON);
	
	/*
	 * Extract the even.
	 * # 89AGHz : [0] [1] [2] [3] [4] [5] ... [484] [485] pixel #
	 * #           |       |       |            |               #
	 * # Outdata: [0]     [1]     [2]     ... [242]       pixel #
	 */
	for (scan = 0; scan < readScanSize; scan++)
	{
		for (pixel = 0; pixel < (dimsize[1] - 1); pixel+=2)
		{
			index_89A = (scan * dimsize[1]) + pixel;
			index_low = (scan * (dimsize[1] / 2)) + (pixel / 2);
			
			/* Get pixel[0] [1] [2]... */
			p_outdata = &((*latlon)[index_low]);
			
			p_outdata->lat = p_89Adata[index_89A].lat;
			p_outdata->lon = p_89Adata[index_89A].lon;
		}
	}
	
	/* Free 89GHz data */
	free(p_89Adata);
	p_89Adata = NULL;
	
	return (datasize / 2);
}


/** @file amtk_latlon.c
 *  Get L3 latitude/longitude size
 *
 * @author     NEC
 * @date       2010/06/30
 */
int getL3LatLonSize(char *fname, int *size)
{
    int         rank = 0;
    int         ret = 0;
   	struct stat	buf;

	ret = stat(fname, &buf);
	if(ret == 0)
	{
		size[0] = ((buf.st_size /sizeof(float))/2);
		size[1] = 2;
		size[2] = 0;
		rank = 2;
	}
	else
	{
		rank = ERROR_FILE_OPEN;
	}

    return rank;
}

/** @file amtk_hdf.c
 * Get Geophysical Information
 *
 * @author     NEC
 * @date       2010/06/30
 */
int getGeophysicalLatlon(char* geophysicalName, int projection_no, char* abbreviation)
{
	FILE	*p_fp = NULL;
	char	buf[256];
	char	abb_table[3][20];
	char	*ptn = NULL;
	char	*p_str = NULL;
	int	ret=0;
	int	find_flag=0;

	p_str = getenv("GEOPHYSICALFILE");
	if(p_str != NULL)
	{
		p_fp = fopen(p_str,"r");
	}
	else
	{
		p_fp = fopen("geophysical_file","r");
	}
	if(p_fp == NULL)
	{
		return ERROR_GEOPHSICAL_FILE;
	}
	while(NULL != fgets(buf,256,p_fp))
	{
		if( buf[0] == '#')
		{
			continue;
		}
		ptn = strtok(buf,",");
		ret = strcmp( ptn, geophysicalName);
		if( ret == 0 )
		{
			ptn = strtok(NULL,",");
			if( ptn == NULL)
			{
				break;
			}
			ptn = strtok(NULL,",");
			if( ptn == NULL)
			{
				break;
			}
			ptn = strtok(NULL,",");
			if( ptn == NULL)
			{
				break;
			}
			strcpy( &abb_table[L3_PROJECTION_EQ][0], ptn );
			ptn = strtok(NULL,",");
			if( ptn == NULL)
			{
				break;
			}
			strcpy( &abb_table[L3_PROJECTION_PN][0], ptn );
			ptn = strtok(NULL,"\n");
			if( ptn == NULL)
			{
				break;
			}
			strcpy( &abb_table[L3_PROJECTION_PS][0], ptn );
			find_flag = 1;
			break;
		}
	}
	fclose(p_fp);

	if( find_flag != 1)
	{
		return ERROR_GEOPHSICAL_FILE;
	}

	ret = strcmp( &abb_table[projection_no][0], "-" );
	if( ret == 0)
	{
		return ERROR_GEOPHSICAL_FILE;
	}

	strcpy( abbreviation, &abb_table[projection_no][0]);

	ret = strcmp( abbreviation, "EQR" );
	if( ret == 0 )
	{
		strcpy( abbreviation, "");
	}else
	{
		strcat( abbreviation, "_");
	}
	return SUCCESS ;
}


/** @file amtk_latlon.c
 *  Get L3 latitude/longitude file name
  *
 * @author     NEC
 * @date       2010/06/30
 */
int	getL3LatLonFileName(hid_t HDF_file_id, char *fname)
{
	int	ret = 0;
	char	projection[80];
	char	granuleID[80];
	char	geophysicalName[80];
	char	fileName[80];
	char	abbreviation[10];
	int	projection_no = 0;
	char    *p_dmybuf = NULL;
	char    *p_str = NULL;

    p_dmybuf = &projection[0];
	
	/* Get "Projection" */
	ret = getMetaDataInside(HDF_file_id, AM2_DEF_META_PROJECTION, &p_dmybuf,
		ERROR_HDF_METAOP_PRJCTION, ERROR_HDF_METANG_PRJCTION);
	if(ret < 0)
	{
		return ret;
	}
	ret = strcmp(&projection[0],"EQR");
	if(ret == 0)
	{
		projection_no = L3_PROJECTION_EQ;
	}else
	{
		ret = strcmp(&projection[0],"PS-N");
		if(ret == 0)
		{
			projection_no = L3_PROJECTION_PN;
		}else
		{
			ret = strcmp(&projection[0],"PS-S");
			if(ret == 0)
			{
				projection_no = L3_PROJECTION_PS;
			}else
			{
				return ERROR_ILLEGAL_METADATA;
			}
		}
	}
	strcpy( fileName, l3_projection[projection_no]);

	p_dmybuf = &geophysicalName[0];
	
	/* Get "GeophysicalName" */
	ret = getMetaDataInside(HDF_file_id, AM2_DEF_META_GEOPHYSICALNAME, &p_dmybuf,
		ERROR_HDF_METAOP_GEONAME, ERROR_HDF_METANG_GEONAME);
	if(ret < 0)
	{
		return ret;
	}

	ret = getGeophysicalLatlon(geophysicalName, projection_no, abbreviation);
	if(ret < 0)
	{
		return ret;
	}
	strcat(fileName, abbreviation);

	p_dmybuf = &granuleID[0];
	
	/* Get "GranuleID" */
	ret = getMetaDataInside(HDF_file_id, AM2_DEF_META_GRANULEID, &p_dmybuf,
		ERROR_HDF_METAOP_GRANULE, ERROR_HDF_METANG_GRANULE);
	if(ret < 0)
	{
		return ret;
	}
	if(granuleID[GRANULEID_RES_POS] == GRANULEID_RES_LO)
	{
		strcat(fileName, L3_LOW_RESO);
	}
	else
	{
		strcat(fileName, L3_HIGH_RESO);
	}

	IOTK_printf("L3 Geophysical_latlon file=%s\n", fileName );

	strcpy( fname, &fileName[0] );

	p_str = getenv("L3LATLONFILEDIR");
	if(p_str == NULL)
    {   
		strcpy(fname, &fileName[0]);
    }
	else
	{
		strcpy(fname, p_str);
		if(fname[strlen(fname)] != '/')
		{
			strcat(fname, "/");
		}
		strcat(fname, &fileName[0]);
	}

	return SUCCESS;
}

/** @file amtk_latlon.c
 *  Get L3 latitude/longitude
  *
 * @author     NEC
 * @date       2010/06/30
 */
int getL3LatLon(hid_t HDF_file_id, AM2_COMMON_LATLON **latlon)
{
	FILE				*p_fp = NULL;
	float				rdata[2];
	int					cnt = 0;
	int					ret = 0;
	long				lnRet = 0L;
	AM2_COMMON_LATLON	*p_data = NULL;
	int					datasize = 0;
	int					memsize = 0;
	union unit1			chgdata;
	union unit1			chgdata1;
	char				l3_fname[1024];
	int				    size[3] = {0,0,0};
	int				    rank = 0;
	int					isLittleEndian = 0;

	ret = getL3LatLonFileName(HDF_file_id, l3_fname);
	if(ret < 0)
	{
		return ret;
	}

    rank = getL3LatLonSize(l3_fname, &size[0]);
    if(rank < 0)
    {
		return rank;
    }

	p_fp = fopen(l3_fname,"rb");
	if(p_fp == NULL)
	{
		return ERROR_FILE_OPEN;
	}

	memsize = size[0]; 
	if(*latlon == NULL)
	{
		datasize = sizeof(AM2_COMMON_LATLON);
		*latlon = (AM2_COMMON_LATLON *)malloc(memsize*datasize);
		if(*latlon == NULL)
		{
			fclose(p_fp);
			return ERROR_MALLOC;
		}
	}

	/* check system byte order */
	isLittleEndian = 1;
	isLittleEndian = *(char *)(&isLittleEndian);

	p_data = *latlon;
	for(cnt = 0; cnt < memsize; cnt++)	
	{
		lnRet = fread(&rdata[0], sizeof(float), 2, p_fp);
		if(lnRet == 0)
		{
			break;
		}

		if(isLittleEndian == 0)
		{
			chgdata.b = rdata[0];
			chgdata1.a[0] = chgdata.a[3];
			chgdata1.a[1] = chgdata.a[2];
			chgdata1.a[2] = chgdata.a[1];
			chgdata1.a[3] = chgdata.a[0];
			(p_data+cnt)->lat = chgdata1.b;
			chgdata.b = rdata[1];
			chgdata1.a[0] = chgdata.a[3];
			chgdata1.a[1] = chgdata.a[2];
			chgdata1.a[2] = chgdata.a[1];
			chgdata1.a[3] = chgdata.a[0];
			(p_data+cnt)->lon = chgdata1.b;
		}
		else
		{
			chgdata.b = rdata[0];
			(p_data+cnt)->lat = chgdata.b;

			chgdata.b = rdata[1];
			(p_data+cnt)->lon = chgdata.b;
		}
	}
	fclose(p_fp);
	return memsize;
}

/** @file amtk_latlon.c
 *  Get latitude/longitude
  *
 * @author     NEC
 * @date       2010/06/30
 */
int getLatLon(hid_t HDF_file_id, AM2_COMMON_LATLON **latlon, int from_scan, int to_scan, int dataset_no)
{

	int					ret = 0;
	int					overlap = 0;
	float				*p_data=NULL;
	AM2_COMMON_LATLON	*p_dmydata = NULL;
	int					cnt = 0;
	int					cnt1 = 0;
	int					datasize=0;
	hid_t				filespace = -1;
	hid_t				memspace = -1;
	int					size[3] = {0,0,0};
	int					rank = 0;
	hid_t				dataset = -1;
	int					flag=IOTK_OFF;
	int					getmemsize=0;
	int					getmemsize1=0;
	int					datacnt = 0;
	float				scale = 0.0;

	for(cnt = 0; cnt < 2; cnt++)
	{
		if( (dataset_no == AM2_LATLON_RS_LO)
			|| (dataset_no == AM2_LATLON_L2_LO) )
		{
			dataset = H5Dopen (HDF_file_id, latlon_name_lo[cnt], H5P_DEFAULT);
		}
		else if( (dataset_no == AM2_LATLON_89A)
			|| (dataset_no == AM2_LATLON_RS_89A)
			|| (dataset_no == AM2_LATLON_L2_89A) )
		{
			dataset = H5Dopen (HDF_file_id, latlon_name_89a[cnt], H5P_DEFAULT);
		}
		else
		{
			dataset = H5Dopen (HDF_file_id, latlon_name_89b[cnt], H5P_DEFAULT);
		}
		if(dataset < 0)
		{
			ret = ERROR_HDF_DATASET_OPEN;
			break;
		}

		rank = getDimSize(dataset, &size[0]);
		if(rank < 0)
		{
			ret = rank;
			break;
		}

		ret = getScaleFactor(dataset, &scale);
		if(ret != SUCCESS)
		{
			break;
		}

		ret = getOverLapScans( HDF_file_id, &overlap);
		if(ret != SUCCESS)
		{
			break;
		}

		if(p_data == NULL)
		{
			datasize = sizeof(float);
			p_data  = (float *)getMemoryAlloc(&size[0], rank, datasize, from_scan, to_scan, &getmemsize);
			if(p_data == NULL)
			{
				ret = ERROR_MALLOC;
				break;
			}
		}

		if(*latlon == NULL)
		{
			datasize = sizeof(AM2_COMMON_LATLON);
			*latlon = (AM2_COMMON_LATLON *)getMemoryAlloc(size, rank, datasize, from_scan, to_scan, &getmemsize1);
			if(*latlon == NULL)
			{
				ret = ERROR_MALLOC;
				break;
			}
		    flag = IOTK_ON;
		}

		ret = setHyperslab(dataset, from_scan, to_scan, &size[0], rank, overlap, &memspace, &filespace);
		if(ret < 0)
		{
			break;
	    }
		ret = H5Dread(dataset,		    		/* hid_t dset_id        */
					  H5T_NATIVE_FLOAT,			/* hid_t mem_type_id    */
					  memspace,					/* hid_t mem_space_id   */
					  filespace,				/* hid_t file_space_id  */
					  H5P_DEFAULT,				/* hid_t plist_id       */
					  p_data);					/* void *buf   *out*    */
		
		/* close dataspace */
		H5Sclose(memspace);
		memspace = -1;
		H5Sclose(filespace);
		filespace = -1;
		
		if(ret < 0)
		{
			ret = ERROR_HDF_DATASET_READ;
			break;
		}
		datacnt = (getmemsize/sizeof(float));
		p_dmydata = *latlon;
		if(cnt == 0)
		{
			for(cnt1 = 0; cnt1 < datacnt; cnt1++)
			{
				(p_dmydata+cnt1)->lat = (float )(*(p_data+cnt1)*scale);
			}
		}
		else
		{
			for(cnt1 = 0; cnt1 < datacnt; cnt1++)
			{
				(p_dmydata+cnt1)->lon = (float )(*(p_data+cnt1)*scale);
			}
		}
		
		/* close dataset */
		H5Dclose(dataset);
		dataset = -1;
	}
	if(p_data != NULL)
	{
		free(p_data);
	}
	if(memspace >= 0)
	{
		H5Sclose(memspace);
	}
	if(filespace >= 0)
	{
		H5Sclose(filespace);
	}
	if(dataset >= 0)
	{
		H5Dclose(dataset);
	}
	if(ret < 0)
	{
		if(flag == IOTK_ON)
		{
			free(*latlon);
			*latlon = NULL;
		}
		datacnt = ret;
	}
	return datacnt;
}

/** @file amtk_latlon.c
 *  Set latitude/longitude
  *
 * @author     NEC
 * @date       2010/06/30
 */
int setLatLon(hid_t HDF_file_id, AM2_COMMON_LATLON *latlon, int from_scan, int to_scan, int dataset_no)
{

	int		ret = 0;
	int		overlap = 0;
	float	*p_data=NULL;
	int		cnt = 0;
	int		cnt1 = 0;
	int		datasize = 0;
	hid_t	filespace = -1;
	hid_t	memspace = -1;
	int		size[3] = {0,0,0};
	int		rank = 0;
	hid_t	dataset = -1;
	int		getmemsize=0;
	int		datacnt = 0;
	float	scale = 0.0;
	float	ans = 0.0;
	int		mret = SUCCESS;

	for(cnt = 0; cnt < 2; cnt++)
	{
		if(dataset_no == AM2_LATLON_L2_LO)
		{
			dataset = H5Dopen (HDF_file_id, latlon_name_lo[cnt], H5P_DEFAULT);
		}
		else if( (dataset_no == AM2_LATLON_89A)
			|| (dataset_no == AM2_LATLON_RS_89A)
			|| (dataset_no == AM2_LATLON_L2_89A) )
		{
			dataset = H5Dopen (HDF_file_id, latlon_name_89a[cnt], H5P_DEFAULT);
		}
		else
		{
			dataset = H5Dopen (HDF_file_id, latlon_name_89b[cnt], H5P_DEFAULT);
		}
		if(dataset < 0)
		{
			mret = ERROR_HDF_DATASET_OPEN;
			break;
		}

		rank = getDimSize(dataset, &size[0]);
		if(rank < 0)
		{
			mret = rank;
			break;
		}

		ret = getScaleFactor(dataset, &scale) ;
		if(ret != SUCCESS)
		{
			mret = ret;
			break;
		}

		ret = getOverLapScans( HDF_file_id, &overlap);
		if(ret != SUCCESS)
		{
			mret = ret;
			break;
		}

		if(p_data == NULL)
		{
			datasize = sizeof(float);
			p_data  = (float *)getMemoryAlloc(&size[0], rank, datasize, from_scan, to_scan, &getmemsize);
			if(p_data == NULL)
			{
				mret = ERROR_MALLOC;
				break;
			}
		}

		ret = setHyperslab(dataset, from_scan, to_scan, &size[0], rank, overlap, &memspace, &filespace);
		if(ret < 0)
		{
			mret = ret;
			break;
		}
		datacnt = (getmemsize/sizeof(float));
		if(cnt == 0)
		{
			for(cnt1 = 0; cnt1 < datacnt; cnt1++)
			{
				ans = (latlon+cnt1)->lat/scale;
				if(floor(ans) < 32768)
				{
				    *(p_data+cnt1) = ans;
				}
				else
				{
				    *(p_data+cnt1) = AM2_DEF_RMISS;
				}
			}
		}
		else
		{
			for(cnt1 = 0; cnt1 < datacnt; cnt1++)
			{
				ans = (latlon+cnt1)->lon/scale;
				if(floor(ans) < 32768)
				{
				    *(p_data+cnt1) = ans;
				}
				else
				{
				    *(p_data+cnt1) = AM2_DEF_RMISS;
				}
			}
		}

		ret = H5Dwrite(dataset,		    		/* hid_t dset_id        */
						H5T_NATIVE_FLOAT,		/* hid_t mem_type_id    */
						memspace,				/* hid_t mem_space_id   */
						filespace,				/* hid_t file_space_id  */
						H5P_DEFAULT,			/* hid_t plist_id       */
						p_data);					/* void *buf   *out*    */
		
		/* close dataspace */
		H5Sclose(memspace);
		memspace = -1;
		H5Sclose(filespace);
		filespace = -1;
		
		if(ret < 0)
		{
			mret = ERROR_HDF_DATASET_WRITE;
			break;
		}
		
		/* close dataset */
		H5Dclose(dataset);
		dataset = -1;
	}
	if(memspace >= 0)
	{
		H5Sclose(memspace);
	}
	if(filespace >= 0)
	{
		H5Sclose(filespace);
	}
	if(dataset >= 0)
	{
		H5Dclose(dataset);
	}
	if(p_data != NULL)
	{
		free(p_data);
	}
	return mret;
}

/** @file amtk_latlon.c
 *  Get latitude/longitude
  *
 * @author     NEC
 * @date       2010/06/30
 */
int AMTK_getLatLon(hid_t HDF_file_id, AM2_COMMON_LATLON **latlon, int from_scan, int to_scan, int dataset_no)
{
    int ret = 0;

	ret = CommonPrmCheck(HDF_file_id, from_scan, to_scan, dataset_no);
	if(ret != SUCCESS)
	{
		return ret;
	}
	switch(dataset_no)
	{
		case AM2_LATLON_06:
		case AM2_LATLON_07:
		case AM2_LATLON_10:
		case AM2_LATLON_18:
		case AM2_LATLON_23:
		case AM2_LATLON_36:
			ret = getRegistration(HDF_file_id, latlon, from_scan, to_scan, dataset_no);
			if(ret < 0)
			{
				return ret;
			}
			break;
		case AM2_LATLON_89A:
		case AM2_LATLON_89B:
		case AM2_LATLON_RS_89A:
		case AM2_LATLON_RS_89B:
			ret = getLatLon(HDF_file_id, latlon, from_scan, to_scan, dataset_no);
			if(ret < 0)
			{
				return ret;
			}
			break;
		case AM2_LATLON_LO:
			ret = getLowlatlonAverage(HDF_file_id, latlon, from_scan, to_scan, dataset_no);
			if(ret < 0)
			{
				return ret;
			}
			break;
		case AM2_LATLON_RS_LO:
			ret = getResamplingPoint(HDF_file_id, latlon, from_scan, to_scan, dataset_no);
			if (ret < 0)
			{
				return ret;
			}
			break;
		case AM2_LATLON_L2_LO:
		case AM2_LATLON_L2_89A:
		case AM2_LATLON_L2_89B:
			ret = getLatLon(HDF_file_id, latlon, from_scan, to_scan, dataset_no);
			if(ret < 0)
			{
				return ret;
			}
			break;
		case AM2_GRID_LATLON:
			ret = getL3LatLon(HDF_file_id, latlon);
			if(ret < 0)
			{
				return ret;
			}
			break;
		default:
			ret = ERROR_ACCESSLABEL_WRITE_NG;
			break;
	}
	return	ret;
}
int amtk_getlatlon_(hid_t *HDF_file_id, AM2_COMMON_LATLON *latlon, int *from_scan, int *to_scan, int *dataset_no)
{
	return AMTK_getLatLon(*HDF_file_id, &latlon, *from_scan, *to_scan, *dataset_no);
}
/* For g77 entry */
int amtk_getlatlon__(hid_t *HDF_file_id, AM2_COMMON_LATLON *latlon, int *from_scan, int *to_scan, int *dataset_no)
{
	return amtk_getlatlon_(HDF_file_id, latlon, from_scan, to_scan, dataset_no);
}

/** @file amtk_latlon.c
 *  Set latitude/longitude
  *
 * @author     NEC
 * @date       2010/06/30
 */
int AMTK_setLatLon(hid_t HDF_file_id, AM2_COMMON_LATLON *latlon, int from_scan, int to_scan, int dataset_no)
{
	int ret = 0;

	if(latlon == NULL)
	{
		return ERROR_NOT_WRITEDATA;
	}
	
	ret = CommonPrmCheck(HDF_file_id, from_scan, to_scan, dataset_no);
	if(ret != SUCCESS)
	{
		return ret;
	}

	if( (dataset_no == AM2_LATLON_89A)
		|| (dataset_no == AM2_LATLON_89B)
		|| (dataset_no == AM2_LATLON_RS_89A)
		|| (dataset_no == AM2_LATLON_RS_89B)
		|| (dataset_no == AM2_LATLON_L2_LO)
		|| (dataset_no == AM2_LATLON_L2_89A)
		|| (dataset_no == AM2_LATLON_L2_89B) )
	{
		ret = setLatLon(HDF_file_id, latlon, from_scan, to_scan, dataset_no);
		if(ret != SUCCESS)
		{
			return ret;
		}
	}
	else
	{
		return ERROR_ACCESSLABEL_WRITE_NG;
	}
	return SUCCESS;
}
int amtk_setlatlon_(hid_t *HDF_file_id, AM2_COMMON_LATLON *latlon, int *from_scan, int *to_scan, int *dataset_no)
{
	return AMTK_setLatLon(*HDF_file_id, latlon, *from_scan, *to_scan, *dataset_no);
}
/* For g77 entry */
int amtk_setlatlon__(hid_t *HDF_file_id, AM2_COMMON_LATLON *latlon, int *from_scan, int *to_scan, int *dataset_no)
{
	return amtk_setlatlon_(HDF_file_id, latlon, from_scan, to_scan, dataset_no);
}

