/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2010 All rights reserved.                  */
/*  File Name    : amtk_scantime.c                                           */
/*  Abstract     : I/O toolkit get function                                  */
/*  Function List                                                            */
/*      getLeapSec()                 : Get leap sec                          */
/*      getLeapSecbyTAI()            : Get leap sec by TAI93                 */
/*      readLeapSec()                : Read leap sec                         */
/*      checkLeapSecDate()           : Check leap sec and date               */
/*      AMTK_getScanTime()           : Get scan time                         */
/*      AMTK_setScanTime()           : Set scan time                         */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2011/06/13                                                    */
/*  Version    1.10                                                          */
/*             1.11 2015/01/05 Add handling Scantime data for AMSR-E         */
/*  Notice                                                                   */
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <math.h>
#include "./AMTK_PLUS.h"
#include "./IOTK_common.h"

#define APPROXIMATELY_ZERO 10e-5

extern char **name_table[];

/** @file amtk_scantime.c
 * Get leap sec
 *
 * @author     NEC
 * @date       2010/06/30
 */
int getLeapSec(LEAPSEC	*t_LeapSec, int year, int month, double *deltat )
{
	int     nDAT = 0;
	int		cnt = 0;
	int		m = 0;

	*deltat = 0.0;
 
    while(t_LeapSec[nDAT].iYear != 0)
	{
		nDAT++;
	}
 
	m = 12*year + month;
	for (cnt = nDAT-1; cnt >=0; cnt--)
	{
		if (m >= (12 * t_LeapSec[cnt].iYear + t_LeapSec[cnt].i_Month)){
			break;
		}
	}

	*deltat = t_LeapSec[cnt].d_Delat;

	return SUCCESS;

}
/** @file amtk_scantime.c
 * Get leap sec by TAI
 *
 * @author     NEC
 * @date       2010/06/30
 */
int getLeapSecbyTAI(LEAPSEC	*t_LeapSec,  double tai93, double *deltat )
{
	int     nDAT = 0;
	int		cnt = 0;

	*deltat = 0.0;
 
    while(t_LeapSec[nDAT].iYear != 0)
	{
		nDAT++;
	}
 
	for (cnt = nDAT-1; cnt >=0; cnt--)
	{
		if (tai93 >= t_LeapSec[cnt].just_Sec){
			break;
		}
	}

	*deltat = t_LeapSec[cnt].d_Delat;

	return SUCCESS;

}

/** @file amtk_scantime.c
 * Read leap sec
 *
 * @author     NEC
 * @date       2010/06/30
 */
int readLeapSec(LEAPSEC *LeapSec)
{
	FILE	*p_fp = NULL;
	char	buf[256];
	char	*ptn = NULL;
	int		cnt=0;
	char    *p_str = NULL;

	p_str = getenv("AMSR2_LEAP_DATA");
	if(p_str != NULL)
	{
		p_fp = fopen(p_str,"r");
	}
	else
	{
		p_fp = fopen("leapsec.dat","r");
	}
	if(p_fp == NULL)
	{
		return ERROR_LEAP_SEC;
	}
	fgets(buf,256,p_fp);
	while(NULL != fgets(buf,256,p_fp))
	{
		ptn = strtok(buf," ");
		LeapSec[cnt].iYear = atoi(ptn);

		ptn = strtok(NULL," ");
		LeapSec[cnt].i_Month  = atoi(ptn);

		LeapSec[cnt].i_Day = 1;

		ptn = strtok(NULL," ");
		LeapSec[cnt].d_Delat = atof(ptn);

		ptn = strtok(NULL," ");
		LeapSec[cnt].drift[0] = atof(ptn);

		ptn = strtok(NULL," ");
		LeapSec[cnt].drift[1] = atof(ptn);

		ptn = strtok(NULL," ");
		LeapSec[cnt].just_Sec = atof(ptn);

      IOTK_printf("%d/%d/%d\n",LeapSec[cnt].iYear,LeapSec[cnt].i_Month,LeapSec[cnt].i_Day);
      IOTK_printf("%lf %lf %lf\n",LeapSec[cnt].d_Delat,LeapSec[cnt].drift[0],LeapSec[cnt].drift[1]);
      IOTK_printf("%lf\n",LeapSec[cnt].just_Sec);

		cnt++;
	}
	fclose(p_fp);

	return(cnt);
}

/** @file amtk_scantime.c
 * Check leap sec and date
 *
 * @author     NEC
 * @date       2010/06/30
 */
void checkLeapSecDate(LEAPSEC *t_LeapSec, AM2_COMMON_SCANTIME *scan_time, double tai93sec)
{
	int		cnt = 0;

	while(t_LeapSec[cnt].iYear != 0)
	{
		if(fabs(t_LeapSec[cnt].just_Sec-1.0  - tai93sec ) < DBL_EPSILON )
		{
			if( (t_LeapSec[cnt].iYear   == scan_time->year)
				&& (t_LeapSec[cnt].i_Month == scan_time->month)
				&& (t_LeapSec[cnt].i_Day   == scan_time->day)
				&& (0                      == scan_time->hour)
				&& (0                      == scan_time->minute)
				&& (0                      == scan_time->second) )
			{
				if(scan_time->month == 1)
				{
					scan_time->year--;
					scan_time->month  = 12;
					scan_time->day    = 31;
					scan_time->hour   = 23;
					scan_time->minute = 59;
					scan_time->second = 60;
				}
				else
				{
					scan_time->month  = 6;
					scan_time->day    = 30;
					scan_time->hour   = 23;
					scan_time->minute = 59;
					scan_time->second = 60;
				}
				IOTK_printf("Change year=%d   %d  %d %d %d %d\n", 
								scan_time->year, scan_time->month, scan_time->day,
								scan_time->hour,scan_time->minute,scan_time->second );
				break;
			}
		}
		cnt++;
	}
	return;
}

/** @file amtk_scantime.c
 * Get scan time
 *
 * @author     NEC
 * @date       2011/06/13
 */
int AMTK_getScanTime(hid_t HDF_file_id, int from_scan, int to_scan, AM2_COMMON_SCANTIME **scan_time){
	int					table_num = 0;
	ACCESS_LABEL		*p_table=NULL;
	int     			dataset_no = 0;
	int					rank = 0;
	int					size[3]={0,0,0};
	hid_t				dataset = 0;
	double				*p_rdata=NULL;	
	int					getmemsize = 0;
	int					getmemsize1 = 0;
	int					ret = 0;
	int					mallocflg = IOTK_OFF;
	double 				fd = 0.0;
	double				deltat = 0.0;
	double				leapsec93 = 0.0;
	AM2_COMMON_SCANTIME *p_dmydata = NULL;
	int					cnt = 0;
	int					readcnt = 0;
	double				m = 0.0, n = 0.0;
	float				scale = 0.0;
	double				wtime = 0.0;
	hid_t				memspace = 0;
	hid_t				filespace = 0;
	int					overlap = 0;
	double				f = 0.0;
	int					t_sec = 0;
	LEAPSEC				t_LeapSec[50];

	if(HDF_file_id == 0)
	{
		return ERROR_ILLEGAL_HDF_FILEID;
	}

	dataset_no = AM2_SCAN_TIME_DEF;
    p_table = search_table(HDF_file_id, dataset_no, &table_num);

	if(p_table == NULL)
	{
		return ERROR_NOT_ACCESSLABEL;
	}

	rank = AMTK_getDimSize(HDF_file_id,  dataset_no, &size[0]);
	if(rank < 0)
	{
		IOTK_printf("getScanTime ERROR - 2[%s]\n",name_table[table_num][p_table->dataset_no]);
		return rank;
	}

	ret = getOverLapScans( HDF_file_id, &overlap);
	if(ret != SUCCESS)
	{
		return ret;
	}

	dataset  = H5Dopen (HDF_file_id, name_table[table_num][p_table->dataset_no], H5P_DEFAULT);
	if(dataset < 0)
	{
		IOTK_printf("getScanTime ERROR - 1[%s]\n",name_table[table_num][p_table->dataset_no]);
		return ERROR_HDF_DATASET_OPEN;
	}

	ret = getScaleFactor(dataset, &scale);
	if(ret != SUCCESS)
	{
		H5Dclose(dataset);
		return ret;
	}

	p_rdata  = (void *)getMemoryAlloc(&size[0], rank, sizeof(double), from_scan, to_scan, &getmemsize);
	if(p_rdata == NULL)
	{
		H5Dclose(dataset);
		return	ERROR_MALLOC;
	}

	ret = setHyperslab(dataset, from_scan, to_scan, &size[0], rank, overlap, &memspace, &filespace);
	if(ret < 0)
	{
		free(p_rdata);
		H5Dclose(dataset);
		return ret;
	}

	ret = H5Dread(dataset,		    		/* hid_t dset_id        */
				  H5T_NATIVE_DOUBLE,		/* hid_t mem_type_id    */
				  memspace,					/* hid_t mem_space_id   */
				  filespace,				/* hid_t file_space_id  */
				  H5P_DEFAULT,				/* hid_t plist_id       */
				  p_rdata);					/* void *buf   *out*    */
	
	/* close dataspace */
	H5Sclose(memspace);
	H5Sclose(filespace);
	
	if(ret < 0)
	{
		free(p_rdata);
		H5Dclose(dataset);

		return ERROR_HDF_DATASET_READ;
	}

	memset(&t_LeapSec[0], 0x00, (sizeof(LEAPSEC)*50));
	if(*scan_time == NULL)
	{
		*scan_time  = (AM2_COMMON_SCANTIME *)getMemoryAlloc(&size[0], rank, sizeof(AM2_COMMON_SCANTIME), 
												from_scan, to_scan, &getmemsize1);
		if(*scan_time == NULL)
		{
			H5Dclose(dataset);
			free(p_rdata);
			return	ERROR_MALLOC;
		}
		mallocflg = IOTK_ON;
	}

	ret = readLeapSec(&t_LeapSec[0]);
	if (ret < 0)
	{
		if(mallocflg == IOTK_ON)
		{
			free(*scan_time);
			*scan_time = NULL;
		}
		return ret;
	}
	
	p_dmydata = *scan_time;
	fd = 0;
	ret = 0;

	/* get 93 leap sec */
	ret = getLeapSec(&t_LeapSec[0], 1993, 1, &leapsec93);
	if(ret < 0)
	{
		if(mallocflg == IOTK_ON)
		{
			free(*scan_time);
			*scan_time = NULL;
		}
		return ret;
	}

	readcnt = getmemsize/sizeof(double);
	for(cnt = 0; cnt < readcnt; cnt++)
	{
		/* Handling to read "Error scan_time data" of AMSR-E. */
		if( ( *(p_rdata+cnt) - AM2_DEF_RMISS) < APPROXIMATELY_ZERO )
		{
			(p_dmydata+cnt)->tai93sec = AM2_DEF_RMISS;
			(p_dmydata+cnt)->year = 0;
			(p_dmydata+cnt)->month = 0;
			(p_dmydata+cnt)->day = 0;
			(p_dmydata+cnt)->hour = 0;
			(p_dmydata+cnt)->minute = 0;
			(p_dmydata+cnt)->second = 0;
			(p_dmydata+cnt)->ms = 0;
			(p_dmydata+cnt)->reserve = 0;

		}
		else
		{
			n = 0;
			wtime = (double )(*(p_rdata+cnt)*(double )scale);
			m = (double )modf((double )wtime, &n);

			fd = n ;
			ret = getLeapSecbyTAI(&t_LeapSec[0],  fd, &deltat);
			if(ret < 0)
			{
				break;
			}
			n = 0;
			m = (double )modf((wtime-(double )deltat+leapsec93), &n);

			t_sec = (int)n;
			ret = SecToDate(t_sec, (p_dmydata+cnt));
			if(ret < 0)
			{
				break;
			}

			f = ((double )m*(double )1000.0);
			(p_dmydata+cnt)->ms = (short )floor(f+0.5);
			(p_dmydata+cnt)->tai93sec = (double )wtime;

			checkLeapSecDate(&t_LeapSec[0], (p_dmydata+cnt), wtime);
		}
	}

	if(ret < 0)
	{
		if(mallocflg == IOTK_ON)
		{
			free(*scan_time);
			*scan_time = NULL;
		}
		readcnt = ret;
	}

	H5Dclose(dataset);

	free(p_rdata);

	return readcnt;
}
int amtk_getscantime_(hid_t *HDF_file_id, int *from_scan, int *to_scan, AM2_COMMON_SCANTIME *scan_time)
{
	return AMTK_getScanTime(*HDF_file_id, *from_scan, *to_scan, &scan_time);
}
/* For g77 entry */
int amtk_getscantime__(hid_t *HDF_file_id, int *from_scan, int *to_scan, AM2_COMMON_SCANTIME *scan_time)
{
	return amtk_getscantime_(HDF_file_id, from_scan, to_scan, scan_time);
}

/** @file amtk_scantime.c
 * Set scan time
 *
 * @author     NEC
 * @date       2011/06/13
 */
int AMTK_setScanTime(hid_t HDF_file_id, AM2_COMMON_SCANTIME *scan_time, int from_scan, int to_scan)
{
	int					ret = 0;
	int     			dataset_no = 0;
	int					table_num = 0;
	ACCESS_LABEL		*p_table=NULL;
	int					rank = 0;
	int					size[3]={0,0,0};
	hid_t				dataset = 0;
	double				*p_wdata=NULL;
	int					getmemsize = 0;
	hid_t				memspace = 0;
	hid_t				filespace = 0;
	int					overlap = 0;
	double				deltat = 0.0;
	double				leapsec93 = 0.0;
	int					total_sec = 0;
	int					cnt = 0;
	int					writecnt = 0;
	double 				n = 0.0;
	double 				ans_sec = 0.0;
	float				scale = 0.0;
	LEAPSEC				t_LeapSec[50];

	if(scan_time == NULL)
	{
		return	ERROR_NOT_WRITEDATA;
	}
 	ret = CommonPrmCheck(HDF_file_id, AM2_SCANNUM_NOCHK, AM2_SCANNUM_NOCHK, AM2_SCAN_TIME_DEF);
	if(ret != SUCCESS)
	{
		return ret;
	}

	dataset_no = AM2_SCAN_TIME_DEF;

	p_table = search_table(HDF_file_id, AM2_SCAN_TIME_DEF, &table_num);
	if(p_table == NULL)
	{
		return ERROR_NOT_ACCESSLABEL;
	}

	rank = AMTK_getDimSize(HDF_file_id,  dataset_no, &size[0]);
	if(rank < 0)
	{
		IOTK_printf("getScanTime ERROR - 2[%s]\n",name_table[table_num][p_table->dataset_no]);
		return rank;
	}

	ret = getOverLapScans( HDF_file_id, &overlap);
	if(ret != SUCCESS)
	{
		return ret;
	}

	p_wdata  = (void *)getMemoryAlloc(&size[0], rank, sizeof(double), from_scan, to_scan, &getmemsize);
	if(p_wdata == NULL)
	{
		return	ERROR_MALLOC;
	}

	writecnt = getmemsize/sizeof(double);
	for(cnt = 0; cnt < writecnt; cnt++)
	{
		ret = checkScanTimeData((scan_time+cnt));
		if(ret != SUCCESS)
		{
			free(p_wdata);
			return ret;
		}
	}

	dataset  = H5Dopen (HDF_file_id, name_table[table_num][p_table->dataset_no], H5P_DEFAULT);
	if(dataset < 0)
	{
		free(p_wdata);
		IOTK_printf("getScanTime ERROR - 1[%s]\n",name_table[table_num][p_table->dataset_no]);
		return ERROR_HDF_DATASET_OPEN;
	}

	ret = getScaleFactor(dataset, &scale);
	if(ret != SUCCESS)
	{
		free(p_wdata);
		H5Dclose(dataset);
		return ret;
	}

	ret = setHyperslab(dataset, from_scan, to_scan, &size[0], rank, overlap, &memspace, &filespace);
	if(ret < 0)
	{
		free(p_wdata);
		H5Dclose(dataset);
		return ret;
	}

	memset(&t_LeapSec[0], 0x00, (sizeof(LEAPSEC)*50));
	ret = readLeapSec(&t_LeapSec[0]);
	if(ret < 0)
	{
		free(p_wdata);
		/* close dataspace */
		H5Sclose(memspace);
		H5Sclose(filespace);
		/* close dataset */
		H5Dclose(dataset);
		return ret;
	}

	/* get 93 leap sec */
	ret = getLeapSec(&t_LeapSec[0], 1993, 1, &leapsec93);
	if(ret < 0)
	{
		free(p_wdata);
		/* close dataspace */
		H5Sclose(memspace);
		H5Sclose(filespace);
		/* close dataset */
		H5Dclose(dataset);
		return ret;
	}

	for(cnt = 0; cnt < writecnt; cnt++)
	{
		/* Handling to set "Error scan_time data" of AMSR-E. */
		if(0 == (scan_time+cnt)->year && 0 == (scan_time+cnt)->month && 0 == (scan_time+cnt)->day)
		{
			*(p_wdata+cnt) = AM2_DEF_RMISS;
		}else{
			ret = getLeapSec(&t_LeapSec[0], (scan_time+cnt)->year, (scan_time+cnt)->month, &deltat);
			if(ret < 0)
			{
				free(p_wdata);
				/* close dataspace */
				H5Sclose(memspace);
				H5Sclose(filespace);
				/* close dataset */
				H5Dclose(dataset);
				return ret;
			}
			ret = DateToSec(*(scan_time+cnt), &total_sec);
			if(ret < 0)
			{
				free(p_wdata);
				/* close dataspace */
				H5Sclose(memspace);
				H5Sclose(filespace);
				/* close dataset */
				H5Dclose(dataset);
				return ret;
			}

			ans_sec = (double )((double )total_sec+(double )deltat-leapsec93);

			n = (double )((double )(scan_time+cnt)->ms/(double )1000.0);

			*(p_wdata+cnt) = (double)(((double )ans_sec+n)/scale);
		}
	}

	ret = H5Dwrite(dataset,		    		/* hid_t dset_id        */
				  H5T_NATIVE_DOUBLE,		/* hid_t mem_type_id    */
				  memspace,					/* hid_t mem_space_id   */
				  filespace,				/* hid_t file_space_id  */
				  H5P_DEFAULT,				/* hid_t plist_id       */
				  p_wdata);					/* void *buf   *out*    */
	free(p_wdata);
	
	/* close dataspace */
	H5Sclose(memspace);
	H5Sclose(filespace);
	/* close dataset */
	H5Dclose(dataset);

	if(ret < 0)
	{
		return ERROR_HDF_DATASET_WRITE;
	}

	return SUCCESS;
}
int amtk_setscantime_(hid_t *HDF_file_id, AM2_COMMON_SCANTIME *scan_time, int *from_scan, int *to_scan)
{
	return AMTK_setScanTime(*HDF_file_id, scan_time, *from_scan, *to_scan);
}
/* For g77 entry */
int amtk_setscantime__(hid_t *HDF_file_id, AM2_COMMON_SCANTIME *scan_time, int *from_scan, int *to_scan)
{
	return amtk_setscantime_(HDF_file_id, scan_time, from_scan, to_scan);
}
/*********************************************************************************/

